@@ -164,7 +164,7 @@ def gen_class_h(bdir, data):
 	fname = '%s/include/konoha/gen/konoha_class_.h' % bdir
 	f = open_h(fname, ['<konoha/konoha_t.h>'] + data.INCLUDE_H)
 
-	write_define(f, 'KONOHA_SERIAL_NUMBER', '%d' % data.serial_number)
+	write_define(f, 'KONOHA_BUILDID', '%d' % data.serial_number)
 	write_dline(f)
 	for u in data.USING_MAP:
 		write_ifndefine(f, 'KONOHA_USING_%s' % u.upper(), '1')
@@ -202,14 +202,16 @@ def gen_class_h(bdir, data):
 	for cname in data.ACLASS_LIST:
 		cname = cname.replace(":", "__")
 		write_define(f, 'ACLASS_%s' % cname, '((knh_class_t)(KNH_TSTRUCT_SIZE+%d))' % cn, 32)
-		write_define(f, 'ATYPE_%s' % cname, 'CLASS_TONNTYPE(ACLASS_%s)' % cname, 32)
+		write_define(f, 'ATYPE_%s' % cname, 'ACLASS_%s' % cname, 32)
+		write_define(f, 'NNATYPE_%s' % cname, 'NNTYPE_cid(ACLASS_%s)' % cname, 32)
 		cn += 1
 	
 	write_chapter(f, '[Iterator<X>]')
 	for cname in data.ICLASS_LIST:
 		cname = cname.replace(":", "__")
 		write_define(f, 'ICLASS_%s' % cname, '((knh_class_t)(KNH_TSTRUCT_SIZE+%d))' % cn, 32)
-		write_define(f, 'ITYPE_%s' % cname, 'CLASS_TONNTYPE(ICLASS_%s)' % cname, 32)
+		write_define(f, 'ITYPE_%s' % cname, 'NNTYPE_cid(ICLASS_%s)' % cname, 32)
+		write_define(f, 'NNITYPE_%s' % cname, 'NNTYPE_cid(ICLASS_%s)' % cname, 32)
 		cn += 1
 	
 	write_chapter(f, '[Object:X]')
@@ -217,7 +219,7 @@ def gen_class_h(bdir, data):
 		cname = cname.replace(":", "__")
 		write_define(f, 'CLASS_%s' % cname, '((knh_class_t)(KNH_TSTRUCT_SIZE+%d))' % cn, 32)
 		write_define(f, 'TYPE_%s' % cname, 'CLASS_%s' % cname, 32)
-		write_define(f, 'NNTYPE_%s' % cname, 'CLASS_TONNTYPE(CLASS_%s)' % cname, 32)
+		write_define(f, 'NNTYPE_%s' % cname, 'NNTYPE_cid(CLASS_%s)' % cname, 32)
 		cn += 1
 	
 	write_chapter(f, '[tString]')
@@ -225,7 +227,7 @@ def gen_class_h(bdir, data):
 	for t in data.STEXT_LIST:
 		nm = data.STEXT_MAP[t]
 		write_define(f, 'TSN_%s' % nm, '%d' % cn, 32)
-		write_define(f, 'TS_%s' % nm, 'knh_tString[TSN_%s]' % nm)
+		write_define(f, 'TS_%s' % nm, '((ctx)->tString[TSN_%s])' % nm)
 		cn += 1
 	write_define(f, 'KNH_TSTRING_SIZE', '%d' % cn, 32)
 	
@@ -237,21 +239,26 @@ def gen_class_h(bdir, data):
 	cn = 0
 	for n in data.MF_LIST:
 		write_define(f, n, '%d' % cn, 32)
-		write_define(f, n.replace('MFN_', 'MF_'), 'knh_tMethodField(%s)' % n)
+		write_define(f, n.replace('MFN_', 'MF_'), 'knh_tMethodField(ctx, %s)' % n)
 		cn += 1
 	write_define(f, 'KNH_TMETHODFIELD_SIZE', '%d' % cn, 32)
 	
+	cn = 1
+	for n in data.EXPT_LIST:
+		write_define(f, 'EXPT_%s' % n, '%d' % cn, 32)
+		cn += 1
+	
 	write_chapter(f, '[KNH_CLASS_T]')
 	
 	for c in data.CLASS_LIST:
 		if not c.struct_name.endswith("_t"):
 			op = ''
-			if c.cname == 'Method':   op = '\tf_method fcall_1;\n\tvoid *code_2;\n'
-			if c.cname == 'Mapper':   op = '\tf_mapper fmap_1;\n'
-			if c.cname == 'Iterator': op = '\tf_next fnext_1;\n'
+			if c.cname == 'Method':   op = '\tknh_fmethod fcall_1;\n\tvoid *pc_start;\n'
+			if c.cname == 'Mapper':   op = '\tknh_fmapper fmap_1;\n'
+			if c.cname == 'Iterator': op = '\tknh_fitrnext fnext_1;\n'
 			if c.cname == 'Token':    op = '\tknh_fileid_t fileid;\n\tknh_ushort_t line;\n\tknh_flag_t flag;\n\tknh_token_t  tt;\n'
 			if c.cname == 'Stmt':     op = '\tknh_fileid_t fileid;\n\tknh_ushort_t line;\n\tknh_flag_t flag;\n\tknh_stmt_t  stt;\n'
-			#if c.cname == 'Compiler': op = '\tknh_fileid_t fileid;\n\tknh_ushort_t line;\n\tknh_flag_t flag;\n\tknh_ushort_t dummy;\n'
+			#if c.cname == 'Asm': op = '\tknh_fileid_t fileid;\n\tknh_ushort_t line;\n\tknh_flag_t flag;\n\tknh_ushort_t dummy;\n'
 			f.write('''
 typedef struct knh_%s_t {
 \tknh_hObject_t h;
@@ -333,7 +340,7 @@ METHOD %s_%s(Ctx *ctx, knh_sfp_t *sfp)
 static
 METHOD %s_%s(Ctx *ctx, knh_sfp_t *sfp)
 {
-\t%s_%s(%s, IS_TRUE(sfp[1].o));
+\t%s_%s(%s, p_bool(sfp[1]));
 \tKNH_RETURN_void(ctx, sfp);
 }
 ''' % (methodbase, ffn, funcbase, ffn, a1))
@@ -345,67 +352,13 @@ METHOD %s_%s(Ctx *ctx, knh_sfp_t *sfp)
 static
 METHOD %s_%s(Ctx *ctx, knh_sfp_t *sfp)
 {
-\t%s_%s(%s, IS_TRUE(sfp[1].o));
+\t%s_%s(%s, p_bool(sfp[1]));
 \tKNH_RETURN_void(ctx, sfp);
 }
 ''' % (methodbase, ffn, funcbase, ffn, a1))
 
 def write_ctable_c(f, c, data):
-	if not c.struct_name.startswith("knh_%s_" % c.cname) or '@Field' in c.options: return
-	if not c.struct_name.startswith("knh_%s_struct" % c.cname):
-		return
-		f.write('''
-static INLINE
-knh_struct_t
-KNH_TSTRUCT_$1(Ctx *ctx,
-	knh_struct_t sid,
-	size_t size,
-	char *name,
-	void *finit, void *fcopy, void *fgc,
-	knh_hcode_t (*fhashCode)($1*),
-	int (*fcompareTo)($1*, $1*)) {
-	return KNH_TSTRUCT(ctx, sid, size, name, finit, fcopy, fgc,
-		(f_hashCode)fhashCode, (f_compareTo)fcompareTo);
-}
-'''.replace('$1', c.cname))
-		return
-	
-	f.write('''
-static INLINE knh_struct_t KNH_TSTRUCT_$1(Ctx *ctx,
-	knh_struct_t sid,
-	size_t size,
-	char *name,
-	void (*finit)(Ctx*, knh_$1_struct*, int, Object*),
-	void (*fcopy)(Ctx*, knh_$1_struct*, knh_$1_struct*),
-	void (*fgc)(Ctx*, knh_$1_struct*, f_traverse),
-	knh_hcode_t (*fhashCode)($1*),
-	int (*fcompareTo)($1*, $1*)) {
-	return KNH_TSTRUCT(ctx, sid, size, name, 
-		(f_tStruct_init)finit, (f_tStruct_copy)fcopy, (f_tStruct_traverse)fgc,
-		(f_hashCode)fhashCode, (f_compareTo)fcompareTo);
-}
-'''.replace('$1', c.cname))
-
-def write_struct_c(f, c, data):
-	if not c.struct_name.startswith("knh_%s_" % c.cname) or '@Field' in c.options: return
-	fh = '%s_hashCode' % (c.funcbase)
-	if not data.FUNC_MAP.has_key(fh) : fh = 'NULL'
-	fc = '%s_compareTo' % (c.funcbase)
-	if not data.FUNC_MAP.has_key(fc) : fc = 'NULL'
-	
-	if not c.struct_name.startswith("knh_%s_struct" % c.cname):
-		f.write('''
-\tKNH_TSTRUCT(ctx, %s, 0, "%s", NULL, NULL, NULL, (f_hashCode)%s, (f_compareTo)%s); ''' % (STRUCT_cname(c.cname), c.cname, fh, fc))
-		return
-	f.write('''
-\tKNH_TSTRUCT_%s(ctx, %s, sizeof(%s),
-\t\t"%s",
-\t\t%s_struct_init,
-\t\t%s_struct_copy, 
-//\t\t%s_struct_compare, 
-\t\t%s_struct_traverse,
-\t\t%s,
-\t\t%s);''' % (c.cname, STRUCT_cname(c.cname), c.struct_name, c.cname, c.funcbase, c.funcbase, c.funcbase, c.funcbase, fh, fc))
+	return
 
 def write_class_c(f, c, data):
 	f.write('''
@@ -452,19 +405,7 @@ def CI(t):
 	return t.replace("!", "").replace("..", "__").replace("[]", "__A")
 
 def write_tmapper_c(f, mm, data):
-	if mm.mapfunc == None: return
-	f.write('''
-INLINE static
-void KNH_TMAPPER_%s_%s(
-	Ctx *ctx,
-	knh_flag_t flag,
-	knh_class_t scid,
-	knh_class_t tcid,
-	knh_%s_t* (*fmapfunc)(Ctx *, knh_%s_t*, knh_Mapper_t *),
-	Object *mapopt) {
-	KNH_TMAPPER(ctx, flag, scid, tcid, (f_mapper)fmapfunc, mapopt);
-}
-''' %(CI(mm.fcname), CI(mm.tcname), C(mm.tcname), C(mm.fcname)))
+	return
 
 def CLASS_cname2(cname):
 	cname = cname.replace("!", "")
@@ -477,12 +418,12 @@ def CLASS_cname2(cname):
 def write_mapmap_c(f, mm, data):
 	if mm.mapfunc == None: return
 	f.write('''
-\tKNH_TMAPPER_%s_%s(ctx, 
+\tKNH_TMAPPER(ctx, 
 \t\t%s, 
 \t\t%s, %s,  
 \t\t%s, %s
 \t);
-''' % (CI(mm.fcname), CI(mm.tcname), mm.make_flag(data), CLASS_cname2(mm.fcname), CLASS_cname2(mm.tcname), mm.mapfunc, mm.mapopt))
+''' % (mm.make_flag(data), CLASS_cname2(mm.fcname), CLASS_cname2(mm.tcname), mm.mapfunc, mm.mapopt))
 
 def write_method_func_c(f, mtd, data):
 	if mtd.methodfunc != None: return
@@ -527,14 +468,14 @@ METHOD %s(Ctx *ctx, knh_sfp_t *sfp)
 	ki = 0
 	for p in params[ci:]:
 		n = 'sfp[%d].o' % (ki + 1)
-		if p.type == 'knh_vargc_t':
+		if p.type == 'int':
 			f.write(''',
 \t\tknh_sfp_argc(ctx, sfp + %d),
 \t\tsfp + %d''' % ((ki + 1), (ki + 1)))
 			break
 		if p.type == 'knh_hcode_t':
 			f.write(''',
-\t\tknh_Object_hashCode(%s)''' % n)
+\t\tknh_Object_hashCode(Ctx *ctx, %s)''' % n)
 			continue
 		if ki < len(mtd.params):
 			f.write(''',
@@ -548,8 +489,8 @@ METHOD %s(Ctx *ctx, knh_sfp_t *sfp)
 }
 ''' % (rmap, rval))
 
-def gen_class_c(bdir, p, data):
-	fname = '%s/konoha/gen/package/%s_.c' % (bdir, p.pname)
+def gen_class2_c(bdir, p, data):
+	fname = '%s/src/gen/package/%s_.c' % (bdir, p.pname)
 	f = open_c(fname, ['<konoha/konoha_dev.h>'])
 	write_chapter(f, '[flag]')
 	for c in p.CLASS_LIST:
@@ -585,102 +526,226 @@ void knh_%s_initapis(Ctx *ctx)
 	close_c(f, fname)
 
 def gen_konoha_init_c(bdir, data):
-	fname = '%s/konoha/gen/konoha_init_.c' % (bdir)
+	fname = '%s/src/gen/method_.c' % (bdir)
 	f = open_c(fname, ['<konoha/konoha_dev.h>'])
 	
-	write_chapter(f, '[prototype]')
-	
-	for key in data.STATICLIB_MAP.keys():
-		f.write('''
-int %s(Ctx *ctx);''' % (key))
-	
+	f.write('''
+#ifdef KNH_CC_METHODAPI
+''')
 	write_chapter(f, '[flag]')
-	
 	for c in data.CLASS_LIST:
 		for fb in data.FLAG_LIST:
 			if fb.cname == c.cname: write_flag_c(f, c, fb, data)
 	
 	write_chapter(f, '[method]')
 	for mtd in data.METHOD_LIST: write_method_func_c(f, mtd, data)
-	write_chapter(f, '[tmapper]')
-	for mm in data.MAPPER_LIST: write_tmapper_c(f, mm, data)
-	
-	write_chapter(f, '[INIT]')
 	
 	f.write('''
-void knh_tString_init0(void)
-{
-	DEBUG_ASSERT(knh_tString != NULL);
+#endif/*KNH_CC_METHODAPI*/
 ''')
+	
+	close_c(f, fname)
 
+###################
+
+def write_structdef_h(f, c, data):
+	if not c.struct_name.startswith("knh_%s_" % c.cname) or '@Field' in c.options: return
+	write_chapter(f, c.cname)
+	fmt = '''
+#define _$CF_init NULL
+#define _$CF_copy NULL 
+#define _$CF_traverse NULL
+#define _$CF_compareTo NULL
+#define _$CF_hashCode NULL
+#define _$CF_newClass NULL
+
+'''.replace('$CF', c.funcbase).replace('_Object_', '_ObjectField_')
+	f.write(fmt)
+
+def write_structdata_c(f, c, data):
+	if not c.struct_name.startswith("knh_%s_" % c.cname) or '@Field' in c.options: return
+	size = '0'
+	if c.struct_name.startswith("knh_%s_struct" % c.cname): size = 'sizeof(%s)' % c.struct_name
+	fmt = '''
+\t{
+\t\t"%s", STRUCT_$C, FLAG_$C, %s,
+\t\t(knh_fstruct_init)$CF_init,
+\t\t(knh_fstruct_copy)$CF_copy, 
+\t\t(knh_fstruct_traverse)$CF_traverse,
+\t\t(knh_fstruct_compareTo)$CF_compareTo,
+\t\t(knh_fstruct_hashCode)$CF_hashCode,
+\t\t(knh_fstruct_newClass)$CF_newClass
+\t},'''.replace('$CF', c.funcbase).replace('$C', c.cname).replace('_Object_', '_ObjectField_')
+	f.write(fmt % (c.cname, size))
+
+def write_classdata_c(f, c, data):
+	fmt = '''
+\t{"%s", CLASS_$C, CFLAG_$C, %s, %s, 0, %d, 0, knh_$C_fdefault},'''.replace('$C', c.cname)
+	f.write(fmt % (c.fullname, STRUCT_cname(c.cname), CLASS_cname(c.super_cname), c.method_size))
+	
+
+def write_methoddata_c(f, mtd, data):
+	if mtd.methodfunc == None: return
+	mfindex = mtd.MF().replace('MF_', 'MFN_')
+	fmt = '''
+\t{%s, %s, %s, %s, 0, %s, (void*)%s},'''
+	f.write(fmt % (mtd.methodfunc, mtd.make_flag(data), CLASS_cname(mtd.cname), METHODN_mname(mtd.mn), mfindex, mtd.optfunc))
+
+
+def write_mapmapdata_c(f, mm, data) :
+	if mm.mapfunc == None: return
+	fmt = '''
+\t{%s, %s, %s, %s},'''
+	fmt = fmt % (mm.mapfunc, mm.make_flag(data), CLASS_cname2(mm.fcname), CLASS_cname2(mm.tcname))
+	f.write(fmt)
+
+def gen_struct_hc(bdir, data):
+	fname = '%s/include/konoha/gen/struct_.h' % (bdir)
+	f = open_h(fname, ['<konoha/konoha_t.h>'])
+
+	f.write('''
+static knh_StringData_t StringData[] = {''')
 	for t in data.STEXT_LIST:
 		nm = data.STEXT_MAP[t]
 		f.write('''
-\tKNH_INITv(knh_tString[TSN_%s], new_String__T(NULL, "%s"));''' % (nm, t))
-		
+\t{"%s", TSN_%s}, ''' % (t, nm))
 	f.write('''
-}''')
-	for c in data.CLASS_LIST:
-			write_ctable_c(f, c, data)
+	{NULL}
+};
+''')
 	
 	f.write('''
-void konoha_class_init0(Ctx *ctx)
-{''')
+static char* StructName[] =  {''')
 	for c in data.CLASS_LIST:
-		write_struct_c(f, c, data)
+		if not c.struct_name.startswith("knh_%s_" % c.cname) or '@Field' in c.options: continue
+		f.write('''
+	"%s",''' % c.cname);
+	f.write('''
+};
+''')
+	f.write('''
+static knh_StructData_t StructData[] = {''')
 	for c in data.CLASS_LIST:
-		write_class_c(f, c, data)
+		write_structdata_c(f, c, data)
+	f.write('''
+	{NULL}
+};
+''')
+
+#	for c in data.CLASS_LIST:
+#		f.write('''
+#define knh_%s_fdefault NULL''' % c.cname)
 	
+	f.write('''
+static knh_ClassData_t ClassData[] = {''')
+	for c in data.CLASS_LIST:
+		#if c.struct_name.startswith("knh_%s_" % c.cname) or '@Field' in c.options: continue
+		write_classdata_c(f, c, data)
+	f.write('''
+	{NULL}
+};
+''')
+
+	f.write('''
+static knh_GenericsData_t GenericsData[] = {''')
 	for cname in data.ACLASS_LIST:
 		cname = cname.replace(":", "__")
 		f.write('''
-	KNH_ACLASS(ctx, ACLASS_%s, CLASS_%s);''' % (cname, cname))
+\t{"%s[]", ACLASS_%s, CLASS_Array, CLASS_%s, CLASS_Nue},''' % (cname, cname, cname))
 	
 	for cname in data.ICLASS_LIST:
 		cname = cname.replace(":", "__")
 		f.write('''
-	KNH_ICLASS(ctx, ICLASS_%s, CLASS_%s);''' % (cname, cname))
-	
-	for cname in data.XCLASS_LIST:
-		bcname, tag = cname.split(':')
-		cname = cname.replace(":", "__")
-		func = 'knh_cspec_' + cname
-		if data.FUNC_MAP.has_key(func):
-			f.write('''
-	KNH_XCLASS(ctx, CLASS_%s, CLASS_%s, %s(ctx, CLASS_%s, CLASS_%s, NULL));''' % (cname, bcname, func, bcname, cname))
+\t{"%s..", ICLASS_%s, CLASS_Iterator, CLASS_%s, CLASS_Nue},''' % (cname, cname, cname))
+	f.write('''
+	{NULL}
+};
+''')
 	
 	f.write('''
-#ifdef KONOHA_MONOLITHIC
+static knh_ExptData_t ExptData[] = {''')
+	for cname in data.EXPT_LIST:
+		e = data.EXPT_MAP[cname]
+		f.write('''
+	{"%s", %s, EXPT_%s, %s},''' % (cname, e.make_flag(data), cname, e.parent.replace("!!", "")))
+	f.write('''
+	{NULL, 0}
+};
 ''')
+
 	for key in data.STATICLIB_MAP.keys():
 		f.write('''
-	KNH_TSTATICLIB__T(ctx, "%s", (void*)%s);''' % (key, data.STATICLIB_MAP[key]))
+int %s(Ctx *ctx);''' % (key))
 
+	f.write('''
+static knh_FuncData_t FuncData[] = {''')
+	for key in data.STATICLIB_MAP.keys():
+		if key.endswith("_load"): continue
+		f.write('''
+	{""%s", (void*)%s},''' % (key, data.STATICLIB_MAP[key]))
+	f.write('''
+#ifdef KONOHA_MONOLITHIC''')
+	for key in data.STATICLIB_MAP.keys():
+		if not key.endswith("_load"): continue
+		f.write('''
+	{"%s", (void*)%s},''' % (key, data.STATICLIB_MAP[key]))
 	f.write('''
 #endif/*KONOHA_MONOLITHIC*/
+	{NULL}
+};
 ''')
 
+	f.write('''
+static knh_FieldNameData_t FieldNameData[] = {''')
 	l = data.FIELDN_LIST
 	l.sort()
 	for n in l:
 		f.write('''
-\tKNH_TFIELDN__T(ctx, %s, "%s");''' % (FIELDN_name(n.replace(':','__')), n))
+\t{"%s", %s},''' % (n, FIELDN_name(n.replace(':','__'))))
+	f.write('''
+	{NULL, 0}
+};
+''')
 
 	write_line(f)
-	
+	f.write('''
+static knh_MethodFieldData_t MethodFieldData[] = {''')
 	for n in data.MF_LIST:
-		f.write('''
-\tknh_tMethodField_add(ctx, %s); //%s''' % (data.MF_MAP[n], n))
-	
-	for mtd in data.METHOD_LIST: write_method_c(f, mtd, data)
-	for mm in data.MAPPER_LIST: write_mapmap_c(f, mm, data)
+		func = data.MF_MAP[n].replace("new_MethodField__", "\n\t{%s, " % n)
+		func = func.replace('KNH_FLAG_MF_VARARGS', '0').replace('(', '+')
+		func = func.replace(')', '},').replace('NULL,', '').replace('ctx,', '')
+		f.write(func)
+	f.write('''
+	{-1, 0}
+};
+''')
 
+	f.write('''
+static knh_MethodData_t MethodData[] = {''')
+	for mtd in data.METHOD_LIST: write_methoddata_c(f, mtd, data)
+	f.write('''
+	{NULL, 0}
+};
+''')
+
+	f.write('''
+static knh_MapperData_t MapperData[] = {''')
+	for mm in data.MAPPER_LIST: write_mapmapdata_c(f, mm, data)
+	f.write('''
+	{NULL, 0}
+};
+''')
+
+	f.write('''
+typedef void (*knh_finit_)(Ctx *);
+
+static knh_finit_ InitFuncData[] = {''')
 	for p in data.INITFUNC_LIST:
 		f.write('''
-\t%s(ctx);''' % p)
-	
+\t%s,''' % p)
 	f.write('''
-}
+	NULL
+};
 ''')
-	close_c(f, fname)
+	close_h(f, fname)
 
