@@ -513,9 +513,10 @@ def write_exec(f):
 #define DBG2_OPDUMP(ctx, pc)
 #endif
 
-#ifdef KNH_USING_THREADEDCODE
+static
 void knh_code_thread(Ctx *ctx, knh_code_t *pc, void **codeaddr)
 {
+#ifdef KNH_USING_THREADEDCODE
 	while(1) {
 		knh_codejmp_t *op = (knh_codejmp_t*)pc;
 		KNH_ASSERT_OPCODE(op->opcode);
@@ -536,22 +537,22 @@ void knh_code_thread(Ctx *ctx, knh_code_t *pc, void **codeaddr)
 		}
 		pc += OPSIZE[op->opcode];
 	}
-}
 #endif/*KNH_USING_THREADEDCODE*/
+}
 
 ''')
 
 	f.write('''
 #ifndef KNH_USING_THREADEDCODE
+#define OPJUMP NULL
 METHOD knh_KLRCode_exec(Ctx *ctx, knh_sfp_t *sfp)
-/*knh_code_t* KNH_CC_FASTCALL konohavm(Ctx *ctx, knh_sfp_t *sfp, knh_code_t *pc_start)*/
 {
 	static void **globalMachineToJump = NULL;
-	register knh_code_t *pc = DP((KLRCode*)DP(sfp[-1].mtd)->code)->code;
-	//register knh_code_t *pc = pc_start;
-	
+	//register knh_code_t *pc = DP((KLRCode*)DP(sfp[-1].mtd)->code)->code;
+	register knh_code_t *pc = ctx->ebp[0].pc;
+	DBG2_ASSERT(pc == DP((KLRCode*)DP(sfp[-1].mtd)->code)->code);
 	L_HEAD:;
-	switch(KNH_KCODE(pc)) {
+	switch(((knh_codejmp_t*)pc)->opcode) {
 ''');
 	for kc in KCODE_LIST:
 		LC = ''
@@ -568,7 +569,7 @@ METHOD knh_KLRCode_exec(Ctx *ctx, knh_sfp_t *sfp)
 ''' % (kc.OPCODE, LC, kc.ctype, kc.ctype, getmacro(kc, 'L_HEAD'), kc.SIZE))
 	f.write('''
 	}
-	WARNING(ctx, "unknown opcode=%d", KNH_KCODE(pc));
+	KNH_WARNING(ctx, "unknown opcode=%d", ((knh_codejmp_t*)pc)->opcode);
 	goto L_HEAD;
 }
 #endif/*KNH_USING_THREADEDCODE*/
@@ -586,9 +587,9 @@ METHOD knh_KLRCode_exec(Ctx *ctx, knh_sfp_t *sfp)
 	f.write('''
 	};
 	
-	register knh_code_t *pc =  DP((KLRCode*)DP(sfp[-1].mtd)->code)->code;
-	//register knh_code_t *pc =  pc_start;
-	//goto *(((knh_codejmp_t*)pc)->codeaddr);
+	register knh_code_t *pc = ctx->ebp[0].pc;
+	DBG2_ASSERT(pc == DP((KLRCode*)DP(sfp[-1].mtd)->code)->code);
+	//register knh_code_t *pc =  DP((KLRCode*)DP(sfp[-1].mtd)->code)->code;
 	goto *OPJUMP[KNH_OPCODE(pc)]; /* this is needed to init */
 
 ''')
