@@ -339,9 +339,10 @@ def CLASS_cname(cname) :
 
 def TYPE_cname(t) :
 	prefix = ''
-	if t.endswith("!") : prefix = 'NN'
+	if t.endswith("[]!"): prefix = 'NNA'
+	elif t.endswith("!") : prefix = 'NN'
 	if t.endswith('[]'): prefix = 'A'
-	if t.endswith('..'): prefix = 'I'
+	if t.endswith('..'): prefix = 'NNI'
 	return '%sTYPE_%s' % (prefix, SAFE_cname(t))
 
 def DEBUG_cname(cname):
