@@ -29,7 +29,6 @@
 #define CLASS_KONOHA_H_
 
 #include<konoha/konoha_t.h>
-#include<konoha/konoha_api.h>
 
 /* ************************************************************************ */
 
@@ -59,23 +58,38 @@ extern "C" {
 #define KNH_OBJECT_FIELDSIZE 64
 #endif
 
+typedef struct knh_ObjectField_t {
+	knh_hObject_t h;
+	Object  **fields;
+	size_t  bsize;
+} knh_ObjectField_t ;
+
 /* ------------------------------------------------------------------------ */
 /* @class Any Object knh_Any_t @Cyclic */
 /* @class This Any knh_Any_t @Cyclic  @Private */
+/* @class Any0 Any knh_Any_t @Cyclic  @Private */
 /* @class Any1 Any knh_Any_t @Cyclic  @Private */
 /* @class Any2 Any knh_Any_t @Cyclic  @Private */
+/* @class Any3 Any knh_Any_t @Cyclic  @Private */
 
-typedef void (*f_wrapper_free)(void *);
+struct knh_Glue_t;
+typedef void (*knh_fgluefree)(Ctx *, void *ptr, knh_ftraverse ftr);
+//typedef Object* (*knh_fccopy)(Ctx *ctx, struct knh_Glue_t *ptr);
 
-typedef struct {
+typedef struct knh_Glue_t {
 	knh_hObject_t h;
-	void *wrapped;
-	f_wrapper_free w_free;
-} knh_Any_t ;
+	void *ptr;
+	knh_fgluefree gluefree;
+} knh_Glue_t ;
 
-typedef knh_Any_t knh_This_t;
-typedef knh_Any_t knh_Any1_t;
-typedef knh_Any_t knh_Any2_t;
+#define CLASS_Glue    CLASS_Any
+
+typedef knh_Glue_t knh_Any_t;
+typedef knh_Glue_t knh_This_t;
+typedef knh_Glue_t knh_Any0_t;
+typedef knh_Glue_t knh_Any1_t;
+typedef knh_Glue_t knh_Any2_t;
+typedef knh_Glue_t knh_Any3_t;
 
 /* ------------------------------------------------------------------------ */
 /* @class Nue Object knh_Nue_t @Immutable @Private */
@@ -91,9 +105,18 @@ typedef struct knh_Nue_t {
 /* ------------------------------------------------------------------------ */
 /* @class Boolean Object knh_Boolean_t @Immutable */
 
+typedef struct {
+	union {
+		knh_boolean_t bvalue;
+		knh_integer_t ivalue;
+		knh_float_t   fvalue;
+		knh_uint64_t  data;
+	};
+} knh_nObject_t ;
+
 typedef struct knh_Boolean_t {
 	knh_hObject_t h;
-	knh_bool_t value;
+	knh_nObject_t n;
 } knh_Boolean_t;
 
 /* ------------------------------------------------------------------------ */
@@ -101,6 +124,7 @@ typedef struct knh_Boolean_t {
 
 typedef struct knh_Number_t {
 	knh_hObject_t h;
+	knh_nObject_t n;
 } knh_Number_t;
 
 /* ------------------------------------------------------------------------ */
@@ -109,10 +133,7 @@ typedef struct knh_Number_t {
 
 typedef struct knh_Int_t {
 	knh_hObject_t h;
-	struct {
-		knh_integer_t value;
-		knh_uinteger_t uvalue;
-	};
+	knh_nObject_t n;
 } knh_Int_t;
 
 typedef knh_Int_t knh_IntX_t;
@@ -123,7 +144,7 @@ typedef knh_Int_t knh_IntX_t;
 
 typedef struct knh_Float_t {
 	knh_hObject_t h;
-	knh_float_t value;
+	knh_nObject_t n;
 } knh_Float_t;
 
 typedef knh_Float_t knh_FloatX_t;
@@ -151,6 +172,7 @@ typedef struct knh_Bytes_t {
 	knh_hObject_t h;
 	knh_uchar_t *buf;
 	size_t size;
+	size_t capacity;
 } knh_Bytes_t ;
 
 /* ------------------------------------------------------------------------ */
@@ -183,22 +205,33 @@ typedef struct knh_Array_t {
 /* ------------------------------------------------------------------------ */
 /* @class Iterator Object knh_Iterator_struct @Param1 */
 
-struct knh_Iterator_t;
-typedef Object* (*f_next)(Ctx *, struct knh_Iterator_t *);
+#ifdef _MSC_VER
+typedef void   (KNH_CC_FASTCALL *knh_fitrnext)(Ctx *, knh_sfp_t *, int n);
+#else
+typedef ITRNEXT (*knh_fitrnext)(Ctx *, knh_sfp_t *, int n);
+#endif
 
 typedef struct knh_Iterator {
-	f_next fnext;
+	knh_fitrnext fnext;
 	Object* source;
-	size_t pos;
-	void* ref;
+	size_t  pos;
+	union {
+		void*   ref;
+		knh_code_t *pc; /* @see(Generator) */
+	};
 	size_t count;
 	size_t start;
 	size_t end;
+	Object* prefetch; /* @see(Generator) */
 } knh_Iterator_struct;
 
 /* ------------------------------------------------------------------------ */
 /* @class DictMap Object knh_DictMap_t @Cyclic @Param1 */
+/* @flag DictMap.IgnoreCase DICTM:1 (%s)->h.flag 'is:set:is:*' */
 /* @class DictSet Object knh_DictSet_t @Cyclic @Param1 */
+/* @flag DictSet.IgnoreCase DICTS:1 (%s)->h.flag 'is:set:is:*' */
+
+typedef int (*knh_fbytescmp)(knh_bytes_t, knh_bytes_t);
 
 typedef struct knh_dict_t {
 	struct knh_String_t *key;
@@ -220,6 +253,7 @@ typedef struct knh_DictMap_t {
 		knh_dict_t *_list;  // To avoid stupid casting
 	};
 	size_t size;
+	knh_fbytescmp fcmp;
 } knh_DictMap_t;
 
 /* ------------------------------------------------------------------------ */
@@ -236,6 +270,7 @@ typedef struct knh_DictSet_t {
 		knh_dict_t *_list;  // To avoid stupid casting
 	};
 	size_t size;
+	knh_fbytescmp fcmp;
 } knh_DictSet_t;
 
 /* ------------------------------------------------------------------------ */
@@ -252,7 +287,7 @@ typedef struct knh_HashMap {
 	size_t capacity;
 	size_t size;
 	knh_hashmapentry_t** array;
-	f_compareTo fcompareTo;
+	knh_fstruct_compareTo fcompareTo;
 	size_t table_size;
 	knh_hashmapentry_t*  unused;
 	knh_hashmapentry_t*  table;
@@ -276,7 +311,7 @@ typedef struct knh_HashSet {
 	size_t capacity;
 	size_t size;
 	knh_hashsetentry_t** array;
-	f_compareTo fcompareTo;
+	knh_fstruct_compareTo fcompareTo;
 	size_t table_size;
 	knh_hashsetentry_t*  unused;
 	knh_hashsetentry_t*  table;
@@ -322,27 +357,28 @@ typedef struct knh_LRUHashMap {
 //} knh_Map ;
 
 /* ------------------------------------------------------------------------ */
-/* @class DictIdx Object knh_DictIdx_struct */
+/* @class DictIdx Object knh_DictIdx_t @Private */
 
-typedef struct knh_DictIdx {
-	knh_int_t    offset;
-	knh_Array_t* terms;
-	struct knh_DictSet_t* termsDictSet;
-} knh_DictIdx_struct ;
+typedef struct knh_DictIdx_t {
+	knh_hObject_t h;
+	struct knh_Array_t *terms;
+	struct knh_DictSet_t *termsDictSet;
+	knh_int_t offset;
+} knh_DictIdx_t;
 
 /* ------------------------------------------------------------------------ */
 /* ------------------------------------------------------------------------ */
 /* @class Class Object knh_Class_t */
-/* @flag Class.Release!Debug CF knh_tClass[(%s)].cflag 'is:*:is:*' */
-/* @flag Class.Immutable CF knh_tClass[(%s)].cflag 'is:*:is:*' */
-/* @flag Class.Cyclic CF knh_tClass[(%s)].cflag 'is:set:*:*' */
-/* @flag Class.MetaExtensible CF knh_tClass[(%s)].cflag 'is:*:is:*' */
-/* @flag Class.Private!Public CF knh_tClass[(%s)].cflag 'is:*:is:*' */
-/* @flag Class.Final CF knh_tClass[(%s)].cflag 'is:*:is:*' */
-/* @flag Class.Singleton CF knh_tClass[(%s)].cflag 'is:*:*:*' */
-/* @ flag Class.System CF knh_tClass[(%s)].cflag 'is:*:is:*' */
-/* @flag Class.NullObject CF knh_tClass[(%s)].cflag 'is:*:*:*' */
-/* @flag Class.Interface  CF knh_tClass[(%s)].cflag 'is:*:is:*' */
+/* @flag Class.Release!Debug CF ctx->tClass[(%s)].cflag 'is:*:is:*' */
+/* @flag Class.Immutable CF ctx->tClass[(%s)].cflag 'is:*:is:*' */
+/* @flag Class.Cyclic CF ctx->tClass[(%s)].cflag 'is:set:*:*' */
+/* @flag Class.MetaExtensible CF ctx->tClass[(%s)].cflag 'is:*:is:*' */
+/* @flag Class.Private!Public CF ctx->tClass[(%s)].cflag 'is:*:is:*' */
+/* @flag Class.Final CF ctx->tClass[(%s)].cflag 'is:*:is:*' */
+/* @flag Class.Singleton CF ctx->tClass[(%s)].cflag 'is:*:*:*' */
+/* @ flag Class.System CF ctx->tClass[(%s)].cflag 'is:*:is:*' */
+/* @flag Class.NullObject CF ctx->tClass[(%s)].cflag 'is:*:*:*' */
+/* @flag Class.Interface  CF ctx->tClass[(%s)].cflag 'is:*:is:*' */
 
 typedef struct knh_Class_t {
 	knh_hObject_t h;
@@ -353,12 +389,13 @@ typedef struct knh_Class_t {
 // dummy
 #define KNH_FLAG_CF_PARAM1 0
 #define KNH_FLAG_CF_PARAM2 0
+#define KNH_FLAG_CF_PARAM4 0
 
 /* ------------------------------------------------------------------------ */
-/* @class ClassStruct Object knh_ClassStruct_struct @Private */
+/* @class ClassStruct Object knh_ClassStruct_t @Private */
 /* @flag ClassStruct.Hidden CFF ((ClassStruct)%s)->fields[n].flag 'is:set:*:*' */
 /* @flag ClassStruct.AutoName CFF ((ClassStruct)%s)->fields[n].flag 'is:set:*:*' */
-/* @flag ClassStruct.SafeValue CFF ((ClassStruct)%s)->fields[n].flag 'is:set:*:*' */
+/* @flag ClassStruct.NullAssinged CFF ((ClassStruct)%s)->fields[n].flag 'is:set:*:*' */
 /* @flag ClassStruct.Protected CFF ((ClassStruct)%s)->fields[n].flag 'is:set:*:*' */
 /* @flag ClassStruct.Getter CFF ((ClassStruct)%s)->fields[n].flag 'is:set:*:*' */
 /* @flag ClassStruct.Setter CFF ((ClassStruct)%s)->fields[n].flag 'is:set:*:*' */
@@ -371,26 +408,28 @@ typedef struct knh_cfield_t {
 	Object        *value;
 } knh_cfield_t ;
 
-typedef struct knh_ClassStruct {
-	knh_struct_t sid;
-	knh_ushort_t fsize;
+typedef struct knh_ClassStruct_t {
+	knh_hObject_t h;
 	knh_cfield_t* fields;
+	/* knh_struct_t sid; knh_ushort_t fsize; */
+	size_t fsize;
 	knh_Array_t*  methods;
-} knh_ClassStruct_struct;
+} knh_ClassStruct_t;
 
 /* ------------------------------------------------------------------------ */
-/* @class MethodField Object knh_MethodField_struct @Private */
+/* @class MethodField Object knh_MethodField_t @Private */
 
-typedef struct  knh_mfield_t {
-	knh_type_t   type  ;
-	knh_fieldn_t  fn    ;
+typedef struct knh_mfield_t {
+	knh_type_t    type;
+	knh_fieldn_t  fn;
 } knh_mfield_t ;
 
-typedef struct knh_MethodField {
-	knh_flag_t flag;
-	knh_ushort_t size;
+typedef struct knh_MethodField_t {
+	knh_hObject_t h;
 	knh_mfield_t* params;
-} knh_MethodField_struct;
+	knh_ushort_t size;
+	knh_ushort_t rsize;
+} knh_MethodField_t;
 
 /* ------------------------------------------------------------------------ */
 /* @class Method Object knh_Method_struct */
@@ -400,7 +439,7 @@ typedef struct knh_MethodField {
 /* @flag Method.Static MF DP((Method*)%s)->flag 'is:*:is:*' */
 /* @flag Method.NullBase MF DP((Method*)%s)->flag 'is:*:is:*' */
 /* @flag Method.VarArgs MF DP((Method*)%s)->flag 'is:set:is:*' */
-/* @flag Method.Smart   MF DP((Method*)%s)->flag 'is:*:*:*' */
+/* @flag Method.Generator MF DP((Method*)%s)->flag 'is:set:*:*' */
 /* @flag Method.Aspect MF DP((Method*)%s)->flag 'is:set:is:*' */
 /* @flag Method.ObjectCode MF DP((Method*)%s)->flag 'is:set:*:*' */
 /* @flag Method.Generated MF DP((Method*)%s)->flag 'is:set:*:*' */
@@ -409,8 +448,8 @@ typedef struct knh_Method {
 	knh_flag_t     flag;  knh_class_t    cid;
 	knh_methodn_t  mn;    knh_ushort_t   delta;
 	struct knh_MethodField_t* mf;
-	f_method       func;
-	void*          code;
+	knh_fmethod       fproceed;
+	void*             code;
 } knh_Method_struct;
 
 /* ------------------------------------------------------------------------ */
@@ -432,7 +471,7 @@ typedef struct knh_Mapper {
 	knh_class_t  scid;
 	knh_class_t  tcid;
 	union {
-		f_mapper     fmap;
+		knh_fmapper     fmap;
 		struct knh_Mapper_t *m1;
 	};
 	union {
@@ -451,21 +490,28 @@ typedef struct knh_ClassMap {
 	knh_ushort_t   capacity;
 } knh_ClassMap_struct ;
 
-///* ------------------------------------------------------------------------ */
-///* class Closure Object knh_ */
-//
-//typedef struct knh_Closure {
-//	Object* self;
-//	struct knh_Method_t* method;
-//} knh_Closure;
+/* ------------------------------------------------------------------------ */
+/* @class Closure Object knh_Closure_struct @Param2 */
+/* @flag Closure.SharedStack CCF:1 (%s)->h.flag 'is:set:*:*' */
+/* @flag Closure.CopiedStack CCF:2 (%s)->h.flag 'is:set:*:*' */
+
+typedef struct knh_Closure {
+	Object* base;
+	struct knh_Method_t* mtd;
+	size_t stacksize;
+	knh_sfp_t *stack;
+} knh_Closure_struct ;
 
 /* ------------------------------------------------------------------------ */
-/* @class AffineConv Object knh_AffineConv_struct @Private */
+/* @class AffineConv Object knh_AffineConv_t @Private */
 
-typedef struct knh_AffineConv {
-	knh_float_t scale;
-	knh_float_t shift;
-} knh_AffineConv_struct;
+typedef float knh_affinefloat_t;
+
+typedef struct knh_AffineConv_t {
+	knh_hObject_t h;
+	knh_affinefloat_t scale;
+	knh_affinefloat_t shift;
+} knh_AffineConv_t;
 
 /* ------------------------------------------------------------------------ */
 /* IntSpec, FloatSpec, StringSpec */
@@ -492,36 +538,36 @@ typedef knh_ClassSpec_t* (*f_makespec)(Ctx *ctx, knh_class_t bcid, char *extra);
 /* @flag IntUnit.Unsigned UF DP((IntUnit*)%s)->spec.flag 'is:*:is:*' */
 
 struct  knh_IntUnit_t;
-typedef knh_bool_t (*f_iunit_chk)(struct knh_IntUnit_t *, knh_integer_t v);
-typedef int (*f_iunit_cmp)(struct knh_IntUnit_t *, knh_integer_t v1, knh_integer_t v2);
-typedef void (*f_iunit_fmt)(struct knh_IntUnit_t *, char *buf, size_t bufsiz, knh_integer_t v);
+typedef knh_bool_t (*knh_fenumchk)(struct knh_IntUnit_t *, knh_integer_t v);
+typedef int (*knh_fenumcmp)(struct knh_IntUnit_t *, knh_integer_t v1, knh_integer_t v2);
+typedef void (*knh_fenumfmt)(struct knh_IntUnit_t *, char *buf, size_t bufsiz, knh_integer_t v);
 
 typedef struct knh_IntUnit {
 	knh_hClassSpec_struct spec;
-	f_iunit_chk fchk;
-	f_iunit_cmp fcmp;
-	f_iunit_fmt ffmt;
+	knh_fenumchk fchk;
+	knh_fenumcmp fcmp;
+	knh_fenumfmt ffmt;
 	knh_integer_t min;
 	knh_integer_t max;
 	knh_integer_t step;
-	knh_int_t bfoffset;
-	size_t bfsize;
-	knh_Int_t** bfarray;
+//	knh_int_t bfoffset;
+//	size_t bfsize;
+//	knh_Int_t** bfarray;
 } knh_IntUnit_struct;
 
 /* ------------------------------------------------------------------------ */
 /* @class FloatUnit Object knh_FloatUnit_struct @Private */
 
 struct knh_FloatUnit_t;
-typedef knh_bool_t (*f_funit_chk)(struct knh_FloatUnit_t *, knh_float_t v);
-typedef int (*f_funit_cmp)(struct knh_FloatUnit_t *, knh_float_t v1, knh_float_t v2);
-typedef void (*f_funit_fmt)(struct knh_FloatUnit_t *, char *buf, size_t bufsiz, knh_float_t v);
+typedef knh_bool_t (*knh_funitchk)(struct knh_FloatUnit_t *, knh_float_t v);
+typedef int (*knh_funitcmp)(struct knh_FloatUnit_t *, knh_float_t v1, knh_float_t v2);
+typedef void (*knh_funitfmt)(struct knh_FloatUnit_t *, char *buf, size_t bufsiz, knh_float_t v);
 
 typedef struct knh_FloatUnit {
 	knh_hClassSpec_struct spec;
-	f_funit_chk fchk;
-	f_funit_cmp fcmp;
-	f_funit_fmt ffmt;
+	knh_funitchk fchk;
+	knh_funitcmp fcmp;
+	knh_funitfmt ffmt;
 	knh_float_t min;
 	knh_float_t max;
 	knh_float_t step;
@@ -557,13 +603,13 @@ typedef struct knh_BytesConv {
 /* @class StringUnit Object knh_StringUnit_struct @Private */
 
 struct knh_StringUnit_t;
-typedef knh_String_t *(*f_sunit_new)(Ctx *, knh_class_t cid, knh_bytes_t, struct knh_String_t *);
-typedef int (*f_sunit_cmp)(struct knh_StringUnit_t *, knh_bytes_t, knh_bytes_t);
+typedef knh_String_t *(*knh_fvcabnew)(Ctx *, knh_class_t cid, knh_bytes_t, struct knh_String_t *);
+typedef int (*knh_fvcabcmp)(struct knh_StringUnit_t *, knh_bytes_t, knh_bytes_t);
 
 typedef struct knh_StringUnit {
 	knh_hClassSpec_struct spec;
-	f_sunit_new fnew;
-	f_sunit_cmp fcmp;
+	knh_fvcabnew fnew;
+	knh_fvcabcmp fcmp;
 	f_bconv     fbconv;
 	struct knh_BytesConv_t *bconv;
 	knh_ushort_t bytelen;
@@ -654,7 +700,7 @@ typedef struct {
 typedef struct knh_ResultSet {
 	struct knh_Connection_t *conn;
 	knh_dbcur_t             *dbcur;
-	knh_fdbcurfree           dbcurfree; /* necessary if conn is closed before */
+	knh_fdbcurfree           dbcur_free;  /* necessary if conn is closed before */
 	knh_String_t            *tableName;
 	knh_class_t              tcid;
 	knh_ushort_t             column_size;
@@ -698,7 +744,7 @@ typedef struct knh_Exception {
 
 typedef struct knh_ExceptionHandler {
 	jmp_buf jmpbuf;
-	void *pc;
+	knh_code_t *pc;
 	struct knh_Exception_t* caught;
 } knh_ExceptionHandler_struct;
 
@@ -716,7 +762,7 @@ typedef struct knh_Script_t {
 /* @class NameSpace Object knh_NameSpace_struct @Private */
 
 typedef int (*f_pkginit)(Ctx *ctx);
-typedef void (*f_pkgtraverse)(Ctx *ctx, f_traverse ftr);
+typedef void (*f_pkgtraverse)(Ctx *ctx, knh_ftraverse ftr);
 
 typedef struct knh_pkgmeta_t {
 	const int buildid;
@@ -745,15 +791,18 @@ typedef struct knh_NameSpace {
 } knh_NameSpace_struct;
 
 /* ------------------------------------------------------------------------ */
-/* @class System Object knh_System_struct @Field */
+/* @class System Object knh_System_struct @Singleton */
 
 typedef struct knh_System {
-	struct knh_DictMap_t* props;
+	knh_uint_t sysid;
+	size_t     ctxcount;
+
 	struct knh_NameSpace_t* ns;
-	struct knh_String_t*       enc;
+	struct knh_DictMap_t* props;
 	struct knh_InputStream_t*  in;
 	struct knh_OutputStream_t* out;
 	struct knh_OutputStream_t* err;
+	struct knh_String_t*       enc;
 	struct knh_DictIdx_t *tfileidDictIdx;
 	struct knh_DictIdx_t *tfieldnDictIdx;
 	struct knh_Array_t   *tMethodFields;
@@ -763,23 +812,19 @@ typedef struct knh_System {
 	struct knh_DictSet_t *funcDictSet;
 	struct knh_DictSet_t *drvapiDictSet;
 	struct knh_DictSet_t *makespecDictSet;
-} knh_System_struct;
 
-/* ------------------------------------------------------------------------ */
-/* @class Runtime Object knh_Runtime_struct */
+	struct knh_DictSet_t *texptNameDictSet;
 
-typedef struct knh_Runtime {
-	knh_uint_t runtime_id;
-	size_t     ctxcount;
 	struct knh_HashMap_t* tmetaHashMap;
 	struct knh_DictMap_t* tnameSpaceDictMap;
 	struct knh_DictMap_t* turnAliasDictMap;
-	struct knh_HashMap_t* tmapperHashMap;
-	struct knh_HashMap_t* tmethodHashMap;
-} knh_Runtime_struct;
+	/* lock */
+	knh_mutex_t           mutex;
+} knh_System_struct;
 
 /* ------------------------------------------------------------------------ */
 /* @class Context Object knh_Context_t */
+/* @flag Context.System CTXF ((Context*)%s)->flag 'is:set:*:*' */
 /* @flag Context.Release!Debug CTXF ((Context*)%s)->flag 'is:set:is:set' */
 /* @flag Context.Admin CTXF ((Context*)%s)->flag 'is:set:*:*' */
 /* @flag Context.Strict CTXF ((Context*)%s)->flag 'is:set:*:*' */
@@ -790,17 +835,8 @@ typedef struct knh_Runtime {
 ///* ------------------------------------------------------------------------ */
 ///* class Mutex Object knh_ */
 //
-//typedef struct knh_Mutex {
-//	knh_uint_t magic;
-//	knh_uint_t loccount;
-//	knh_mutex_t mutex;
-//} knh_Mutex;
 
 
-/* ------------------------------------------------------------------------ */
-
-#define KONOHA_TABLE_MAGIC   4219881
-
 /* ------------------------------------------------------------------------ */
 /* ------------------------------------------------------------------------ */
 /* include generated header file */
