@@ -39,7 +39,7 @@
 #else
 #define KONOHA_NAME     "konoha"
 #define KONOHA_MAJOR_VERSION "0.3"
-#define KONOHA_VERSION  "0.3.0.1"
+#define KONOHA_VERSION  "0.3.1"
 #endif
 
 #define KONOHA_URL		"http://konoha.sourceforge.jp/"
@@ -119,6 +119,7 @@
 #endif/*__GNUC__*/
 
 #ifdef _MSC_VER
+//#define KNH_DBGMODE2
 #define KNH_CC_FASTCALL  /* __fastcall */
 #if _MSC_VER == 1500
 	#define KONOHA_CC_VERSION "Microsoft Visual C++ 9.0 (1500)"
@@ -141,7 +142,6 @@
 #define KONOHA_CC_VERSION "UNKNOWN COMPILER"
 #endif
 
-
 #ifndef likely
 #define likely(x)	   x
 #define unlikely(x)	   x
@@ -162,14 +162,20 @@
 
 /* ======================================================================== */
 
+#ifndef L1_CACHE_BYTES
+#define L1_CACHE_BYTES   32
+#endif
+
 #ifdef KNH_FASTMODE
 #define KNH_USING_FASTMALLOC 1
+#define KNH_USING_OBJECT32   (sizeof(void*)*8)
 //#define KNH_USING_WORDKLRC   1
 #endif
 
 /* ======================================================================== */
 /* COMMON */
 
+
 #define KONOHA_TSTRUCT_SIZE                128
 #define KONOHA_TGLOBAL_SIZE                256
 #define KONOHA_TCLASS_SIZE                 1024
@@ -184,7 +190,7 @@
 #define KONOHA_DEFAULT_ARRAY_SIZE           16
 #define KONOHA_STRING_BUFSIZ               128
 #define CLASSNAME_BUFSIZ                   128
-#define FILENAME_BUFSIZ                  512
+#define FILENAME_BUFSIZ                    512
 
 /* object specification */
 
@@ -203,14 +209,6 @@
 
 /* ======================================================================== */
 /* ======================================================================== */
-/* [Thread] */
 
-#ifdef KNH_USING_THREAD
-#define KNH_LOCK(ctx, o)            knh_Object_lock(ctx, o)
-#define KNH_UNLOCK(ctx, o)          knh_Object_unlock(ctx, o)
-#else
-#define KNH_LOCK(ctx, o)
-#define KNH_UNLOCK(ctx, o)
-#endif
 
 #endif /*KONOHA_CONFIG_H_*/
