@@ -13,9 +13,10 @@
 #define KNH_ASSERT(c)    assert(c)
 #define DEBUG_ASSERT(c)  assert(c)
 #define SAFE_(s)         s
-
 #endif/*KNH_FASTMODE*/
 
+#define KNH_ABORT()      abort()
+
 /* ======================================================================== */
 /* [DBGMODE2] */
 
@@ -42,17 +43,10 @@
 #define DBG2_RESIZE(o, p, os, ns) \
 	DBG2_P("RESIZE %s(%p) %d => %d\n\tOLD_BLOCK(%p-%p)", STRUCTN((o->h.bcid)), o, (int)os, (int)ns, p, (p + os)); \
 
-#define DBG2_ASSERT(c, fmt, ...) \
-	if(!(c)) { \
-		fflush(stdout); \
-		fprintf(stderr, "DBG2!![%s:%d/%s]:\n\t", KNH_SAFEFILE(__FILE__), __LINE__, __FUNCTION__); \
-		fprintf(stderr, fmt, ## __VA_ARGS__); \
-		fprintf(stderr, "\n"); \
-		KNH_ASSERT(c); \
-	} \
+#define DBG2_ASSERT(c) KNH_ASSERT(c);
 
-#define KNH_MALLOC(ctx, size) DBG2_malloc(ctx, size)
-#define KNH_FREE(ctx, p, size)   DBG2_free(ctx, p, size)
+#define KNH_MALLOC(ctx, size) DBG2_malloc(ctx, size, (char*)__FUNCTION__)
+#define KNH_FREE(ctx, p, size)   DBG2_free(ctx, p, size, (char*)__FUNCTION__)
 #define DBG2_ABORT()    abort()
 
 #else/*KNH_DBGMODE2*/
@@ -62,7 +56,7 @@
 #define DBG2_DUMP(ctx, o, opt, msg)
 #define TODO2(msg)
 #define DBG2_RESIZE(o, p, os, ns)
-#define DBG2_ASSERT(c, fmt, ...)   KNH_ASSERT(c)
+#define DBG2_ASSERT(c)
 #define DBG2_ABORT()
 
 #ifdef KNH_USING_FASTMALLOC
