@@ -49,8 +49,6 @@ extern "C" {
 #define INLINE             /*__inline__*/
 #endif
 
-#define synchronized
-
 #define PUBLIC
 #define PRIVATE            static
 
@@ -71,7 +69,7 @@ extern "C" {
 /* ------------------------------------------------------------------------ */
 
 typedef int                      knh_bool_t;
-#define knh_boolean_t            knh_bool_t
+typedef int                      knh_boolean_t;
 
 /* ------------------------------------------------------------------------ */
 /* Int, knh_int_t */
@@ -320,10 +318,11 @@ typedef struct knh_hObject_t {
 typedef struct knh_Object_t {
 	knh_hObject_t h;
 	void *ref;
-	void *ref2;
-	void *ref3;
+	void *ref2_unused;
+	void *ref3_unused;
 } knh_Object_t ;
 
+
 #ifdef KNH_OBJECT_MAGIC
 	#ifdef KNH_FASTMODE
 		#define KNH_ASSERT_ISOBJECT(o)
@@ -346,6 +345,7 @@ typedef struct knh_Object_t {
 
 #define KNH_FIELDn(v,n)            ((Script*)(v))->fields[(n)]
 
+
 /* ------------------------------------------------------------------------ */
 /* Context */
 /* ------------------------------------------------------------------------ */
@@ -379,7 +379,7 @@ typedef struct knh_sfp_t {
 		struct knh_String_t *s;
 		struct knh_Bytes_t  *ba;
 		struct knh_Iterator_t *it;
-		struct knh_COBJ_t   *n;
+		struct knh_Glue_t   *n;
 		struct knh_OutputStream_t *w;
 		struct knh_Method_t *mtd;
 		struct knh_Mapper_t *mpr;
@@ -392,7 +392,7 @@ typedef struct knh_sfp_t {
 		knh_float_t   fvalue;
 		knh_code_t    *pc;
 		knh_uint_t     op;
-		knh_uint64_t  data;
+		knh_uint64_t   data;
 	};
 } knh_sfp_t;
 
@@ -431,6 +431,7 @@ typedef struct knh_sfp_t {
 	}\
 
 #define KNH_ITREND(ctx, sfp, n) {\
+		knh_Iterator_close(ctx, sfp[0].it);\
 		KNH_MOV(ctx, sfp[n].o, KNH_VOID);\
 		return; \
 	}\
@@ -443,7 +444,7 @@ typedef struct knh_sfp_t {
 		return; \
 	}\
 
-#define KNH_ITREND_Float(ctx, sfp, n, value) {\
+#define KNH_ITRNEXT_Float(ctx, sfp, n, value) {\
 		sfp[n].fvalue = value;\
 		KNH_MOV(ctx, sfp[n].o, KNH_FLOAT0);\
 		return; \
@@ -473,6 +474,157 @@ typedef struct knh_sfp_t {
 		return;\
 	}\
 
+/* ------------------------------------------------------------------------ */
+/* [Context] */
+/* ------------------------------------------------------------------------ */
+
+typedef const struct knh_Context_t    Ctx;
+
+#define KNH_TOBJECT_SIZE     4096
+#define SIZEOF_TOBJECT       (sizeof(knh_Object_t) * KNH_TOBJECT_SIZE)
+
+#define KNH_FASTMALLOC_SIZE  (sizeof(knh_Object_t) - sizeof(void*))
+#define KNH_FASTMALLOC_BSIZE (KNH_FASTMALLOC_SIZE/sizeof(knh_Object_t*))
+
+/* ------------------------------------------------------------------------ */
+
+#define STRUCT_newid         ((knh_struct_t)-1)
+
+#define SIZEOF_TSTRUCT    (KNH_TSTRUCT_SIZE * sizeof(knh_tStruct_t))
+
+char *konoha_getStructName(Ctx *ctx, knh_struct_t sid);
+#define STRUCTN(sid)   konoha_getStructName(ctx, sid)
+
+#ifndef KNH_TSTRUCT_SIZE
+#define KNH_TSTRUCT_SIZE 256
+#endif
+
+typedef void (*knh_ftraverse)(Ctx *ctx, Object *);
+typedef knh_uint_t                knh_hcode_t;  /* knh_hcode_t */
+#define knh_hcode_join(s1,s2)	   ((knh_hcode_t)s1 << (sizeof(knh_short_t)*8)) + s2;
+
+typedef void        (*knh_fstruct_init)(Ctx *, Object *, int);
+typedef void        (*knh_fstruct_traverse)(Ctx *, Object*, knh_ftraverse);
+typedef int         (*knh_fstruct_compareTo)(Ctx *ctx, Object*, Object*);
+typedef knh_hcode_t (*knh_fstruct_hashCode)(Ctx *ctx, Object *);
+typedef Object*     (*knh_fstruct_copy)(Ctx *, Object *);
+typedef void        (*knh_fstruct_newClass)(Ctx *ctx, knh_class_t cid);
+
+typedef struct {
+	knh_fstruct_traverse   ftraverse;
+	knh_fstruct_compareTo  fcompareTo;
+	knh_fstruct_init       finit;
+	size_t                 size;
+	knh_flag_t             flag;
+	knh_struct_t           sid;
+	knh_fstruct_hashCode   fhashCode;
+	knh_fstruct_copy       fcopy;
+	knh_fstruct_newClass   fnewClass;
+	char                  *name;
+} knh_tStruct_t ;
+
+/* ------------------------------------------------------------------------ */
+
+#ifndef KNH_TCLASS_SIZE
+#define KNH_TCLASS_SIZE 1024
+#endif
+
+#define SIZEOF_TCLASS  (KNH_TCLASS_SIZE * sizeof(knh_tClass_t))
+
+typedef knh_Object_t* (*knh_fdefault)(Ctx *ctx, knh_class_t cid);
+
+typedef struct {
+	knh_flag_t    cflag;   knh_flag_t    oflag;
+	knh_class_t   bcid;    knh_class_t   supcid;
+	knh_class_t   p1;      knh_type_t    p2;
+	knh_ushort_t  offset;  knh_struct_t  sid;
+	knh_ushort_t  size;    knh_ushort_t  bsize;
+	union {
+		knh_short_t   keyidx;
+		knh_type_t    r0;
+	};
+	union {
+		knh_short_t   keyidx2;
+		knh_type_t    p3;
+	};
+	struct knh_String_t       *sname;
+	struct knh_String_t       *lname;
+	struct knh_Class_t        *class_cid;
+	struct knh_ClassStruct_t  *cstruct;
+	struct knh_ClassMap_t     *cmap;
+	struct knh_Object_t       *cspec;
+	knh_fdefault                  fdefault;
+} knh_tClass_t;
+
+/* ------------------------------------------------------------------------ */
+
+#ifndef KNH_TEXPT_SIZE
+#define KNH_TEXPT_SIZE (KNH_TCLASS_SIZE/4)
+#endif
+
+typedef struct {
+	knh_flag_t   flag;
+	knh_expt_t   parent;
+	struct knh_String_t     *name;
+} knh_tExpt_t;
+
+#define SIZEOF_TEXPT  (KNH_TEXPT_SIZE * sizeof(knh_tExpt_t))
+
+/* ------------------------------------------------------------------------ */
+
+#ifndef KNH_TINT_MIN
+#define KNH_TINT_MIN (-1)  /* @property */
+#endif
+
+#ifndef KNH_TINT_MAX
+#ifdef KNH_FASTMODE
+#define KNH_TINT_MAX 1024  /* @property */
+#else
+#define KNH_TINT_MAX 256  /* @property */
+#endif
+#endif
+
+#define SIZEOF_TINT (sizeof(knh_Object_t*) * (KNH_TINT_MAX - KNH_TINT_MIN + 1))
+#define SIZEOF_TSTRING (sizeof(knh_Object_t*) * KNH_TSTRING_SIZE)
+
+#define knh_rootNameSpace     DP(ctx->sys)->ns
+#define knh_systemEncoding    DP(ctx->sys)->enc
+
+#define KNH_NULL            (ctx->constNull)
+#define KNH_VOID            (ctx->constVoid)
+#define KNH_TRUE            (ctx->constTrue)
+#define KNH_FALSE           (ctx->constFalse)
+#define KNH_SYSTEM          (ctx->sys)
+#define KNH_INT0            (ctx->tInt[0 - KNH_TINT_MIN])
+#define KNH_FLOAT0          (ctx->constFloat0)
+
+/* ------------------------------------------------------------------------ */
+
+typedef int knh_mutex_t;
+typedef int (*knh_fmutexlock)(knh_mutex_t*);
+typedef int (*knh_fmutexunlock)(knh_mutex_t*);
+
+#define KNH_LOCK(ctx, lock)  (ctx->share)->flock(lock);
+#define KNH_UNLOCK(ctx, lock)  (ctx->share)->funlock(lock);
+
+/* ------------------------------------------------------------------------ */
+
+typedef struct knh_ContextShare_t {
+	size_t statUsedMemorySize;
+	size_t statUsedObjectSize;
+	size_t maxObjectTableSize;
+	size_t tObjectTableSize;
+	size_t tClassSize;
+	size_t maxContextSize;
+	size_t tContextSize;
+	knh_struct_t tStructSize; knh_expt_t   tExptSize;
+	/* 32bytes used until here*/
+	knh_fmutexlock        flock;
+	knh_fmutexunlock      funlock;
+	knh_mutex_t           objectLock;
+	knh_mutex_t           tableLock;
+} knh_ContextShare_t ;
+
 /* ------------------------------------------------------------------------ */
 
 typedef struct knh_Context_t {
@@ -480,12 +632,37 @@ typedef struct knh_Context_t {
 	knh_Object_t        *unusedObject;
 	size_t               unusedObjectSize;
 
-	struct knh_Runtime_t        *runtime;
-	knh_flag_t                   flag;
-	knh_nsid_t                   nsid;
+	/* stack */
 	knh_sfp_t*                   stack;
 	knh_sfp_t*                   ebp;
 	size_t                       stacksize;
+
+	/* root table */
+	struct knh_Context_t *tContext;
+	struct knh_ContextShare_t   *share;
+
+	/* system table */
+	knh_Object_t **tObjectTable;
+	knh_tStruct_t  *tStruct;
+	knh_tClass_t   *tClass;
+	knh_tExpt_t    *tExpt;
+//	struct knh_DictSet_t  *texptNameDictSet;
+
+	/* system const */
+	knh_Object_t   *constNull;
+	knh_Object_t   *constVoid;
+	knh_Object_t   *constTrue;
+	knh_Object_t   *constFalse;
+	struct knh_Int_t     **tInt;
+	struct knh_Float_t    *constFloat0;
+	struct knh_String_t  **tString;
+	struct knh_System_t   *sys;
+
+	knh_mutex_t           *objectLock;
+	knh_mutex_t           *tableLock;
+
+	knh_flag_t                   flag;
+	knh_ushort_t                 ctxid;
 	char*                        doing;
 	struct knh_String_t*         enc;
 	struct knh_InputStream_t*    in;
@@ -500,82 +677,40 @@ typedef struct knh_Context_t {
 	struct knh_NameSpace_t      *ns;
 	struct knh_DictMap_t        *tsymbolDictMap;
 	struct knh_Asm_t            *cmpr;
+	struct knh_Context_t        *parent;
+
 } knh_Context_t ;
 
-typedef const knh_Context_t    Ctx;
-
-/* ------------------------------------------------------------------------ */
-/* Struct, Class */
 /* ------------------------------------------------------------------------ */
 
-#ifndef KNH_TSTRUCT_SIZE
-#define KNH_TSTRUCT_SIZE 256
-#endif
+#define KONOHA_MAGIC        314159
 
-typedef void (*knh_ftraverse)(Ctx *ctx, Object *);
-
-typedef void        (*f_tStruct_init)(Ctx *, void *, int, Object *);
-typedef void        (*f_tStruct_copy)(Ctx *, void *, void *);
-typedef int         (*f_tStruct_compare)(void *, void *);
-typedef void        (*f_tStruct_traverse)(Ctx *, void *, knh_ftraverse);
-
-typedef knh_uint_t                knh_hcode_t;  /* knh_hcode_t */
-typedef union {
-	knh_hcode_t hcode;
-	struct {
-		knh_ushort_t u1;
-		knh_ushort_t u2;
-	} value2;
-	float fvalue;
-} knh_hcode_u;
-
-#define knh_hcode_join(s1,s2)	   ((knh_hcode_t)s1 << (sizeof(knh_short_t)*8)) + s2;
-
-typedef knh_hcode_t (*f_hashCode)(Object *);
-typedef int         (*f_compareTo)(Object*, Object*);
-
-typedef struct {
-	size_t                 size;
-	char                  *name;
-	f_tStruct_init         finit;
-	f_tStruct_traverse     ftraverse;
-	f_tStruct_copy         fcopy;
-	f_hashCode             fhashCode;
-	f_compareTo            fcompareTo;
-} knh_tStruct_t ;
+typedef struct konoha_t {
+	knh_uint_t  magic;
+	Ctx *ctx;
+} konoha_t ;
 
 /* ------------------------------------------------------------------------ */
 
-typedef int (*f_compare)(knh_Object_t *, knh_Object_t *);
-typedef knh_Object_t* (*f_default)(Ctx *ctx, knh_class_t cid);
+#define KONOHA_CHECK_(konoha) \
+	if(konoha.magic != KONOHA_MAGIC) { \
+		KNH_SAYS("This is not a Konoha Scripting Engine"); \
+		return; \
+	}\
 
-typedef struct {
-	knh_flag_t    cflag;   knh_flag_t    oflag;
-	knh_class_t   bcid;    knh_class_t   supcid;
-	knh_class_t   p1;      knh_type_t    p2;
-	knh_ushort_t  offset;  knh_struct_t  sid;
-	knh_ushort_t  size;    knh_ushort_t  bsize;
-	knh_short_t   keyidx;  knh_short_t   keyidx2;
-	struct knh_String_t       *sname;
-	struct knh_String_t       *lname;
-	struct knh_Class_t        *class_cid;
-	struct knh_ClassStruct_t  *cstruct;
-	struct knh_ClassMap_t     *cmap;
-	struct knh_Object_t       *cspec;
-	f_default                  fdefault;
-} knh_tClass_t;
+#define KONOHA_CHECK(konoha, value) \
+	if(konoha.magic != KONOHA_MAGIC) { \
+		KNH_SAYS("This is not a Konoha Scripting Engine"); \
+		return value; \
+	}\
 
-/* ------------------------------------------------------------------------ */
-
-typedef knh_uint_t                knh_vargc_t;
-typedef knh_uint_t                knh_stackop_t;
+//typedef knh_uint_t                int;
+//typedef knh_uint_t                knh_stackop_t;
 
 /* ------------------------------------------------------------------------ */
 /* KLRCode */
 /* ------------------------------------------------------------------------ */
 
-/* ------------------------------------------------------------------------ */
-
 typedef struct {
 	size_t pos;
 	struct knh_Bytes_t          *ba;
@@ -600,6 +735,7 @@ typedef MAPPER (*knh_fmapper)(Ctx *, knh_sfp_t *);
 
 typedef knh_ushort_t             knh_sline_t;
 
+
 /* ======================================================================== */
 /* driver */
 /* ======================================================================== */
@@ -721,12 +857,6 @@ typedef struct {
 /* mutex */
 /* ======================================================================== */
 
-#ifdef KNH_USING_PTHREAD
-#define KNH_USING_THREAD 1
-#define knh_mutex_t  pthread_mutex_t
-#else
-#define knh_mutex_t  int  /* dummy */
-#endif
 
 /* ------------------------------------------------------------------------ */
 
