@@ -64,8 +64,8 @@ METHOD knh__Array_new__array(Ctx *ctx, knh_sfp_t *sfp)
 {
 	Array *o = (Array*)sfp[0].o;
 	size_t init = IS_NULL(sfp[1].o) ? KNH_ARRAY_INITSIZE : p_size(sfp[1]);
-	knh_class_t p1 = knh_tClass[knh_Object_cid(o)].p1;
-	Object *v = (p1 == CLASS_Any) ? KNH_NULL : knh_tClass_defaultValueNN(ctx, p1);
+	knh_class_t p1 = ctx->tClass[knh_Object_cid(o)].p1;
+	Object *v = (p1 == CLASS_Any) ? KNH_NULL : konoha_getClassDefaultValue(ctx, p1);
 	if(init > 0) {
 		o->list = knh_oarray_malloc(ctx, init, v);
 		o->size = init;
@@ -80,8 +80,8 @@ METHOD knh__Array_new__array2D(Ctx *ctx, knh_sfp_t *sfp)
 {
 	Array *o = (Array*)sfp[0].o;
 	size_t init = p_size(sfp[1]) * p_size(sfp[2]);
-	knh_class_t p1 = knh_tClass[knh_Object_cid(o)].p1;
-	Object *v = (p1 == CLASS_Any) ? KNH_NULL : knh_tClass_defaultValueNN(ctx, p1);
+	knh_class_t p1 = ctx->tClass[knh_Object_cid(o)].p1;
+	Object *v = (p1 == CLASS_Any) ? KNH_NULL : konoha_getClassDefaultValue(ctx, p1);
 	if(init > 0) {
 		o->list = knh_oarray_malloc(ctx, init, v);
 		o->size = init;
@@ -97,8 +97,8 @@ METHOD knh__Array_new__array3D(Ctx *ctx, knh_sfp_t *sfp)
 {
 	Array *o = (Array*)sfp[0].o;
 	size_t init = p_size(sfp[1]) * p_size(sfp[2]) * p_size(sfp[3]);
-	knh_class_t p1 = knh_tClass[knh_Object_cid(o)].p1;
-	Object *v = (p1 == CLASS_Any) ? KNH_NULL : knh_tClass_defaultValueNN(ctx, p1);
+	knh_class_t p1 = ctx->tClass[knh_Object_cid(o)].p1;
+	Object *v = (p1 == CLASS_Any) ? KNH_NULL : konoha_getClassDefaultValue(ctx, p1);
 	if(init > 0) {
 		o->list = knh_oarray_malloc(ctx, init, v);
 		o->size = init;
@@ -115,8 +115,7 @@ METHOD knh__Array_new__init(Ctx *ctx, knh_sfp_t *sfp)
 {
 	Array *o = (Array*)sfp[0].o;
 	knh_sfp_t *v = sfp + 1;
-	knh_vargc_t ac = knh_sfp_argc(ctx, v);
-	size_t i;
+	int i, ac = knh_sfp_argc(ctx, v);
 	for(i = 0; i < ac; i++) {
 		knh_sfp_boxing(ctx, v + i);
 		knh_Array_add(ctx, o, v[i].o);
@@ -153,8 +152,7 @@ METHOD knh__Array_opLshift(Ctx *ctx, knh_sfp_t *sfp)
 	Array *o = (Array*)sfp[0].o;
 	if(!knh_Object_isImmutable(o)) {
 		knh_sfp_t *v = sfp + 1;
-		knh_vargc_t ac = knh_sfp_argc(ctx, v);
-		size_t i;
+		int i, ac = knh_sfp_argc(ctx, v);
 		for(i = 0; i < ac; i++) {
 			knh_sfp_boxing(ctx, v + i);
 			knh_Array_add(ctx, o, v[i].o);
@@ -274,13 +272,13 @@ METHOD knh__Array_opSubset(Ctx *ctx, knh_sfp_t *sfp)
 	if(o->size > 0) {
 		size_t s = IS_NULL(sfp[1].o) ? 0 : knh_array_index(ctx, p_int(sfp[1]), o->size);
 		size_t e = IS_NULL(sfp[2].o) ? (o->size) : knh_array_index(ctx, p_int(sfp[2]), o->size);
-		Array *newo = (Array*)new_Object__init(ctx, o->h.flag, o->h.cid);
+		Array *newo = (Array*)new_Object_init(ctx, o->h.flag, o->h.cid, 0);
 		newo->list = knh_oarray_copy(ctx, o->list, s, e - s);
 		newo->size = e - s;
 		KNH_RETURN(ctx, sfp, newo);
 	}
 	else {
-		KNH_RETURN(ctx, sfp, new_Object__init(ctx, o->h.flag, o->h.cid));
+		KNH_RETURN(ctx, sfp, new_Object_init(ctx, o->h.flag, o->h.cid, 0));
 	}
 }
 
@@ -293,13 +291,13 @@ METHOD knh__Array_opSubsete(Ctx *ctx, knh_sfp_t *sfp)
 	if(o->size > 0) {
 		size_t s = IS_NULL(sfp[1].o) ? 0 : knh_array_index(ctx, p_int(sfp[1]), o->size);
 		size_t e = IS_NULL(sfp[2].o) ? (o->size) : knh_array_index(ctx, p_int(sfp[2]), o->size) + 1;
-		Array *newo = (Array*)new_Object__init(ctx, o->h.flag, o->h.cid);
+		Array *newo = (Array*)new_Object_init(ctx, o->h.flag, o->h.cid, 0);
 		newo->list = knh_oarray_copy(ctx, o->list, s, e - s);
 		newo->size = e - s;
 		KNH_RETURN(ctx, sfp, newo);
 	}
 	else {
-		KNH_RETURN(ctx, sfp, new_Object__init(ctx, o->h.flag, o->h.cid));
+		KNH_RETURN(ctx, sfp, new_Object_init(ctx, o->h.flag, o->h.cid, 0));
 	}
 }
 
@@ -312,7 +310,7 @@ METHOD knh__Array_opOffset(Ctx *ctx, knh_sfp_t *sfp)
 	if(o->size > 0) {
 		size_t s = IS_NULL(sfp[1].o) ? 0 : knh_array_index(ctx, p_int(sfp[1]), o->size);
 		knh_int_t offset = IS_NULL(sfp[2].o) ? 0 : p_int(sfp[2]);
-		Array *newo = (Array*)new_Object__init(ctx, o->h.flag, o->h.cid);
+		Array *newo = (Array*)new_Object_init(ctx, o->h.flag, o->h.cid, 0);
 		if(offset == 0) {
 
 		}
@@ -329,7 +327,7 @@ METHOD knh__Array_opOffset(Ctx *ctx, knh_sfp_t *sfp)
 		KNH_RETURN(ctx, sfp, newo);
 	}
 	else {
-		KNH_RETURN(ctx, sfp, new_Object__init(ctx, o->h.flag, o->h.cid));
+		KNH_RETURN(ctx, sfp, new_Object_init(ctx, o->h.flag, o->h.cid, 0));
 	}
 }
 
@@ -409,7 +407,7 @@ METHOD knh__Array_indexOf(Ctx *ctx, knh_sfp_t *sfp)
 	knh_sfp_boxing(ctx, sfp + 1);
 	size_t i;
 	for(i = 0; i < o->size; i++) {
-		if(knh_Object_compareTo(o->list[i], sfp[1].o) == 0) {
+		if(knh_Object_compareTo(ctx, o->list[i], sfp[1].o) == 0) {
 			KNH_RETURN_Int(ctx, sfp, i);
 		}
 	}
@@ -425,7 +423,7 @@ METHOD knh__Array_lastIndexOf(Ctx *ctx, knh_sfp_t *sfp)
 	knh_sfp_boxing(ctx, sfp + 1);
 	size_t i;
 	for(i = o->size - 1; i >= 0; i--) {
-		if(knh_Object_compareTo(o->list[i], sfp[1].o) == 0) {
+		if(knh_Object_compareTo(ctx, o->list[i], sfp[1].o) == 0) {
 			KNH_RETURN_Int(ctx, sfp, i);
 		}
 	}
@@ -441,7 +439,7 @@ METHOD knh__Array_opHas(Ctx *ctx, knh_sfp_t *sfp)
 	knh_sfp_boxing(ctx, sfp + 1);
 	size_t i;
 	for(i = 0; i < o->size; i++) {
-		if(knh_Object_compareTo(o->list[i], sfp[1].o) == 0) {
+		if(knh_Object_compareTo(ctx, o->list[i], sfp[1].o) == 0) {
 			KNH_RETURN_Boolean(ctx, sfp, 1);
 		}
 	}
@@ -542,12 +540,12 @@ void knh_Array__k(Ctx *ctx, Array *o, OutputStream *w, String *m)
 static
 ITRNEXT knh_Array_var_next(Ctx *ctx, knh_sfp_t *sfp, int n)
 {
-	Array *o = (Array*)knh_Iterator_source(sfp[0].it);
+	Array *o = (Array*)DP(sfp[0].it)->source;
 	KNH_ASSERT(IS_bArray(o));
-	size_t pos = knh_Iterator_pos(sfp[0].it);
+	size_t pos = DP(sfp[0].it)->pos;
 	while(pos < o->size) {
 		if(IS_NOTNULL(o->list[pos])) {
-			knh_Iterator_setpos(sfp[0].it, pos+1);
+			DP(sfp[0].it)->pos = pos+1;
 			KNH_ITRNEXT(ctx, sfp, n, o->list[pos]);
 		}
 		pos++;
@@ -561,7 +559,7 @@ ITRNEXT knh_Array_var_next(Ctx *ctx, knh_sfp_t *sfp, int n)
 MAPPER knh_Array_Iterator(Ctx *ctx, knh_sfp_t *sfp)
 {
 	KNH_MAPPED(ctx, sfp,
-		new_Iterator(ctx, knh_tClass[(sfp[0].o)->h.cid].p1, sfp[0].o, knh_Array_var_next));
+		new_Iterator(ctx, ctx->tClass[(sfp[0].o)->h.cid].p1, sfp[0].o, knh_Array_var_next));
 }
 
 /* ------------------------------------------------------------------------ */
@@ -570,7 +568,7 @@ MAPPER knh_Array_Iterator(Ctx *ctx, knh_sfp_t *sfp)
 METHOD knh__Array_opItr(Ctx *ctx, knh_sfp_t *sfp)
 {
 	Array *o = (Array*)sfp[0].o;
-	KNH_RETURN(ctx, sfp, new_Iterator(ctx, knh_tClass[o->h.cid].p1, UP(o), knh_Array_var_next));
+	KNH_RETURN(ctx, sfp, new_Iterator(ctx, ctx->tClass[o->h.cid].p1, UP(o), knh_Array_var_next));
 }
 
 /* ------------------------------------------------------------------------ */
@@ -579,7 +577,7 @@ METHOD knh__Array_opItr(Ctx *ctx, knh_sfp_t *sfp)
 MAPPER knh_Iterator_Array(Ctx *ctx, knh_sfp_t *sfp)
 {
 	Iterator *it = sfp[0].it;
-	Array *a = new_Array(ctx, knh_tClass[it->h.cid].p1, 0);
+	Array *a = new_Array(ctx, ctx->tClass[it->h.cid].p1, 0);
 	it->fnext_1(ctx, sfp, 1);
 	while(HAS_ITRNEXT(sfp[1].o)) {
 		knh_Array_add(ctx, a, sfp[1].o);
@@ -593,9 +591,9 @@ MAPPER knh_Iterator_Array(Ctx *ctx, knh_sfp_t *sfp)
 Mapper *knh_tMapper_newIteratorArray(Ctx *ctx, knh_class_t icid, knh_class_t acid)
 {
 	Mapper *mpr = new_Mapper(ctx, KNH_FLAG_MMF_TOTAL, acid, icid, knh_Array_Iterator, KNH_NULL);
-	knh_ClassMap_add(ctx, knh_tClass[acid].cmap, mpr);
+	knh_ClassMap_add(ctx, ctx->tClass[acid].cmap, mpr);
 	mpr = new_Mapper(ctx, KNH_FLAG_MMF_TOTAL, icid, acid, knh_Iterator_Array, KNH_NULL);
-	knh_ClassMap_add(ctx, knh_tClass[icid].cmap, mpr);
+	knh_ClassMap_add(ctx, ctx->tClass[icid].cmap, mpr);
 	return mpr;
 }
 
@@ -604,9 +602,9 @@ Mapper *knh_tMapper_newIteratorArray(Ctx *ctx, knh_class_t icid, knh_class_t aci
 Mapper *knh_tMapper_newArrayIterator(Ctx *ctx, knh_class_t acid, knh_class_t icid)
 {
 	Mapper *mpr = new_Mapper(ctx, KNH_FLAG_MMF_TOTAL, icid, acid, knh_Iterator_Array, KNH_NULL);
-	knh_ClassMap_add(ctx, knh_tClass[icid].cmap, mpr);
+	knh_ClassMap_add(ctx, ctx->tClass[icid].cmap, mpr);
 	mpr = new_Mapper(ctx, KNH_FLAG_MMF_TOTAL, acid, icid, knh_Array_Iterator, KNH_NULL);
-	knh_ClassMap_add(ctx, knh_tClass[acid].cmap, mpr);
+	knh_ClassMap_add(ctx, ctx->tClass[acid].cmap, mpr);
 	return mpr;
 }
 
