@@ -38,6 +38,8 @@ extern "C" {
 /* ======================================================================== */
 /* [constructor] */
 
+void knh_KLRCode__dumpmtd(Ctx *ctx, KLRCode* o, OutputStream *w, Method *mtd);
+
 /* ------------------------------------------------------------------------ */
 /* @method[VARARGS] This! Object.new:init(Any value) @VARARGS */
 
@@ -46,16 +48,16 @@ METHOD knh__Object_new__init(Ctx *ctx, knh_sfp_t *sfp)
 	Object *o = sfp[0].o;
 //	knh_class_t cid = knh_Object_cid(o);
 //	knh_sfp_t *v = sfp + 1;
-//	knh_vargc_t ac = knh_sfp_argc(ctx, v);
+//	int ac = knh_sfp_argc(ctx, v);
 //	size_t i;
 //	for(i = 0; i < ac; i+= 2) {
 //		if(IS_bString(v[i].s)) {
 			TODO();
 //			knh_fieldn_t fn = knh_tName_get_fnq(ctx, knh_String_tobytes(v[i].s), FIELDN_NONAME);
 //			if(fn == FIELDN_NONAME) continue;
-//			knh_index_t idx = knh_Class_queryField(cid, fn);
+//			knh_index_t idx = knh_Class_queryField(ctx, cid, fn);
 //			if(idx == -1) continue;
-//			knh_cfield_t *cf = knh_Class_fieldAt(cid, idx);
+//			knh_cfield_t *cf = knh_Class_fieldAt(ctx, cid, idx);
 //			knh_type_t type = knh_pmztype_totype(ctx, cf->type, cid);
 //			DBG2_P("[%d] %s%s %s", (int)idx, TYPEQN(type), FIELDN(fn));
 //			KNH_SETv(ctx, KNH_FIELDn(o, idx), knh_Object_opAs__type(ctx, v[i+1].o, type, KNH_FIELDn(o,idx)));
@@ -71,26 +73,26 @@ METHOD knh__Object_new__init(Ctx *ctx, knh_sfp_t *sfp)
 /* @method Int! Object.compareTo(Any o) */
 
 INLINE
-int knh_Object_compareTo(Object *o1, Object *o2)
+int knh_Object_compareTo(Ctx *ctx, Object *o1, Object *o2)
 {
 	knh_class_t bcid = o1->h.bcid;
 	knh_class_t bcid2 = o2->h.bcid;
 	if(bcid == bcid2) {
 		if(bcid == CLASS_Object) {
-			int idx = knh_tClass[o1->h.cid].keyidx;
+			int idx = ctx->tClass[o1->h.cid].keyidx;
 			if(idx != -1) {
-				int res = knh_Object_compareTo(KNH_FIELDn(o1, idx), KNH_FIELDn(o2, idx));
+				int res = knh_Object_compareTo(ctx, KNH_FIELDn(o1, idx), KNH_FIELDn(o2, idx));
 				if(res == 0) return res;
-				idx = knh_tClass[o1->h.cid].keyidx2;
+				idx = ctx->tClass[o1->h.cid].keyidx2;
 				if(idx != -1) {
-					return knh_Object_compareTo(KNH_FIELDn(o1, idx), KNH_FIELDn(o2, idx));
+					return knh_Object_compareTo(ctx, KNH_FIELDn(o1, idx), KNH_FIELDn(o2, idx));
 				}
 				return res;
 			}
 			return (knh_int_t)o1 - (knh_int_t)o2;
 		}
 		else {
-			return knh_tStruct[bcid].fcompareTo(o1, o2);
+			return ctx->tStruct[bcid].fcompareTo(ctx, o1, o2);
 		}
 	}
 	else {
@@ -106,9 +108,9 @@ int knh_Object_compareTo(Object *o1, Object *o2)
 /* ------------------------------------------------------------------------ */
 
 INLINE
-knh_bool_t knh_Object_equals(Object *o1, Object *o2)
+knh_bool_t knh_Object_equals(Ctx *ctx, Object *o1, Object *o2)
 {
-	return (knh_Object_compareTo( o1, o2) == 0);
+	return (knh_Object_compareTo(ctx, o1, o2) == 0);
 }
 
 
@@ -119,7 +121,7 @@ knh_bool_t knh_Object_equals(Object *o1, Object *o2)
 
 METHOD knh__Object_opEq(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_RETURN_Boolean(ctx, sfp, knh_Object_compareTo( sfp[0].o, sfp[1].o) == 0);
+	KNH_RETURN_Boolean(ctx, sfp, knh_Object_compareTo(ctx,  sfp[0].o, sfp[1].o) == 0);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -127,7 +129,7 @@ METHOD knh__Object_opEq(Ctx *ctx, knh_sfp_t *sfp)
 
 METHOD knh__Object_opNeq(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_RETURN_Boolean(ctx, sfp, knh_Object_compareTo( sfp[0].o, sfp[1].o) != 0);
+	KNH_RETURN_Boolean(ctx, sfp, knh_Object_compareTo(ctx,  sfp[0].o, sfp[1].o) != 0);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -135,7 +137,7 @@ METHOD knh__Object_opNeq(Ctx *ctx, knh_sfp_t *sfp)
 
 METHOD knh__Object_opLt(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_RETURN_Boolean(ctx, sfp, knh_Object_compareTo(sfp[0].o, sfp[1].o) < 0);
+	KNH_RETURN_Boolean(ctx, sfp, knh_Object_compareTo(ctx, sfp[0].o, sfp[1].o) < 0);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -143,7 +145,7 @@ METHOD knh__Object_opLt(Ctx *ctx, knh_sfp_t *sfp)
 
 METHOD knh__Object_opLte(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_RETURN_Boolean(ctx, sfp, knh_Object_compareTo(sfp[0].o, sfp[1].o) <= 0);
+	KNH_RETURN_Boolean(ctx, sfp, knh_Object_compareTo(ctx, sfp[0].o, sfp[1].o) <= 0);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -151,7 +153,7 @@ METHOD knh__Object_opLte(Ctx *ctx, knh_sfp_t *sfp)
 
 METHOD knh__Object_opGt(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_RETURN_Boolean(ctx, sfp, knh_Object_compareTo(sfp[0].o, sfp[1].o) > 0);
+	KNH_RETURN_Boolean(ctx, sfp, knh_Object_compareTo(ctx, sfp[0].o, sfp[1].o) > 0);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -159,7 +161,7 @@ METHOD knh__Object_opGt(Ctx *ctx, knh_sfp_t *sfp)
 
 METHOD knh__Object_opGte(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_RETURN_Boolean(ctx, sfp, knh_Object_compareTo(sfp[0].o, sfp[1].o) >= 0);
+	KNH_RETURN_Boolean(ctx, sfp, knh_Object_compareTo(ctx, sfp[0].o, sfp[1].o) >= 0);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -223,7 +225,7 @@ METHOD knh__Object_opTo(Ctx *ctx, knh_sfp_t *sfp)
 
 METHOD knh__Object_getClass(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_RETURN(ctx, sfp, knh_tClass[(sfp[0].o)->h.cid].class_cid);
+	KNH_RETURN(ctx, sfp, ctx->tClass[(sfp[0].o)->h.cid].class_cid);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -287,14 +289,14 @@ void knh_Object__s(Ctx *ctx, Object *b, OutputStream *w, String *m)
 void knh_Object__k(Ctx *ctx, Object *o, OutputStream *w, String *m)
 {
 	if(o->h.bcid == CLASS_Object) {
-		size_t bsize = knh_tClass[o->h.cid].bsize;
+		size_t bsize = ctx->tClass[o->h.cid].bsize;
 		knh_write__s(ctx, w, CLASSN(o->h.cid));
 		if(bsize > 0) {
 			size_t i;
 			Object **v = (Object**)o->ref;
 			knh_putc(ctx, w, '{');
 			for(i = 0; i < bsize; i++) {
-				knh_cfield_t *cf = knh_Class_fieldAt(knh_Object_cid(o), i);
+				knh_cfield_t *cf = knh_Class_fieldAt(ctx, knh_Object_cid(o), i);
 				if(i > 0) {
 					knh_write_delim(ctx, w);
 				}
@@ -322,8 +324,8 @@ void knh_Object__dump(Ctx *ctx, Object *b, OutputStream *w, String *m)
 	}
 	knh_int_t i, c = 0;
 	knh_putc(ctx, w, '[');
-	for(i = 0; i < knh_tClass[cid].bsize; i++) {
-		knh_cfield_t *cf = knh_Class_fieldAt(cid, i);
+	for(i = 0; i < ctx->tClass[cid].bsize; i++) {
+		knh_cfield_t *cf = knh_Class_fieldAt(ctx, cid, i);
 		if(cf->fn == FIELDN_NONAME || KNH_FLAG_IS(cf->flag, KNH_FLAG_CFF_HIDDEN)) continue;
 		if(c > 0) {
 			knh_write_delim(ctx, w);
@@ -402,10 +404,6 @@ void knh_Nue__k(Ctx *ctx, Nue *b, OutputStream *w, String *m)
 /* ======================================================================== */
 /* [Class] */
 
-/* ------------------------------------------------------------------------ */
-/* ======================================================================== */
-/* [method] */
-
 /* ======================================================================== */
 /* [movabletext] */
 
@@ -508,14 +506,14 @@ void knh_Class_NAME__man(Ctx *ctx, knh_class_t cid, OutputStream *w)
 	knh_write_EOL(ctx, w);
 
 	knh_write_TAB(ctx, w);
-	knh_write(ctx, w, knh_String_tobytes(knh_tClass[cid].lname));
+	knh_write(ctx, w, knh_String_tobytes(ctx->tClass[cid].lname));
 	knh_write_EOL(ctx, w);
 
-	while(knh_tClass[cid].supcid != CLASS_Object) {
-		cid = knh_tClass[cid].supcid;
+	while(ctx->tClass[cid].supcid != CLASS_Object) {
+		cid = ctx->tClass[cid].supcid;
 		knh_write_TAB(ctx, w);
 		knh_write(ctx, w, STEXT("extends "));
-		knh_write(ctx, w, knh_String_tobytes(knh_tClass[cid].lname));
+		knh_write(ctx, w, knh_String_tobytes(ctx->tClass[cid].lname));
 		knh_write_EOL(ctx, w);
 	}
 }
@@ -530,10 +528,10 @@ void knh_Method__man(Ctx *ctx, Method *o, OutputStream *w, knh_class_t cid)
 		knh_putc(ctx, w, ' ');
 	}
 
-	if(knh_Method_rtype(o) == TYPE_void) {
+	if(knh_Method_rztype(o) == TYPE_void) {
 		knh_write(ctx, w, knh_String_tobytes(TS_void));
 	}else{
-		knh_write__type(ctx, w, knh_pmztype_totype(ctx, knh_Method_rtype(o), cid));
+		knh_write__type(ctx, w, knh_pmztype_totype(ctx, knh_Method_rztype(o), cid));
 	}
 	knh_putc(ctx, w, ' ');
 
@@ -549,7 +547,7 @@ void knh_Method__man(Ctx *ctx, Method *o, OutputStream *w, knh_class_t cid)
 		if(i > 0) {
 			knh_write_delim(ctx, w);
 		}
-		knh_mfield_t mf = knh_Method_pfields(o, i);
+		knh_mfield_t mf = knh_Method_pfield(o, i);
 		knh_write__type(ctx, w, knh_pmztype_totype(ctx, mf.type, cid));
 		knh_putc(ctx, w, ' ');
 		knh_write(ctx, w, B(FIELDN(mf.fn)));
@@ -577,27 +575,27 @@ void knh_Class__man(Ctx *ctx, Class *o, OutputStream *w, String *m)
 	KNH_LPUSH(ctx, dm);
 	size_t i = 0;
 	while(1) {
-		Array *a = DP(knh_tClass[cid].cstruct)->methods;
+		Array *a = ctx->tClass[cid].cstruct->methods;
 		for(i = 0; i < knh_Array_size(a); i++) {
 			Method *mtd = (Method*)knh_Array_n(a, i);
 			char *op = knh_methodop_tochar(DP(mtd)->mn);
 			if(op == NULL) {
-				knh_format_methodn(bufmn, sizeof(bufmn), DP(mtd)->mn);
+				knh_format_methodn(ctx, bufmn, sizeof(bufmn), DP(mtd)->mn);
 				//DBG2_P("mn='%s'", bufmn);
 				knh_bytes_t name = B(bufmn);
-				if(IS_NULL(knh_DictMap_get__b( dm, name))) {
+				if(IS_NULL(knh_DictMap_get__b(ctx,  dm, name))) {
 					knh_DictMap_set(ctx, dm, new_String(ctx, name, NULL), UP(mtd));
 				}
 			}
 			else {
 				knh_bytes_t name = B(op);
-				if(IS_NULL(knh_DictMap_get__b( dm, name))) {
+				if(IS_NULL(knh_DictMap_get__b(ctx,  dm, name))) {
 					knh_DictMap_set(ctx, dm, new_String__T(ctx, op), UP(mtd));
 				}
 			}
 		}
 		if(cid == CLASS_Object) break;
-		cid = knh_tClass[cid].supcid;
+		cid = ctx->tClass[cid].supcid;
 	}
 
 	int cnt = 0;
@@ -643,7 +641,7 @@ void knh_Class__man(Ctx *ctx, Class *o, OutputStream *w, String *m)
 		if(IS_Method(mtd)) {
 			if(METHODN_IS_MOVTEXT(DP(mtd)->mn)) continue;
 			if(DP(mtd)->cid == CLASS_Object && cid != CLASS_Object) continue;
-//			if(DP(mtd)->cid != knh_tClass[cid].bcid) continue;
+//			if(DP(mtd)->cid != ctx->tClass[cid].bcid) continue;
 			if(hasCaption == 0) {
 				knh_write__s(ctx, w, knh_message_text(KMSG_METHOD));
 				knh_write_EOL(ctx, w);
@@ -681,10 +679,76 @@ void knh_Class__man(Ctx *ctx, Class *o, OutputStream *w, String *m)
 	}
 	knh_write_EOL(ctx, w);
 
-	knh_ClassMap__man(ctx, knh_tClass[cid].cmap, w, cid);
+	knh_ClassMap__man(ctx, ctx->tClass[cid].cmap, w, cid);
 	KNH_LOCALBACK(ctx, lsfp);
 }
 
+/* ======================================================================== */
+/* [Method] */
+
+/* ------------------------------------------------------------------------ */
+/* @method void Method.%s(OutputStream w, String m) */
+
+void knh_Method__s(Ctx *ctx, Method *o, OutputStream *w, String *m)
+{
+	knh_write__s(ctx, w, CTXCLASSN(DP(o)->cid));
+	knh_putc(ctx, w, '.');
+	knh_write__mn(ctx, w, DP(o)->mn);
+}
+
+/* ------------------------------------------------------------------------ */
+/* @method void Method.%k(OutputStream w, String m) */
+
+void knh_Method__k(Ctx *ctx, Method *o, OutputStream *w, String *m)
+{
+	if(knh_Method_isAbstract(o)) {
+		knh_write(ctx, w, STEXT("@abstract"));
+		knh_putc(ctx, w, ' ');
+	}
+
+	if(knh_Method_rztype(o) == TYPE_void) {
+		knh_write(ctx, w, knh_String_tobytes(TS_void));
+	}else{
+//		knh_write__type(ctx, w, knh_pmztype_totype(ctx, knh_Method_rtype(o), DP(o)->cid));
+		knh_write__type(ctx, w, knh_Method_rztype(o));
+	}
+	knh_putc(ctx, w, ' ');
+	knh_Method__s(ctx, o, w, (String*)KNH_NULL);
+	knh_putc(ctx, w, '(');
+	size_t i;
+	for(i = 0; i < knh_Method_psize(o); i++) {
+		if(i > 0) {
+			knh_write_delim(ctx, w);
+		}
+		knh_mfield_t mf = knh_Method_pfield(o, i);
+		//knh_write__type(ctx, w, knh_pmztype_totype(ctx, mf.type, DP(o)->cid));
+		knh_write__type(ctx, w, mf.type);
+		knh_putc(ctx, w, ' ');
+		knh_write(ctx, w, B(FIELDN(mf.fn)));
+	}
+	if(knh_Method_isVarArgs(o)) {
+		knh_write_delim(ctx, w);
+		knh_write_dots(ctx, w);
+	}
+	knh_putc(ctx, w, ')');
+}
+
+/* ------------------------------------------------------------------------ */
+/* @method void Method.%dump(OutputStream w, String m) */
+
+
+void knh_Method__dump(Ctx *ctx, Method *o, OutputStream *w, String *m)
+{
+	knh_Method__k(ctx, o, w, m);
+	knh_println(ctx, w, STEXT(""));
+	if(knh_Method_isObjectCode(o)) {
+		if(IS_KLRCode((Object*)DP(o)->code)) {
+			knh_KLRCode__dumpmtd(ctx, (KLRCode*)DP(o)->code, w, o);
+		}
+	}
+}
+
+
 /* ------------------------------------------------------------------------ */
 
 #ifdef __cplusplus
