@@ -43,7 +43,7 @@ extern "C" {
 
 METHOD knh__System_getIn(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_RETURN(ctx, sfp, DP(knh_rootSystem)->in);
+	KNH_RETURN(ctx, sfp, DP(ctx->sys)->in);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -53,10 +53,10 @@ METHOD knh__System_setIn(Ctx *ctx, knh_sfp_t *sfp)
 {
 	if(knh_Context_isAdmin(ctx)) {
 		if(IS_NULL(sfp[1].o)) {
-			KNH_SETv(ctx, DP(knh_rootSystem)->in, knh_tClass_defaultValueNN(ctx, CLASS_InputStream));
+			KNH_SETv(ctx, DP(ctx->sys)->in, konoha_getClassDefaultValue(ctx, CLASS_InputStream));
 		}
 		else {
-			KNH_SETv(ctx, DP(knh_rootSystem)->in, sfp[1].o);
+			KNH_SETv(ctx, DP(ctx->sys)->in, sfp[1].o);
 		}
 	}
 	KNH_RETURN_void(ctx, sfp);
@@ -67,7 +67,7 @@ METHOD knh__System_setIn(Ctx *ctx, knh_sfp_t *sfp)
 
 METHOD knh__System_getOut(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_RETURN(ctx, sfp, DP(knh_rootSystem)->out);
+	KNH_RETURN(ctx, sfp, DP(ctx->sys)->out);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -77,10 +77,10 @@ METHOD knh__System_setOut(Ctx *ctx, knh_sfp_t *sfp)
 {
 	if(knh_Context_isAdmin(ctx)) {
 		if(IS_NULL(sfp[1].o)) {
-			KNH_SETv(ctx, DP(knh_rootSystem)->out, knh_tClass_defaultValueNN(ctx, CLASS_OutputStream));
+			KNH_SETv(ctx, DP(ctx->sys)->out, konoha_getClassDefaultValue(ctx, CLASS_OutputStream));
 		}
 		else {
-			KNH_SETv(ctx, DP(knh_rootSystem)->out, sfp[1].o);
+			KNH_SETv(ctx, DP(ctx->sys)->out, sfp[1].o);
 		}
 	}
 	KNH_RETURN_void(ctx, sfp);
@@ -91,7 +91,7 @@ METHOD knh__System_setOut(Ctx *ctx, knh_sfp_t *sfp)
 
 METHOD knh__System_getErr(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_RETURN(ctx, sfp, DP(knh_rootSystem)->err);
+	KNH_RETURN(ctx, sfp, DP(ctx->sys)->err);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -101,10 +101,10 @@ METHOD knh__System_setErr(Ctx *ctx, knh_sfp_t *sfp)
 {
 	if(knh_Context_isAdmin(ctx)) {
 		if(IS_NULL(sfp[1].o)) {
-			KNH_SETv(ctx, DP(knh_rootSystem)->err, knh_tClass_defaultValueNN(ctx, CLASS_OutputStream));
+			KNH_SETv(ctx, DP(ctx->sys)->err, konoha_getClassDefaultValue(ctx, CLASS_OutputStream));
 		}
 		else {
-			KNH_SETv(ctx, DP(knh_rootSystem)->err, sfp[1].o);
+			KNH_SETv(ctx, DP(ctx->sys)->err, sfp[1].o);
 		}
 	}
 	KNH_RETURN_void(ctx, sfp);
@@ -205,9 +205,9 @@ void knh_System__dump(Ctx *ctx, System *o, OutputStream *w, String *m)
 
 Object* knh_Context_getProperty(Ctx *ctx, Context *b, knh_bytes_t key)
 {
-	Object *v = knh_DictMap_get__b( b->props, key);
+	Object *v = knh_DictMap_get__b(ctx,  b->props, key);
 	if(IS_NULL(v)) {
-		return knh_System_getProperty(ctx, knh_rootSystem, key);
+		return knh_System_getProperty(ctx, ctx->sys, key);
 	}
 	return v;
 }
@@ -230,7 +230,7 @@ METHOD knh__Context_listProperties(Ctx *ctx, knh_sfp_t *sfp)
 	if(IS_NULL(sfp[1].s)) {
 		KNH_MOV(ctx, sfp[1].o, TS_EMPTY);
 	}
-	DictMap *map = DP(knh_rootSystem)->props;
+	DictMap *map = DP(ctx->sys)->props;
 	size_t i;
 	for(i = 0; i < map->size; i++) {
 		if(IS_NOTNULL(knh_DictMap_valueAt(map, i))) {
@@ -289,7 +289,7 @@ METHOD knh__Context_setIn(Ctx *ctx, knh_sfp_t *sfp)
 {
 	KNH_ASSERT(ctx == (Ctx*)sfp[0].o);
 	if(IS_NULL(sfp[1].o)) {
-		KNH_SETv(ctx, ((Context*)ctx)->in, knh_tClass_defaultValueNN(ctx, CLASS_InputStream));
+		KNH_SETv(ctx, ((Context*)ctx)->in, konoha_getClassDefaultValue(ctx, CLASS_InputStream));
 	}
 	else {
 		KNH_SETv(ctx, ((Context*)ctx)->in, sfp[1].o);
@@ -304,7 +304,7 @@ METHOD knh__Context_setOut(Ctx *ctx, knh_sfp_t *sfp)
 {
 	KNH_ASSERT(ctx == (Ctx*)sfp[0].o);
 	if(IS_NULL(sfp[1].o)) {
-		KNH_SETv(ctx, ((Context*)ctx)->out, knh_tClass_defaultValueNN(ctx, CLASS_OutputStream));
+		KNH_SETv(ctx, ((Context*)ctx)->out, konoha_getClassDefaultValue(ctx, CLASS_OutputStream));
 	}
 	else {
 		KNH_SETv(ctx, ((Context*)ctx)->out, sfp[1].o);
@@ -319,7 +319,7 @@ METHOD knh__Context_setErr(Ctx *ctx, knh_sfp_t *sfp)
 {
 	KNH_ASSERT(ctx == (Ctx*)sfp[0].o);
 	if(IS_NULL(sfp[1].o)) {
-		KNH_SETv(ctx, ((Context*)ctx)->err, knh_tClass_defaultValueNN(ctx, CLASS_OutputStream));
+		KNH_SETv(ctx, ((Context*)ctx)->err, konoha_getClassDefaultValue(ctx, CLASS_OutputStream));
 	}
 	else {
 		KNH_SETv(ctx, ((Context*)ctx)->err, sfp[1].o);
