@@ -41,48 +41,88 @@ extern "C" {
 #define _knh_Object_cid(o)           (o)->h.cid
 #define _knh_Object_bcid(o)          (o)->h.bcid
 
+/* ======================================================================== */
+/* [macros] */
 
+#define _IS_NULL(o)          (((Object*)o)->h.cid == CLASS_Nue)
+#define _IS_NOTNULL(o)       (((Object*)o)->h.cid != CLASS_Nue)
+
+#define _IS_TRUE(o)         ((o)->h.bcid == CLASS_Boolean && ((Int*)o)->n.bvalue)
+#define _IS_FALSE(o)        ((o)->h.bcid == CLASS_Boolean && (((Int*)o)->n.bvalue == 0))
+#define _new_Boolean(ctx, c)    (c) ? KNH_TRUE : KNH_FALSE
+
+#define _BOOL_ISTRUE(o)         (o == KNH_TRUE)
+#define _BOOL_ISFALSE(o)        (o == KNH_FALSE)
+
+///* ------------------------------------------------------------------------ */
+//
+//knh_Object_t *new_Boolean(Ctx *ctx, int c)
+//{
+//	return (c) ? KNH_TRUE : KNH_FALSE;
+//}
 
 /* ======================================================================== */
-/* [COBJ] */
-
-static
-void knh_fcfree__default(Ctx *ctx, void *ptr, knh_ftraverse ftr)
-{
-	;
-}
+/* [Null] */
 
 /* ------------------------------------------------------------------------ */
 
-static
-Object *knh_fccopy__default(Ctx *ctx, knh_COBJ_t *ptr)
+Object *new_Nue(Ctx *ctx, String *msg)
 {
-	TODO();
-	return (Object*)ptr;
+	KNH_ASSERT(IS_String(msg));
+	knh_Nue_t *o = (knh_Nue_t*)new_hObject(ctx, FLAG_Nue, CLASS_Nue, CLASS_Nue);
+	KNH_INITv(o->orign, msg);
+	o->str = (knh_uchar_t*)knh_String_tochar(msg);
+	o->size = knh_String_strlen(msg);
+	return (Object*)o;
 }
 
+#define _new_Nue__b(ctx, txt)   new_Nue(ctx, new_String(ctx, txt, NULL));
+#define _new_Nue__s(ctx, txt)   new_Nue(ctx, new_String(ctx, B(txt), NULL));
+
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knh_COBJ_init(Ctx *ctx, Object *o, void *ptr, knh_fcfree cf, knh_fccopy cc)
-{
-	KNH_ASSERT(IS_bAny(o));
-	knh_COBJ_t *c = (knh_COBJ_t*)o;
-	if(ptr != NULL) {
-		c->cptr = ptr;
-		if(cf == NULL) cf = knh_fcfree__default;
-		c->cfree = cf;
-		if(cc == NULL) cc = knh_fccopy__default;
-		c->ccopy = cc;
-	}
-	else {
-		if(c->cptr != NULL) {
-			c->cfree(ctx, c->cptr, knh_Object_RCsweep);
-		}
-		c->cptr = NULL;
-		c->cfree = knh_fcfree__default;
-		c->ccopy = knh_fccopy__default;
-	}
-}
+
+
+///* ======================================================================== */
+///* [COBJ] */
+//
+//static
+//void knh_fcfree__default(Ctx *ctx, void *ptr, knh_ftraverse ftr)
+//{
+//	;
+//}
+//
+///* ------------------------------------------------------------------------ */
+//
+//static
+//Object *knh_fccopy__default(Ctx *ctx, knh_Glue_t *ptr)
+//{
+//	TODO();
+//	return (Object*)ptr;
+//}
+//
+///* ------------------------------------------------------------------------ */
+//
+//KNHAPI(void) knh_COBJ_init(Ctx *ctx, Object *o, void *ptr, knh_fgluefree cf, knh_fccopy cc)
+//{
+//	KNH_ASSERT(IS_bAny(o));
+//	knh_Glue_t *c = (knh_Glue_t*)o;
+//	if(ptr != NULL) {
+//		c->cptr = ptr;
+//		if(cf == NULL) cf = knh_fcfree__default;
+//		c->cfree = cf;
+//		if(cc == NULL) cc = knh_fccopy__default;
+//		c->ccopy = cc;
+//	}
+//	else {
+//		if(c->cptr != NULL) {
+//			c->cfree(ctx, c->cptr, knh_Object_RCsweep);
+//		}
+//		c->cptr = NULL;
+//		c->cfree = knh_fcfree__default;
+//		c->ccopy = knh_fccopy__default;
+//	}
+//}
 
 /* ======================================================================== */
 /* [new] */
@@ -93,111 +133,6 @@ METHOD knh__Object_new(Ctx *ctx, knh_sfp_t *sfp)
 	KNH_RETURN(ctx, sfp, sfp[0].o);
 }
 
-/* ======================================================================== */
-/* [init] */
-
-Object *new_Object__init(Ctx *ctx, knh_flag_t flag, knh_class_t cid)
-{
-	KNH_ASSERT_cid(cid);
-
-	switch(knh_tClass[cid].bcid) {
-
-	case CLASS_Nue :
-		KNH_ASSERT(ctx == NULL); /* stop */
-		return (Object*)KNH_NULL;
-	case CLASS_Boolean :
-		KNH_ASSERT(ctx == NULL); /* stop */
-		return (Object*)KNH_FALSE;
-	case CLASS_Class :
-		KNH_ASSERT(ctx == NULL); /* stop */
-		return (Object*)knh_tClass[CLASS_Object].class_cid;
-
-	case CLASS_Int :       case CLASS_IntX :
-	case CLASS_Float :     case CLASS_FloatX :
-	case CLASS_String :    case CLASS_StringX :
-		KNH_ASSERT(ctx == NULL); /* stop */
-		return knh_tClass[cid].fdefault(ctx, cid);
-
-	case CLASS_Context:
-		{
-			KNH_ASSERT(ctx == NULL); /* stop */
-			return (Object*)ctx;
-		}
-
-	case CLASS_Bytes :
-		{
-			KNH_ASSERT((knh_tClass[cid].size == 0));
-			knh_Bytes_t *b = (knh_Bytes_t*)new_Object(ctx, flag, cid);
-			b->buf = (knh_uchar_t*)NULL;
-			b->size = 0;
-			return (Object*)b;
-		}
-
-	case CLASS_Range:
-	case CLASS_Tuple2:
-		{
-			KNH_ASSERT((knh_tClass[cid].size == 0));
-			knh_Tuple2_t *t = (knh_Tuple2_t*)new_Object(ctx, flag, cid);
-			KNH_INITv(t->first, KNH_NULL);
-			KNH_INITv(t->second, KNH_NULL);
-			return (Object*)t;
-		}
-
-	case CLASS_Array :
-		{
-			KNH_ASSERT((knh_tClass[cid].size == 0));
-			knh_Array_t *a = (knh_Array_t*)new_Object(ctx, flag, cid);
-			a->list = NULL;
-			a->size = 0;
-			return (Object*)a;
-		}
-
-	case CLASS_DictMap :
-	case CLASS_DictSet :
-		{
-			KNH_ASSERT((knh_tClass[cid].size == 0));
-			knh_DictMap_t *d = (knh_DictMap_t*)new_Object(ctx, flag, cid);
-			d->_list = NULL;
-			d->size = 0;
-			d->fcmp = knh_bytes_strcmp;
-			return (Object*)d;
-		}
-	case CLASS_DictIdx :
-		{
-			knh_DictIdx_t *d = (knh_DictIdx_t*)new_Object(ctx, flag, cid);
-			KNH_INITv(d->terms, new_Array(ctx, CLASS_String, 0));
-			KNH_INITv(d->termsDictSet, new_DictSet(ctx, 0));
-			d->offset = 0;
-			return (Object*)d;
-		}
-
-	case CLASS_Regex:
-		{
-			KNH_ASSERT((knh_tClass[cid].size == 0));
-			knh_Regex_t *re = (knh_Regex_t*)new_Object(ctx, flag, cid);
-			KNH_INITv(re->pattern, TS_EMPTY);
-			re->df = knh_System_getRegexDriver(ctx, STEXT("NOP"));
-			re->reg = NULL;
-			return UP(re);
-		}
-	case CLASS_Any:  /* This is native interface */
-		{
-			KNH_ASSERT((knh_tClass[cid].size == 0));
-			knh_COBJ_t *a = (knh_COBJ_t*)new_Object(ctx, flag, cid);
-			a->cptr = NULL;
-			a->cfree = knh_fcfree__default;
-			a->ccopy = knh_fccopy__default;
-			return (Object*)a;
-		}
-
-	case CLASS_Object:
-	default :
-		{
-			return new_Object(ctx, flag, cid);
-		}
-	}
-}
-
 /* ======================================================================== */
 /* [copy] */
 
@@ -213,14 +148,9 @@ Object *knh_Object_copy(Ctx *ctx, Object *b)
 /* ======================================================================== */
 /* [commons] */
 
-knh_hcode_t knh_Object_hashCode(Object *o)
+knh_hcode_t knh_Object_hashCode(Ctx *ctx, Object *o)
 {
-	if(o->h.cid == CLASS_Object) {
-		return (knh_hcode_t)o / sizeof(Object*);
-	}
-	else {
-		return knh_tStruct[o->h.bcid].fhashCode(o);
-	}
+	return ctx->tStruct[o->h.bcid].fhashCode(ctx, o);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -235,13 +165,6 @@ knh_bytes_t knh_Object_tobytes(Ctx *ctx, Object *o)
 	return STEXT("");
 }
 
-/* ======================================================================== */
-/* [structs] */
-
-/* ======================================================================== */
-/* [mapping] */
-
-
 /* ------------------------------------------------------------------------ */
 
 #ifdef __cplusplus
