@@ -76,7 +76,7 @@ void knh_String_checkASCII(String *o)
 
 KNHAPI(String*) new_String(Ctx *ctx, knh_bytes_t t, String *orign)
 {
-	knh_String_t *o = (String*)new_PObject__NNctx(ctx, FLAG_String, CLASS_String, CLASS_String);
+	knh_String_t *o = (String*)new_hObject(ctx, FLAG_String, CLASS_String, CLASS_String);
 	if(orign == NULL) {
 		o->size = t.len;
 		o->str = (knh_uchar_t*)KNH_MALLOC(ctx, KNH_SIZE(o->size+1));
@@ -107,42 +107,25 @@ KNHAPI(String*) new_String(Ctx *ctx, knh_bytes_t t, String *orign)
 
 /* ------------------------------------------------------------------------ */
 
-String *new_String0(Ctx *ctx, knh_bytes_t t, String *orign)
+knh_String_t *new_StringX__T(Ctx *ctx, knh_class_t cid, char *text)
 {
-	knh_String_t *o = (String*)new_PObject0(ctx, FLAG_String, CLASS_String, CLASS_String);
-	if(orign == NULL) {
-		o->size = t.len;
-		o->str = (knh_uchar_t*)KNH_MALLOC(ctx, KNH_SIZE(o->size+1));
-		knh_memcpy(o->str, t.buf, t.len);
-		o->str[o->size] = '\0';
-		o->orign = NULL;
-		knh_String_checkASCII(o);
-	}else {
-		KNH_ASSERT(IS_bString(orign));
-		o->size = t.len;
-		o->str = (knh_uchar_t*)t.buf;
-		if(orign->orign == NULL) {
-			KNH_INITv(o->orign, orign);
-		}
-		else {
-			KNH_ASSERT(IS_bString(orign->orign));
-			KNH_INITv(o->orign, orign->orign);
-		}
-		if(knh_String_isASCII(orign)) {
-			knh_String_setASCII(o, 1);
-		}
-		else {
-			knh_String_checkASCII(o);
-		}
-	}
+	knh_String_t *o = (knh_String_t*)new_hObject(ctx, FLAG_String, CLASS_String, cid);
+	o->str = (knh_uchar_t*)text;
+	o->size = knh_strlen(text);
+	o->orign = NULL;
+	knh_String_setTextSgm(o, 1);
+	knh_String_checkASCII(o);
 	return o;
 }
 
+#define _new_String__T(ctx, text)    new_StringX__T(ctx, CLASS_String, text)
+
+
 /* ------------------------------------------------------------------------ */
 
 String *new_StringX__fast(Ctx *ctx, knh_class_t cid, knh_bytes_t t, String *orign)
 {
-	knh_String_t *o = (String*)new_PObject0(ctx, FLAG_String, CLASS_String, cid);
+	knh_String_t *o = (String*)new_hObject(ctx, FLAG_String, CLASS_String, cid);
 	if(orign == NULL) {
 		o->size = t.len;
 		o->str = (knh_uchar_t*)KNH_MALLOC(ctx, KNH_SIZE(o->size+1));
@@ -189,7 +172,7 @@ String *new_StringX(Ctx *ctx, knh_class_t cid, knh_bytes_t t, String *orign)
 {
 	KNH_ASSERT_cid(cid);
 	{
-		StringUnit *u = (StringUnit*)knh_tClass[cid].cspec;
+		StringUnit *u = (StringUnit*)ctx->tClass[cid].cspec;
 		KNH_ASSERT(IS_StringUnit(u));
 		if(DP(u)->fbconv != NULL) {
 			Bytes *ba = knh_Context_openBConvBuf(ctx);
@@ -229,48 +212,6 @@ String *new_String__float(Ctx *ctx, knh_float_t n)
 	return new_String(ctx, B(buf), NULL);
 }
 
-/* ------------------------------------------------------------------------ */
-
-Object *knh_String_fdefault(Ctx *ctx, knh_class_t cid)
-{
-	return (Object*)TS_EMPTY;
-}
-
-/* ======================================================================== */
-/* [commons] */
-
-knh_hcode_t knh_String_hashCode(String *o)
-{
-	KNH_ASSERT(IS_bString(o));
-	knh_hcode_t h = o->size;
-	size_t i;
-	for(i = 0; i < o->size; i++) {
-		h = o->str[i] + 31 * h;
-	}
-	return h;
-}
-
-/* ------------------------------------------------------------------------ */
-
-int knh_String_compareTo(String *o, String *o2)
-{
-	if(o->h.cid == CLASS_String || o2->h.cid == CLASS_String) {
-		size_t i, max = knh_uint_min(o->size, o2->size);
-		for(i = 0; i < max; i++) {
-			int res = o->str[i] - o2->str[i];
-			if(res != 0) return res;
-		}
-		return o->size - o2->size;
-	}
-	else {
-		if(o->h.cid == o2->h.cid) {
-			StringUnit *u = (StringUnit*)knh_tClass[o->h.cid].cspec;
-			KNH_ASSERT(IS_StringUnit(u));
-			return DP(u)->fcmp(u, knh_String_tobytes(o), knh_String_tobytes(o2));
-		}
-		return (int)(o - o2);
-	}
-}
 
 /* ======================================================================== */
 /* [misc] */
