@@ -46,39 +46,21 @@ Mapper* new_Mapper__interface(Ctx *ctx, knh_class_t scid, knh_class_t tcid)
 
 /* ------------------------------------------------------------------------ */
 
-static
-knh_bool_t knh_MethodField_equalsType(MethodField *o, MethodField *o2)
-{
-	knh_index_t i;
-	if(DP(o)->size != DP(o2)->size) return 0;
-	for(i = 0; i < DP(o)->size; i++) {
-		if(DP(o)->params[i].type != DP(o2)->params[i].type) return 0;
-	}
-	if(knh_MethodField_isVarArgs(o)) {
-		return knh_MethodField_isVarArgs(o2) ? 1 : 0;
-	}
-	else {
-		return knh_MethodField_isVarArgs(o2) ? 0 : 1;
-	}
-}
-
-/* ------------------------------------------------------------------------ */
-
 static
 void knh_class_addInterface(Ctx *ctx, knh_class_t cid, knh_class_t icid)
 {
 	KNH_ASSERT(!knh_class_isInterface(icid));
-	if(knh_class_instanceof(cid, icid)) {
+	if(knh_class_instanceof(ctx, cid, icid)) {
 		return ;
 	}
 	else {
 		knh_class_t isupcid = icid;
 		int allchecked = 1;
 		while(isupcid != CLASS_Object) {
-			ClassStruct *cs = knh_tClass[isupcid].cstruct;
+			ClassStruct *cs = ctx->tClass[isupcid].cstruct;
 			size_t i;
-			for(i = 0; i < knh_Array_size(DP(cs)->methods); i++) {
-				Method *imtd = (Method*)knh_Array_n(DP(cs)->methods, i);
+			for(i = 0; i < knh_Array_size(cs->methods); i++) {
+				Method *imtd = (Method*)knh_Array_n(cs->methods, i);
 				if(knh_Method_isPrivate(imtd)) {
 					continue;
 				}
@@ -87,22 +69,22 @@ void knh_class_addInterface(Ctx *ctx, knh_class_t cid, knh_class_t icid)
 					if(IS_NOTNULL(mtd)) {
 						if(!knh_MethodField_equalsType(DP(mtd)->mf, DP(imtd)->mf)) {
 							char bufcm[CLASSNAME_BUFSIZ];
-							knh_format_cmethodn(bufcm, sizeof(bufcm), DP(imtd)->cid, DP(imtd)->mn);
+							knh_format_cmethodn(ctx, bufcm, sizeof(bufcm), DP(imtd)->cid, DP(imtd)->mn);
 							DBG2_P("mismatch!! %s", bufcm);
 							allchecked = 0;
 						}
 					}
 				}
 			}
-			isupcid = knh_tClass[isupcid].supcid;
+			isupcid = ctx->tClass[isupcid].supcid;
 		}
 		if(allchecked) {
 			isupcid = icid;
 			while(isupcid != CLASS_Object) {
-				ClassStruct *cs = knh_tClass[isupcid].cstruct;
+				ClassStruct *cs = ctx->tClass[isupcid].cstruct;
 				size_t i;
-				for(i = 0; i < knh_Array_size(DP(cs)->methods); i++) {
-					Method *imtd = (Method*)knh_Array_n(DP(cs)->methods, i);
+				for(i = 0; i < knh_Array_size(cs->methods); i++) {
+					Method *imtd = (Method*)knh_Array_n(cs->methods, i);
 					if(knh_Method_isPrivate(imtd)) {
 						continue;
 					}
@@ -116,10 +98,10 @@ void knh_class_addInterface(Ctx *ctx, knh_class_t cid, knh_class_t icid)
 						}
 					}
 				}
-				isupcid = knh_tClass[isupcid].supcid;
+				isupcid = ctx->tClass[isupcid].supcid;
 			}
 			DBG2_P("add interface %s to %s", CLASSN(icid), CLASSN(cid));
-			knh_ClassMap_add(ctx, knh_tClass[cid].cmap, new_Mapper__interface(ctx, cid, icid));
+			knh_ClassMap_add(ctx, ctx->tClass[cid].cmap, new_Mapper__interface(ctx, cid, icid));
 		}
 	}
 }
@@ -129,7 +111,7 @@ void knh_class_addInterface(Ctx *ctx, knh_class_t cid, knh_class_t icid)
 static
 Object *knh_tClass_fdefault__OBJECT(Ctx *ctx, knh_class_t cid)
 {
-	return new_Object__init(ctx, knh_tClass[cid].oflag, cid);
+	return new_Object_init(ctx, ctx->tClass[cid].oflag, cid, 0);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -139,7 +121,7 @@ int knh_StmtCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 {
 	char bufn[CLASSNAME_BUFSIZ];
 	knh_snprintf(bufn, sizeof(bufn), "%s.%s", knh_String_tochar(DP(ns)->nsname),
-			knh_Token_tochar(StmtCLASS_class(stmt)));
+			sToken(StmtCLASS_class(stmt)));
 	{
 		knh_tClass_t *TC = NULL;
 		knh_class_t cid  = knh_NameSpace_getClass(ctx, ns, B(bufn));
@@ -148,7 +130,7 @@ int knh_StmtCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 			supcid = CLASS_Object;
 		}
 		else {
-			supcid = knh_NameSpace_getClass(ctx, ns, knh_Token_tobytes(StmtCLASS_superclass(stmt)));
+			supcid = knh_NameSpace_getClass(ctx, ns, knh_Token_tobytes(ctx, StmtCLASS_superclass(stmt)));
 			if(supcid == CLASS_unknown && cid == CLASS_unknown) {
 				knh_Asm_perror(ctx, abr, KMSG_UCLASSN, NULL);
 				supcid = CLASS_Object;
@@ -164,34 +146,34 @@ int knh_StmtCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 		}
 		if(cid == CLASS_unknown) {
 			cid = knh_tClass_newId(ctx);
-			KNH_ASSERT(knh_tClass[cid].class_cid == NULL);
+			KNH_ASSERT(ctx->tClass[cid].class_cid == NULL);
 		}
 		else {
 			DEBUG3("redefine %s", CLASSN(cid));
 			KNH_ASSERT_cid(cid);
-			if(knh_tClass[cid].bcid != CLASS_Object) {
+			if(ctx->tClass[cid].bcid != CLASS_Object) {
 				knh_Asm_perror(ctx, abr, KMSG_EOVERRIDE, CLASSN(cid));
 				knh_Stmt_done(ctx, stmt);
 				return 0;
 			}
-			if(!knh_StmtMETA_isOverride(stmt)) {
+			if(!knh_StmtMETA_isOverride(ctx, stmt)) {
 				knh_Asm_perror(ctx, abr, KMSG_AOVERRIDE, CLASSN(cid));
 				knh_Stmt_done(ctx, stmt);
 				return 0;
 			}
-			if(knh_tClass[cid].supcid != supcid) {
+			if(ctx->tClass[cid].supcid != supcid) {
 				knh_Asm_perror(ctx, abr, KMSG_DIFFCLASSN, NULL);
-				supcid = knh_tClass[cid].supcid;
+				supcid = ctx->tClass[cid].supcid;
 				knh_Token_perrata(ctx, StmtCLASS_superclass(stmt), CLASSN(supcid));
 			}
 			oldcid = cid;
 			cid = knh_tClass_newId(ctx);
-			KNH_ASSERT(knh_tClass[cid].class_cid == NULL);
+			KNH_ASSERT(ctx->tClass[cid].class_cid == NULL);
 		}
 
 		DP(StmtCLASS_class(stmt))->cid = cid;
-		TC = (knh_tClass_t*)&(knh_tClass[cid]);
-		TC->cflag  = knh_Stmt_metaflag__class(ctx, stmt);
+		TC = (knh_tClass_t*)&(ctx->tClass[cid]);
+		TC->cflag  = knh_StmtCLASS_flag(ctx, stmt);
 		TC->oflag  = KNH_FLAG_CF2OF(TC->cflag);
 
 		TC->bcid   = CLASS_Object;
@@ -200,11 +182,11 @@ int knh_StmtCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 			TC->offset = 0;
 		}else {
 			KNH_ASSERT(supcid < cid);
-			TC->offset = knh_tClass[supcid].bsize;
+			TC->offset = ctx->tClass[supcid].bsize;
 		}
 
 		KNH_ASSERT(TC->class_cid == NULL);
-		KNH_TCLASS_NAME(ctx, cid, new_String(ctx, B(bufn), NULL));
+		konoha_setClassName(ctx, cid, new_String(ctx, B(bufn), NULL));
 		KNH_INITv(TC->cstruct, new_ClassStruct0(ctx, 0, 8));
 		KNH_INITv(TC->cmap, new_ClassMap0(ctx, 0));
 		KNH_INITv(TC->cspec, KNH_NULL);
@@ -220,7 +202,7 @@ int knh_StmtCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 				int i, n = DP(istmt)->size;
 				for(i = 0; i < n; i++) {
 					Token *tk = DP(stmt)->tokens[i];
-					knh_class_t icid = knh_NameSpace_getClass(ctx, ns, knh_Token_tobytes(tk));
+					knh_class_t icid = knh_NameSpace_getClass(ctx, ns, knh_Token_tobytes(ctx, tk));
 					if(icid == CLASS_unknown) {
 						knh_Token_perror(ctx, tk, KMSG_UCLASSN);
 						continue;
@@ -245,7 +227,7 @@ int knh_StmtIMPORT_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 {
 	String *fname = (String*)DP(StmtIMPORT_file(stmt))->data;
 	char bufp[FILENAME_BUFSIZ], buff[FILENAME_BUFSIZ];
-	String *cfname = knh_tfileid_name(SP(stmt)->fileid);
+	String *cfname = knh_tfileid_name(ctx, SP(stmt)->fileid);
 	KNH_ASSERT(IS_String(fname));
 	if(knh_String_endsWith(cfname, STEXT(".k"))) {
 		knh_format_parentpath(bufp, sizeof(bufp), knh_String_tobytes(cfname), 1);
@@ -282,7 +264,7 @@ static
 int knh_StmtUIMPORT_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 {
 	Token *tk = DP(stmt)->tokens[0];
-	knh_bytes_t name = knh_Token_tobytes(tk);
+	knh_bytes_t name = knh_Token_tobytes(ctx, tk);
 	knh_index_t loc = knh_bytes_rindex(name, '.');
 	if(loc != -1) {
 		if(knh_System_using(ctx, knh_bytes_first(name, loc))) {
@@ -323,7 +305,7 @@ int knh_StmtUALIAS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 /* ------------------------------------------------------------------------ */
 
 static
-char *knh_format_classurn(char *buf, size_t bufsiz, knh_class_t bcid, knh_bytes_t urn)
+char *knh_format_classurn(Ctx *ctx, char *buf, size_t bufsiz, knh_class_t bcid, knh_bytes_t urn)
 {
 	knh_snprintf(buf, bufsiz, KNH_CLASSSPEC_FMT, CLASSN(bcid), urn.buf);
 	return buf;
@@ -357,16 +339,16 @@ int knh_StmtXCLASS_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_class
 	Token *tkurn = DP(stmt)->tokens[1];
 
 	char bufcu[CLASSNAME_BUFSIZ];
-	knh_format_classurn(bufcu, sizeof(bufcu), bcid, knh_Token_tobytes(tkurn));
+	knh_format_classurn(ctx, bufcu, sizeof(bufcu), bcid, knh_Token_tobytes(ctx, tkurn));
 	knh_class_t cid = knh_tClass_findcid(ctx, B(bufcu));
 	if(cid == bcid || cid == CLASS_unknown) {
 		knh_Token_perror(ctx, tkurn, KMSG_UPATH);
 		cid = bcid;
 	}
 
-	knh_class_t oldcid = knh_NameSpace_getClass(ctx, ns, knh_Token_tobytes(tkclassn));
+	knh_class_t oldcid = knh_NameSpace_getClass(ctx, ns, knh_Token_tobytes(ctx, tkclassn));
 	if(oldcid != CLASS_unknown && cid != oldcid) {
-		KNH_WARNING(ctx, "Overriding %s", knh_Token_tobytes(tkclassn));
+		KNH_WARNING(ctx, "Overriding %s", knh_Token_tobytes(ctx, tkclassn));
 	}
 
 	KNH_ASSERT(IS_String(DP(tkclassn)->data));
@@ -404,7 +386,7 @@ static
 int knh_StmtUFUNC_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 {
 	Token *tk = DP(stmt)->tokens[0];
-	knh_bytes_t name = knh_Token_tobytes(tk);
+	knh_bytes_t name = knh_Token_tobytes(ctx, tk);
 	if(SP(tk)->tt == TT_CMETHODN) {
 		knh_index_t loc = knh_bytes_rindex(name, '.');
 		knh_class_t cid = knh_NameSpace_getClass(ctx, ns, knh_bytes_first(name, loc));
@@ -427,7 +409,7 @@ int knh_StmtUFUNC_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 		}
 		else {
 			KNH_ASSERT_cid(cid);
-			Array *a = DP(knh_tClass[cid].cstruct)->methods;
+			Array *a = (ctx->tClass[cid].cstruct)->methods;
 			size_t i;
 			for(i = 0; i < knh_Array_size(a); i++) {
 				Method *mtd = (Method*)knh_Array_n(a, i);
@@ -436,7 +418,7 @@ int knh_StmtUFUNC_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 				knh_NameSpace_setFuncClass(ctx, ns, DP(mtd)->mn, cid);
 				if(knh_Context_isVerbose(ctx)) {
 					char bufcm[CLASSNAME_BUFSIZ];
-					knh_format_cmethodn(bufcm, sizeof(bufcm), cid, DP(mtd)->mn);
+					knh_format_cmethodn(ctx, bufcm, sizeof(bufcm), cid, DP(mtd)->mn);
 					KNH_NOTICE(ctx, "using %s", bufcm);
 				}
 			}
@@ -462,7 +444,7 @@ int knh_StmtUMAPMAP_decl(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 //	//DEBUG_STMT(ctx, stmt);
 //	//Asm *mc = (Asm*)v;
 //	/* ['weave', 'ANY', 'METHODFN'] */
-//	fprintf(stdout, "weaving %s into %s..\n", knh_Token_tochar((Token*)stmt->terms[1]), knh_Token_tochar((Token*)stmt->terms[2]));
+//	fprintf(stdout, "weaving %s into %s..\n", sToken((Token*)stmt->terms[1]), sToken((Token*)stmt->terms[2]));
 //	Method *mtd = knh_Class_getMethod(ctx, CLASS_Amazon, METHODN_placeOrder);
 //	if(IS_Method(mtd)) {
 //		String *aspect = ((Token*)stmt->terms[1])->data;
@@ -523,13 +505,14 @@ int knh_Stmt_eval(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, int isEval)
 	}
 
 	if(isEval) {
-		KNH_MOV(ctx, lsfp[0].o, new_ExceptionHandler(ctx));
+		ExceptionHandler *hdr = new_ExceptionHandler(ctx);
+		KNH_MOV(ctx, lsfp[0].o, hdr);
 		KNH_TRY(ctx, L_CATCH, lsfp, 0);
 		{
 			KNH_MOV(ctx, lsfp[2].o, scr);
 			KNH_SCALL(ctx, lsfp, 1, mtd, 0/*args*/);
 			if(isExpr && !knh_Stmt_isVOID(stmt)) {
-				DBG2_P("return %s %d", CTXCLASSN(knh_Object_cid(lsfp[1].o)), (int)lsfp[1].ivalue);
+				DBG2_P("returning %s %lld", CTXCLASSN(knh_Object_cid(lsfp[1].o)), lsfp[1].ivalue);
 				knh_sfp_format(ctx, lsfp+1, mt, KNH_STDOUT, KNH_NULL);
 				knh_write_EOL(ctx, KNH_STDOUT);
 			}
