@@ -45,7 +45,7 @@ static Stmt *new_StmtMETA(Ctx *ctx,  knh_tokens_t *tc, knh_stmt_t stt);
 static void knh_Stmt_addMETA(Ctx *ctx, Stmt *o, knh_tokens_t *tc);
 static void knh_Stmt_add_STMT1(Ctx *ctx, Stmt *o, knh_tokens_t *tc);
 static void knh_Stmt_add_SEMICOLON(Ctx *ctx, Stmt *o, knh_tokens_t *tc);
-static int knh_Token_isLVALUE(Token *tk);
+static int knh_Token_isLVALUE(Ctx *ctx, Token *tk);
 static void knh_Stmt_toLVALUE(Ctx *ctx, Stmt *stmt, int pe);
 
 /* ------------------------------------------------------------------------ */
@@ -72,12 +72,12 @@ static void knh_tokens_skipSEMICOLON(knh_tokens_t *tc)
 
 /* ------------------------------------------------------------------------ */
 
-static
-void knh_tokens_nextStmt(knh_tokens_t *tc)
+static void knh_tokens_nextStmt(knh_tokens_t *tc)
 {
 	Token **ts = tc->ts;
 	int i;
 	for(i = tc->c; i < tc->e; i++) {
+		DBG2_P("i=%d, %s", i, TOKENNo(ts[i]));
 		if(knh_Token_isBOL(ts[i])) {
 			tc->c = i;
 			knh_tokens_skipSEMICOLON(tc);
@@ -142,11 +142,11 @@ void knh_tokens_ignore(Ctx *ctx, knh_tokens_t *tc)
 
 /* namespace classname konoha.Class */
 
-static knh_bool_t knh_Token_isNSCLASSN(Token *tk)
+static knh_bool_t knh_Token_isNSCLASSN(Ctx *ctx, Token *tk)
 {
 	if(SP(tk)->tt == TT_NAME) {
 		size_t i, prev = 0;
-		knh_bytes_t t = knh_Token_tobytes(tk);
+		knh_bytes_t t = knh_Token_tobytes(ctx, tk);
 		for(i = 0; i < t.len; i++) {
 			if(isupper(t.buf[i])) {
 				if(prev == '.') return 1; else return 0;
@@ -167,7 +167,7 @@ static knh_bool_t knh_Token_isNSCLASSN(Token *tk)
 static void knh_Stmt_add_NSCLASSN(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 {
 	if(SP(o)->stt == STT_ERR) return;
-	if(tc->c < tc->e && knh_Token_isNSCLASSN(tc->ts[tc->c])) {
+	if(tc->c < tc->e && knh_Token_isNSCLASSN(ctx, tc->ts[tc->c])) {
 		knh_Stmt_add(ctx, o, UP(tc->ts[tc->c]));
 		tc->c += 1;
 	}
@@ -192,10 +192,10 @@ static void knh_Stmt_add_TYPEN(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 
 /* ------------------------------------------------------------------------ */
 
-static int  knh_Token_isCLASSN(Token *tk)
+static int  knh_Token_isCLASSN(Ctx *ctx, Token *tk)
 {
 	if(SP(tk)->tt == TT_TYPEN) {
-		knh_bytes_t t = knh_Token_tobytes(tk);
+		knh_bytes_t t = knh_Token_tobytes(ctx, tk);
 		if(isupper(t.buf[0])) {
 			size_t i;
 			for(i = 0; i < t.len; i++) {
@@ -212,7 +212,7 @@ static int  knh_Token_isCLASSN(Token *tk)
 static void knh_Stmt_add_CLASSN(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 {
 	if(SP(o)->stt == STT_ERR) return;
-	if(tc->c < tc->e && knh_Token_isCLASSN(tc->ts[tc->c])) {
+	if(tc->c < tc->e && knh_Token_isCLASSN(ctx, tc->ts[tc->c])) {
 		knh_Stmt_add(ctx, o, UP(tc->ts[tc->c]));
 		tc->c += 1;
 	}
@@ -223,10 +223,10 @@ static void knh_Stmt_add_CLASSN(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 
 /* ------------------------------------------------------------------------ */
 
-static int knh_Token_isCLASSTN(Token *tk)
+static int knh_Token_isCLASSTN(Ctx *ctx, Token *tk)
 {
 	if(SP(tk)->tt == TT_TYPEN) {
-		knh_bytes_t t = knh_Token_tobytes(tk);
+		knh_bytes_t t = knh_Token_tobytes(ctx, tk);
 		return isupper(t.buf[0]);
 	}
 	return 0;
@@ -237,7 +237,7 @@ static int knh_Token_isCLASSTN(Token *tk)
 static void knh_Stmt_add_CLASSTN(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 {
 	if(SP(o)->stt == STT_ERR) return;
-	if(tc->c < tc->e && knh_Token_isCLASSTN(tc->ts[tc->c])) {
+	if(tc->c < tc->e && knh_Token_isCLASSTN(ctx, tc->ts[tc->c])) {
 		knh_Stmt_add(ctx, o, UP(tc->ts[tc->c]));
 		tc->c += 1;
 	}
@@ -248,10 +248,10 @@ static void knh_Stmt_add_CLASSTN(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 
 /* ------------------------------------------------------------------------ */
 
-static int knh_Token_isPATH(Token *tk)
+static int knh_Token_isPATH(Ctx *ctx, Token *tk)
 {
 	if(SP(tk)->tt == TT_URN) return 1;
-	return knh_Token_isNSCLASSN(tk);
+	return knh_Token_isNSCLASSN(ctx, tk);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -259,7 +259,7 @@ static int knh_Token_isPATH(Token *tk)
 static void knh_Stmt_add_PATH(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 {
 	if(SP(o)->stt == STT_ERR) return;
-	if(tc->c < tc->e && knh_Token_isPATH(tc->ts[tc->c])) {
+	if(tc->c < tc->e && knh_Token_isPATH(ctx, tc->ts[tc->c])) {
 		knh_Stmt_add(ctx, o, UP(tc->ts[tc->c]));
 		tc->c += 1;
 	}
@@ -273,7 +273,7 @@ static void knh_Stmt_add_PATH(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 //static int knh_Token_isMETHODN(Token *tk)
 //{
 //	if(SP(tk)->tt == TT_NAME) {
-//		knh_bytes_t t = knh_Token_tobytes(tk);
+//		knh_bytes_t t = knh_Token_tobytes(ctx, tk);
 //		if(isupper(t.buf[0])) {
 //			size_t i;
 //			for(i = 0; i < t.len; i++) {
@@ -306,7 +306,7 @@ static int knh_Token_isVARN(Token *tk)
 {
 	if(SP(tk)->tt == TT_NAME) {
 		size_t i;
-		knh_bytes_t t = knh_Token_tobytes(tk);
+		knh_bytes_t t = knh_String_tobytes(DP(tk)->text);
 		for(i = 0; i < t.len; i++) {
 			if(t.buf[i] == '.' || t.buf[i] == ':') return 0;
 		}
@@ -522,6 +522,7 @@ static Token* knh_tokens_curToken(knh_tokens_t *tc)
 		if(c == 0) break;
 		c--;
 	}
+	tc->c = c + 1;
 	return tc->ts[c];
 }
 
@@ -700,7 +701,7 @@ Stmt* knh_Stmt_add_SECONDNAME(Ctx *ctx, Stmt *stmt, Token *tk, int lr)
 	KNH_ASSERT(DP(stmt)->size == 1);
 	{
 		Stmt *stmt_head = stmt;
-		knh_bytes_t name = knh_Token_tobytes(tk);
+		knh_bytes_t name = knh_Token_tobytes(ctx, tk);
 		knh_index_t idx = knh_bytes_index(name, '.');
 		while(idx > 0) {
 			knh_Stmt_add(ctx, stmt_head, TM(new_TokenFUNCNAME(ctx, tk, knh_bytes_first(name, idx), 0, KNH_RVALUE)));
@@ -721,7 +722,7 @@ static Term *new_TermNAME(Ctx *ctx, Token *tk, int lr)
 {
 	KNH_ASSERT(SP(tk)->tt == TT_NAME);
 	{
-		knh_bytes_t t = knh_Token_tobytes(tk);
+		knh_bytes_t t = knh_Token_tobytes(ctx, tk);
 		if(knh_bytes_rindex(t, '.') == -1) {
 			return TM(tk);
 		}
@@ -740,7 +741,7 @@ Stmt *new_StmtNAME1(Ctx *ctx, Token *tk, int lr)
 {
 	KNH_ASSERT(SP(tk)->tt == TT_NAME);
 	{
-		knh_bytes_t t = knh_Token_tobytes(tk);
+		knh_bytes_t t = knh_Token_tobytes(ctx, tk);
 		if(knh_bytes_rindex(t, '.') == -1) {
 			Stmt *stmt = new_Stmt(ctx, 0, STT_CALL);
 			knh_Stmt_add(ctx, stmt, TM(tk));
@@ -760,7 +761,7 @@ static
 Stmt *new_StmtNAME2(Ctx *ctx, Token *tk, int lr)
 {
 	DEBUG3_ASSERT(SP(tk)->tt == TT_NAME || SP(tk)->tt == TT_CMETHODN);
-	knh_bytes_t t = knh_Token_tobytes(tk);
+	knh_bytes_t t = knh_Token_tobytes(ctx, tk);
 	if(knh_bytes_rindex(t, '.') == -1) {
 		Stmt *stmt = new_Stmt(ctx, 0, STT_CALL);
 		knh_Stmt_add(ctx, stmt, TM(tk));
@@ -978,7 +979,7 @@ void knh_tokens_toArray(Ctx *ctx, knh_tokens_t *tc, Array *a, int ignoreCOMMA)
 			knh_Array_add(ctx, a, DP(ts[i])->data);
 		}
 		else {
-			knh_Array_add(ctx, a, UP(new_String(ctx, knh_Token_tobytes(ts[i]), NULL)));
+			knh_Array_add(ctx, a, UP(new_String(ctx, knh_Token_tobytes(ctx, ts[i]), NULL)));
 		}
 	}
 }
@@ -1031,7 +1032,7 @@ void knh_Stmt_add_PAIR(Ctx *ctx, Stmt *stmt, knh_tokens_t *tc)
 			return ;
 		}
 		if(!IS_bString(DP(ts[s])->data)) {
-			KNH_SETv(ctx, DP(ts[s])->data, new_String(ctx, knh_Token_tobytes(ts[s]), NULL));
+			KNH_SETv(ctx, DP(ts[s])->data, new_String(ctx, knh_Token_tobytes(ctx, ts[s]), NULL));
 		}
 		knh_Stmt_add(ctx, stmt, TM(knh_Token_toCONST(ts[s])));
 		return;
@@ -1122,7 +1123,7 @@ static
 Stmt *new_StmtPROPN(Ctx *ctx, Token *tk, int lr)
 {
 	Stmt *stmt = new_Stmt(ctx, 0, STT_CALL);
-	DBG2_P("tk=%s,%d", knh_Token_tochar(tk), knh_Token_isTailWildCard(tk));
+	DBG2_P("tk=%s,%d", sToken(tk), knh_Token_isTailWildCard(tk));
 	DBG2_ABORT();
 	if(lr == KNH_LVALUE) {
 		knh_Stmt_add(ctx, stmt, TM(new_TokenMN(ctx, FL(tk), METHODN_setProperty)));
@@ -1144,7 +1145,7 @@ Term *new_TermVALUE(Ctx *ctx, Token *tk, int lr)
 	switch(SP(tk)->tt) {
 		case TT_NAME:
 		{
-			knh_bytes_t t = knh_Token_tobytes(tk);
+			knh_bytes_t t = knh_Token_tobytes(ctx, tk);
 			if(ISB(t, "null")) {
 				knh_Token_setCONST(ctx, tk, KNH_NULL);
 				return TM(tk);
@@ -1179,7 +1180,7 @@ Term *new_TermVALUE(Ctx *ctx, Token *tk, int lr)
 		case TT_ERR:
 			return TM(tk);
 	}
-	DBG2_P("unexpected value: (tt=%s) %s", knh_token_tochar(SP(tk)->tt), knh_Token_tochar(tk));
+	DBG2_P("unexpected value: (tt=%s) %s", knh_token_tochar(SP(tk)->tt), sToken(tk));
 	knh_Token_perror(ctx, tk, KMSG_UTOKEN);
 	SP(tk)->tt = TT_ERR;
 	return TM(tk);
@@ -1212,6 +1213,7 @@ Stmt *new_StmtINSTMT(Ctx *ctx, Token *tk)
 #if defined(KNH_DBGMODE2)
 		if(prev == tc.c) { /* infinate loop */
 			DBG2_P("Infinate loop? prev = %d, c = %d, e = %d", prev, tc.c, tc.e);
+			KNH_ABORT();
 			break;
 		}
 #endif
@@ -1378,7 +1380,7 @@ Stmt *new_StmtOP(Ctx *ctx, knh_tokens_t *tc, Token *op)
 		{
 			Term *lval = new_TermEXPR(ctx, &ltc, KNH_LVALUE);
 			if(IS_Token(lval)) {
-				if(knh_Token_isLVALUE((Token*)lval)) {
+				if(knh_Token_isLVALUE(ctx, (Token*)lval)) {
 					Stmt *stmt = new_Stmt(ctx, 0, STT_LET);
 					knh_Stmt_add(ctx, stmt, lval);
 					knh_Stmt_add(ctx, stmt, TM(new_StmtNEXT(ctx, op, &rtc)));
@@ -1436,9 +1438,9 @@ Stmt *new_StmtCAST(Ctx *ctx, Token *tk_cast, knh_tokens_t *tc)
 
 /* ------------------------------------------------------------------------ */
 
-static int knh_Token_isNEW(Token *tk)
+static int knh_Token_isNEW(Ctx *ctx, Token *tk)
 {
-	char *p = knh_Token_tochar(tk);
+	char *p = sToken(tk);
 	return (p[0]=='n' && p[1] == 'e' && p[2] == 'w' && (p[3]==0 || p[3]==':'));
  }
 
@@ -1561,7 +1563,7 @@ static Term *new_TermEXPR(Ctx *ctx, knh_tokens_t *tc, int lr)
 	/** first **/
 	KNH_ASSERT(fc < e);
 	if(SP(ts[oc])->tt == TT_NAME || SP(ts[oc])->tt == TT_CMETHODN) {
-		if(knh_Token_isNEW(ts[oc])) {  /* @TEST new:gc Person("naruto") */
+		if(knh_Token_isNEW(ctx, ts[oc])) {  /* @TEST new:gc Person("naruto") */
 			if(DP(ts[oc])->tt_next == TT_TYPEN) {
 				if(DP(ts[oc+1])->tt_next == TT_PARENTHESIS) {
 					stmt = new_Stmt(ctx, 0, STT_NEW);
@@ -1598,10 +1600,10 @@ static Term *new_TermEXPR(Ctx *ctx, knh_tokens_t *tc, int lr)
 				knh_Token_perror(ctx, ts[oc+2], KMSG_ESYNTAX);
 				return TM(new_StmtERR(ctx, tc));
 			}
-			else if(DP(ts[oc])->tt_next == TT_NAME && ISB(knh_Token_tobytes(ts[oc+1]), "byte") && DP(ts[oc+1])->tt_next == TT_BRANCET) {
+			else if(DP(ts[oc])->tt_next == TT_NAME && ISB(knh_Token_tobytes(ctx, ts[oc+1]), "byte") && DP(ts[oc+1])->tt_next == TT_BRANCET) {
 				stmt = new_Stmt(ctx, 0, STT_NEW);
 				knh_Stmt_add(ctx, stmt, TM(new_TokenMN(ctx, FL(ts[oc]), METHODN_new)));
-				knh_Stmt_add(ctx, stmt, TM(new_Token__S(ctx, FL(ts[oc]), TT_TYPEN, knh_tClass[CLASS_Bytes].sname)));
+				knh_Stmt_add(ctx, stmt, TM(new_Token__S(ctx, FL(ts[oc]), TT_TYPEN, ctx->tClass[CLASS_Bytes].sname)));
 				pc = oc + 2;
 				KNH_ASSERT(SP(ts[pc])->tt == TT_BRANCET);
 				SP(ts[pc])->tt = TT_PARENTHESIS;
@@ -1744,9 +1746,9 @@ static Term *new_TermEXPR(Ctx *ctx, knh_tokens_t *tc, int lr)
 /* ======================================================================== */
 /* [LET] */
 
-static int knh_Token_isLCONSTN(Token *tk)
+static int knh_Token_isLCONSTN(Ctx *ctx, Token *tk)
 {
-	knh_bytes_t t = knh_Token_tobytes(tk);
+	knh_bytes_t t = knh_Token_tobytes(ctx, tk);
 	size_t i;
 	for(i = 0; i < t.len; i++) {
 		if(islower(t.buf[i])) return 0;
@@ -1756,9 +1758,9 @@ static int knh_Token_isLCONSTN(Token *tk)
 
 /* ------------------------------------------------------------------------ */
 
-static int knh_Token_isLVALUE(Token *tk)
+static int knh_Token_isLVALUE(Ctx *ctx, Token *tk)
 {
-	if(SP(tk)->tt == TT_TYPEN && knh_Token_isLCONSTN(tk)) {
+	if(SP(tk)->tt == TT_TYPEN && knh_Token_isLCONSTN(ctx, tk)) {
 		SP(tk)->tt = TT_CONSTN;
 	}
 	return knh_Token_isVARN(tk) || SP(tk)->tt == TT_CONSTN;
@@ -1808,7 +1810,7 @@ Stmt *new_StmtLET(Ctx *ctx, knh_tokens_t *lvalue_tc, knh_tokens_t *rvalue_tc)
 {
 	Term *lval = new_TermEXPR(ctx, lvalue_tc, KNH_LVALUE);
 	if(IS_Token(lval)) {
-		if(knh_Token_isLVALUE((Token*)lval)) {
+		if(knh_Token_isLVALUE(ctx, (Token*)lval)) {
 			Stmt *stmt = new_Stmt(ctx, 0, STT_LET);
 			knh_Stmt_add(ctx, stmt, lval);
 			knh_Stmt_add(ctx, stmt, new_TermEXPR(ctx, rvalue_tc, KNH_RVALUE));
@@ -1939,7 +1941,6 @@ Stmt* new_StmtLETEXPR(Ctx *ctx, knh_tokens_t *tc)
 		return new_Stmt(ctx, 0, STT_DONE);
 	}
 	else {
-
 		Term *term = new_TermEXPR(ctx, &expr_tc, KNH_RVALUE);
 		if(IS_Token(term)) {
 			Stmt *stmt = new_Stmt(ctx, 0, STT_CALL1);
@@ -1960,7 +1961,9 @@ void knh_StmtMETA_addLabel(Ctx *ctx, Stmt *o, Token *tk)
 	if(IS_NULL(DP(o)->metaDictMap)) {
 		KNH_SETv(ctx, DP(o)->metaDictMap, new_DictMap0(ctx, 2));
 	}
-	knh_DictMap_set(ctx, DP(o)->metaDictMap, TS_ATlabel, UP(tk));
+	if(IS_DictMap(DP(o)->metaDictMap)) {
+		knh_DictMap_set(ctx, DP(o)->metaDictMap, TS_ATlabel, UP(tk));
+	}
 }
 
 /* ------------------------------------------------------------------------ */
@@ -1971,8 +1974,10 @@ void knh_StmtMETA_addMeta(Ctx *ctx, Stmt *o, String *p)
 	if(IS_NULL(DP(o)->metaDictMap)) {
 		KNH_SETv(ctx, DP(o)->metaDictMap, new_DictMap0(ctx, 2));
 	}
-	KNH_ASSERT(IS_String(p));
-	knh_DictMap_set(ctx, DP(o)->metaDictMap, p, UP(p));
+	if(IS_DictMap(DP(o)->metaDictMap)) {
+		KNH_ASSERT(IS_String(p));
+		knh_DictMap_set(ctx, DP(o)->metaDictMap, p, UP(p));
+	}
 }
 
 /* ------------------------------------------------------------------------ */
@@ -1983,8 +1988,7 @@ void knh_StmtMETA_addMeta2(Ctx *ctx, Stmt *o, String *k, Token *p)
 	if(IS_NULL(DP(o)->metaDictMap)) {
 		KNH_SETv(ctx, DP(o)->metaDictMap, new_DictMap0(ctx, 2));
 	}
-	KNH_ASSERT(IS_String(k));
-	{
+	if(IS_DictMap(DP(o)->metaDictMap)) {
 		knh_tokens_t tc;
 		knh_Token_tc(p, &tc);
 		if(tc.e == 0) {
@@ -1995,7 +1999,7 @@ void knh_StmtMETA_addMeta2(Ctx *ctx, Stmt *o, String *k, Token *p)
 				knh_DictMap_set(ctx, DP(o)->metaDictMap, k, DP(tc.ts[0])->data);
 			}
 			else {
-				knh_DictMap_set(ctx, DP(o)->metaDictMap, k, UP(new_String(ctx, knh_Token_tobytes(tc.ts[0]), NULL)));
+				knh_DictMap_set(ctx, DP(o)->metaDictMap, k, UP(new_String(ctx, knh_Token_tobytes(ctx, tc.ts[0]), NULL)));
 			}
 		}
 		else {
@@ -2017,18 +2021,18 @@ void knh_Stmt_addMETA(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 	int i;
 	for(i = tc->meta; i < tc->c; i++) {
 		if(SP(ts[i])->tt == TT_LABEL) {
-			DEBUG3("LABEL!! %s", knh_Token_tochar(ts[i]));
+			DEBUG3("LABEL!! %s", sToken(ts[i]));
 			knh_StmtMETA_addLabel(ctx, o, ts[i]);
 			continue;
 		}
 		if(SP(ts[i])->tt == TT_METAN) {
 			if(DP(ts[i])->tt_next == TT_PARENTHESIS) {
-				DEBUG3("META2!! %s", knh_Token_tochar(ts[i-1]));
+				DEBUG3("META2!! %s", sToken(ts[i-1]));
 				knh_StmtMETA_addMeta2(ctx, o, (String*)DP(ts[i-1])->data, ts[i]);
 				i++;
 			}
 			else {
-				DEBUG3("META!! %s", knh_Token_tochar(ts[i]));
+				DEBUG3("META!! %s", sToken(ts[i]));
 				knh_StmtMETA_addMeta(ctx, o, (String*)DP(ts[i])->data);
 			}
 			continue;
@@ -2044,6 +2048,10 @@ Stmt *new_StmtMETA(Ctx *ctx,  knh_tokens_t *tc, knh_stmt_t stt)
 {
 	Stmt *o = new_Stmt(ctx, 0, stt);
 	knh_Stmt_addMETA(ctx, o, tc);
+	int c = tc->c - 1;
+	if(c < 0) c = 0;
+	SP(o)->line = SP(tc->ts[c])->line;
+	SP(o)->fileid = SP(tc->ts[c])->fileid;
 	return o;
 }
 
@@ -2051,9 +2059,6 @@ Stmt *new_StmtMETA(Ctx *ctx,  knh_tokens_t *tc, knh_stmt_t stt)
 /* ======================================================================== */
 /* [COMMONS] */
 
-
-/* ------------------------------------------------------------------------ */
-
 static void knh_Stmt_add_STMT1(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 {
 	if(SP(o)->stt == STT_ERR) return;
@@ -2089,7 +2094,7 @@ static int knh_Token_isNSNAME(Token *tk)
 {
 	if(SP(tk)->tt == TT_NAME) {
 		size_t i;
-		knh_bytes_t t = knh_Token_tobytes(tk);
+		knh_bytes_t t = knh_String_tobytes(DP(tk)->text);
 		for(i = 0; i < t.len; i++) {
 			if(islower(t.buf[i]) || isdigit(t.buf[i]) || t.buf[i] == '.') {
 				continue;
@@ -2286,7 +2291,7 @@ Stmt *new_StmtUSING(Ctx *ctx, knh_tokens_t *tc)
 {
 	if(tc->c < tc->e) {
 		Token *tk = tc->ts[tc->c];
-		knh_bytes_t op = knh_Token_tobytes(tk);
+		knh_bytes_t op = knh_Token_tobytes(ctx, tk);
 
 		if(ISB(op, "import")) {
 			tc->c += 1;
@@ -2309,7 +2314,7 @@ Stmt *new_StmtUSING(Ctx *ctx, knh_tokens_t *tc)
 		}
 
 		/* using naruto.* */
-		if(knh_Token_isNSCLASSN(tk)) {
+		if(knh_Token_isNSCLASSN(ctx, tk)) {
 			return new_StmtUIMPORT(ctx, tc);
 		}
 
@@ -2319,7 +2324,7 @@ Stmt *new_StmtUSING(Ctx *ctx, knh_tokens_t *tc)
 		}
 
 		/* using Int:ns */
-		if(knh_Token_isCLASSTN(tk)) {
+		if(knh_Token_isCLASSTN(ctx, tk)) {
 			if(knh_bytes_startsWith(op, STEXT("Int:"))) {
 				return new_StmtUENUM(ctx, tc);
 			}
@@ -2367,7 +2372,7 @@ static void knh_Stmt_add_EXTENDS(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 		knh_Stmt_add_CLASSN(ctx, o, tc);
 	}
 	else {
-		knh_Stmt_add_S(ctx, o, TT_TYPEN, knh_tClass[CLASS_Object].sname);
+		knh_Stmt_add_S(ctx, o, TT_TYPEN, ctx->tClass[CLASS_Object].sname);
 	}
 	if(tc->c < tc->e && SP(tc->ts[tc->c])->tt == TT_IMPLEMENTS) {
 		tc->c += 1;
@@ -2582,6 +2587,7 @@ knh_Stmt_add_PEACH(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 	if(SP(o)->stt == STT_ERR) return;
 	if(tc->c < tc->e && SP(tk)->tt != TT_PARENTHESIS) {
 		knh_Stmt_tokens_perror(ctx, o, tc, KMSG_ESYNTAX);
+		return;
 	}
 
 	knh_tokens_t ptc;
@@ -2590,7 +2596,7 @@ knh_Stmt_add_PEACH(Ctx *ctx, Stmt *o, knh_tokens_t *tc)
 	knh_Token_tc(tc->ts[tc->c], &ptc);
 	for(i = 1; i < ptc.e; i++) {
 		if(SP(ptc.ts[i])->tt == TT_FROM || SP(ptc.ts[i])->tt == TT_COLON
-			|| (i > 0 && SP(ptc.ts[i])->tt == TT_NAME && ISB(knh_Token_tobytes(ptc.ts[i]), "in"))) {
+			|| (i > 0 && SP(ptc.ts[i])->tt == TT_NAME && ISB(knh_Token_tobytes(ctx, ptc.ts[i]), "in"))) {
 			val_tc.c = 0;
 			val_tc.e = i;
 			val_tc.ts = ptc.ts;
@@ -3011,8 +3017,7 @@ Stmt* new_StmtFUNCEXPR(Ctx *ctx, knh_tokens_t *tc)
 /* ------------------------------------------------------------------------ */
 /* [pragma] */
 
-static
-void konohac_pragma(Ctx *ctx, knh_tokens_t *tc)
+static void konohac_pragma(Ctx *ctx, knh_tokens_t *tc)
 {
 	TODO();
 	knh_tokens_nextStmt(tc);
