@@ -407,7 +407,7 @@ void knh_Token_addTypeParam(Ctx *ctx, Token *tkt, Array *a, size_t s)
 
 	L_PASS2:;
 	knh_wbuf_t wbuf = knh_Context_wbuf(ctx);
-	knh_write(ctx, wbuf.w, knh_Token_tobytes(tkt));
+	knh_write(ctx, wbuf.w, knh_Token_tobytes(ctx, tkt));
 	knh_putc(ctx, wbuf.w, '<');
 	knh_Array_remove(ctx, a, s);
 	i = s;
@@ -427,7 +427,7 @@ void knh_Token_addTypeParam(Ctx *ctx, Token *tkt, Array *a, size_t s)
 			}
 			break;
 		}
-		knh_write(ctx, wbuf.w, knh_Token_tobytes(tk));
+		knh_write(ctx, wbuf.w, knh_Token_tobytes(ctx, tk));
 		knh_Array_remove(ctx, a, i);
 	}
 	knh_putc(ctx, wbuf.w, '>');
@@ -464,11 +464,11 @@ void knh_Token_join(Ctx *ctx, Token *o)
 					}
 				}
 			}
-			if(SP(tk)->tt == TT_NAME && ISB(knh_Token_tobytes(tk), "byte")) {
+			if(SP(tk)->tt == TT_NAME && ISB(knh_Token_tobytes(ctx, tk), "byte")) {
 				Token *tkn = (Token*)knh_Array_n(a, i+1);
 				if(SP(tkn)->tt == TT_BRANCET && IS_NULL(DP(tkn)->data)) {
 					SP(tk)->tt = TT_TYPEN;
-					KNH_SETv(ctx, DP(tk)->data, knh_tClass[CLASS_Bytes].sname);
+					KNH_SETv(ctx, DP(tk)->data, ctx->tClass[CLASS_Bytes].sname);
 					if(i>0) {
 						Token *tkp = (Token*)knh_Array_n(a, i-1);
 						DP(tkp)->tt_next = TT_TYPEN;
