@@ -49,7 +49,7 @@ static
 knh_fieldn_t knh_Token_getfnq(Ctx *ctx, Token *o)
 {
 	if(SP(o)->tt == TT_NAME) {
-		return knh_tName_get_fnq(ctx, knh_Token_tobytes(o), FIELDN_NEWID);
+		return knh_tName_get_fnq(ctx, knh_Token_tobytes(ctx, o), FIELDN_NEWID);
 	}
 	else if(SP(o)->tt == TT_FN) {
 		return DP(o)->fn;
@@ -79,7 +79,7 @@ knh_methodn_t knh_Token_getmn(Ctx *ctx, Token *o)
 		return mn;
 	}
 
-	knh_bytes_t name = knh_Token_tobytes(o);
+	knh_bytes_t name = knh_Token_tobytes(ctx, o);
 	if(SP(o)->tt == TT_NAME) {
 		knh_methodn_t mn = knh_tName_getMethodn(ctx, name, METHODN_NEWID);
 		if(knh_Token_isGetter(o)) {
@@ -119,7 +119,7 @@ knh_type_t knh_Token_gettype(Ctx *ctx, Token *o, NameSpace *ns, knh_class_t defc
 		return NNTYPE_cid(DP(o)->cid);
 	}
 	else {
-		knh_bytes_t name = knh_Token_tobytes(o);
+		knh_bytes_t name = knh_Token_tobytes(ctx, o);
 		knh_type_t cid;
 		if(ISB(name, "void")) {
 			return TYPE_void;
@@ -163,7 +163,7 @@ knh_class_t knh_Token_getcid(Ctx *ctx, Token *o, NameSpace *ns, knh_class_t defc
 		if(knh_Token_isExceptionType(o)) {
 			return CLASS_Exception;
 		}
-		knh_bytes_t name = knh_Token_tobytes(o);
+		knh_bytes_t name = knh_Token_tobytes(ctx, o);
 		knh_class_t cid = knh_NameSpace_getClass(ctx, ns, name);
 		if(cid == CLASS_unknown) {
 
@@ -195,7 +195,7 @@ knh_class_t knh_Token_getcid(Ctx *ctx, Token *o, NameSpace *ns, knh_class_t defc
 void knh_Asm_perrorMTD(Ctx *ctx, Asm *abr, int pe, knh_class_t cid, knh_methodn_t mn)
 {
 	char bufcm[CLASSNAME_BUFSIZ];
-	knh_format_cmethodn(bufcm, sizeof(bufcm), cid, mn);
+	knh_format_cmethodn(ctx, bufcm, sizeof(bufcm), cid, mn);
 	knh_Asm_perror(ctx, abr, pe, bufcm);
 }
 
@@ -215,8 +215,7 @@ void knh_Asm_perrorMPR(Ctx *ctx, Asm *abr, int pe, knh_class_t scid, knh_class_t
 
 Token* new_TokenCONST(Ctx *ctx, Any *fln, Any *data)
 {
-	knh_Token_t *o = (Token*)new_Object_malloc(ctx, FLAG_Token, CLASS_Token, sizeof(knh_Token_struct));
-	knh_Token_struct_init(ctx, DP(o), 0, NULL);
+	knh_Token_t *o = (Token*)new_Object_bcid(ctx, CLASS_Token, 0);
 	knh_Token_setFL(o, fln);
 	knh_Token_setCONST(ctx, o, data);
 	return o;
@@ -276,7 +275,7 @@ Token* knh_Token_toDEFVAL(Token *o, knh_class_t cid)
 static
 int knh_Token_toSYSVAL(Ctx *ctx, Token *tk, Asm *abr)
 {
-	knh_bytes_t t = knh_Token_tobytes(tk);
+	knh_bytes_t t = knh_Token_tobytes(ctx, tk);
 	if(IS_SYSVAL(t, "CTX")) {
 		SP(tk)->tt = TT_SYSVAL;
 		DP(tk)->index = KNH_SYS_CTX; DP(tk)->type = NNTYPE_Context;
@@ -312,7 +311,7 @@ int knh_Token_toSYSVAL(Ctx *ctx, Token *tk, Asm *abr)
 		return 1;
 	}
 	else if(IS_SYSVAL(t, "__file__")) {
-		knh_Token_setCONST(ctx, tk, UP(knh_tfileid_name(SP(tk)->fileid)));
+		knh_Token_setCONST(ctx, tk, UP(knh_tfileid_name(ctx, SP(tk)->fileid)));
 		return 1;
 	}
 	else if(IS_SYSVAL(t, "__method__") || IS_SYSVAL(t, "__function__")) {
@@ -422,7 +421,7 @@ int knh_Asm_declareScriptVariable(Ctx *ctx, Asm *abr, knh_flag_t flag, knh_type_
 {
 	knh_Script_t *scr = knh_Asm_getScript(ctx, abr);
 	knh_class_t cid = knh_Object_cid(scr);
-	knh_cfield_t *cf = DP(knh_tClass[cid].cstruct)->fields;
+	knh_cfield_t *cf = (ctx->tClass[cid].cstruct)->fields;
 	knh_index_t idx = knh_Asm_addVariableTable(ctx, abr, cf, KNH_SCRIPT_FIELDSIZE, flag, type, fn, value);
 	if(idx != -1) {
 		KNH_SETv(ctx, scr->fields[idx], value);
@@ -463,10 +462,10 @@ knh_index_t knh_Asm_indexOfVariable(Asm *abr, knh_fieldn_t fnq)
 }
 
 #define knh_Asm_indexOfScriptVariable(abr, fnq)\
-	knh_Class_queryField(knh_Object_cid(knh_Asm_getScript(ctx, abr)), fnq)
+	knh_Class_queryField(ctx, knh_Object_cid(knh_Asm_getScript(ctx, abr)), fnq)
 
 #define knh_Asm_indexOfFieldVariable(abr, fnq)\
-	knh_Class_queryField(DP(abr)->this_cid, fnq)
+	knh_Class_queryField(ctx, DP(abr)->this_cid, fnq)
 
 /* ------------------------------------------------------------------------ */
 
@@ -512,9 +511,9 @@ static int knh_TokenNAME_typing(Ctx *ctx, Token *tk, Asm *abr)
 	}
 
 	L_FIELD:;
-	idx = knh_Class_queryField(DP(abr)->this_cid, fnq);
+	idx = knh_Class_queryField(ctx, DP(abr)->this_cid, fnq);
 	if(idx != -1) {
-		knh_cfield_t *cf = knh_Class_fieldAt(DP(abr)->this_cid, idx);
+		knh_cfield_t *cf = knh_Class_fieldAt(ctx, DP(abr)->this_cid, idx);
 		knh_type_t type = knh_pmztype_totype(ctx, cf->type, DP(abr)->this_cid);
 		knh_Token_toFIELD(ctx, tk, idx, type);
 		return 1;
@@ -524,9 +523,9 @@ static int knh_TokenNAME_typing(Ctx *ctx, Token *tk, Asm *abr)
 	L_GLOBAL:;
 	{
 		Script *scr = knh_Asm_getScript(ctx, abr);
-		idx = knh_Class_queryField(knh_Object_cid(scr), fnq);
+		idx = knh_Class_queryField(ctx, knh_Object_cid(scr), fnq);
 		if(idx != -1) {
-			knh_cfield_t *cf = knh_Class_fieldAt(knh_Object_cid(scr), idx);
+			knh_cfield_t *cf = knh_Class_fieldAt(ctx, knh_Object_cid(scr), idx);
 			knh_type_t type = knh_pmztype_totype(ctx, cf->type, knh_Object_cid(scr));
 			knh_Token_toGLOBAL(ctx, tk, idx, type);
 			return knh_Asm_globalIndex(ctx, abr);
@@ -546,7 +545,7 @@ static int knh_TokenNAME_typing(Ctx *ctx, Token *tk, Asm *abr)
 static
 int knh_TokenCONSTN_typing(Ctx *ctx, Token *o, Asm *abr, NameSpace *ns)
 {
-	knh_bytes_t t = knh_Token_tobytes(o);
+	knh_bytes_t t = knh_Token_tobytes(ctx, o);
 	knh_index_t loc = knh_bytes_rindex(t, '.');
 	if(loc == -1) {
 		if(knh_Token_toSYSVAL(ctx, o, abr)) {
@@ -594,7 +593,7 @@ int knh_TokenTYEPN_typing(Ctx *ctx, Token *o, Asm *abr, NameSpace *ns)
 		if(knh_Token_toSYSVAL(ctx, o, abr)) {
 			return 1;
 		}
-		knh_bytes_t t = knh_Token_tobytes(o);
+		knh_bytes_t t = knh_Token_tobytes(ctx, o);
 		if(knh_NameSpace_existsConst(ctx, ns, t)) {
 			knh_Token_setCONST(ctx, o, knh_NameSpace_getConst(ctx, ns, t));
 			return 1;
@@ -606,7 +605,7 @@ int knh_TokenTYEPN_typing(Ctx *ctx, Token *o, Asm *abr, NameSpace *ns)
 	SP(o)->tt = TT_CLASSID;
 	DP(o)->cid = cid;
 	DP(o)->type = NNTYPE_Class;
-	KNH_SETv(ctx, DP(o)->data, knh_tClass[cid].class_cid);
+	KNH_SETv(ctx, DP(o)->data, ctx->tClass[cid].class_cid);
 	return 1;
 }
 
@@ -615,7 +614,7 @@ int knh_TokenTYEPN_typing(Ctx *ctx, Token *o, Asm *abr, NameSpace *ns)
 static
 int knh_TokenCMETHODN_typing(Ctx *ctx, Token *o, NameSpace *ns)
 {
-	knh_bytes_t t = knh_Token_tobytes(o);
+	knh_bytes_t t = knh_Token_tobytes(ctx, o);
 	knh_index_t idx = knh_bytes_index(t, '.');
 	if(idx == -1) return 0;
 
@@ -643,7 +642,7 @@ int knh_TokenCMETHODN_typing(Ctx *ctx, Token *o, NameSpace *ns)
 static
 knh_class_t knh_Token_tagcNUM(Ctx *ctx, Token *o, knh_class_t reqc, NameSpace *ns)
 {
-	knh_bytes_t t = knh_Token_tobytes(o), tag = STEXT("");
+	knh_bytes_t t = knh_Token_tobytes(ctx, o), tag = STEXT("");
 	size_t i = 1;
 	int ishex = 0;
 	if(t.buf[0] == '0' && (t.buf[1] == 'x' || t.buf[1] == 'b')) {
@@ -728,7 +727,7 @@ static
 int knh_TokenNUM_typing(Ctx *ctx, Token *o, NameSpace *ns, knh_class_t reqc)
 {
 	KNH_ASSERT_cid(reqc);
-	knh_bytes_t t = knh_Token_tobytes(o);
+	knh_bytes_t t = knh_Token_tobytes(ctx, o);
 
 	if(reqc == CLASS_Boolean) {
 		knh_perror(ctx, SP(o)->fileid, SP(o)->line, KMSG_WTRUEFALSE, NULL);
@@ -743,7 +742,7 @@ int knh_TokenNUM_typing(Ctx *ctx, Token *o, NameSpace *ns, knh_class_t reqc)
 		return 1;
 	}
 
-	knh_class_t req_bcid = knh_tClass[reqc].bcid;
+	knh_class_t req_bcid = ctx->tClass[reqc].bcid;
 	if(req_bcid != CLASS_Int && req_bcid != CLASS_Float) {
 		req_bcid = knh_bytes_guessNUMcid(ctx, t);
 		reqc = req_bcid;
@@ -783,7 +782,7 @@ int knh_TokenNUM_typing(Ctx *ctx, Token *o, NameSpace *ns, knh_class_t reqc)
 static
 void knh_TokenTSTR_typing(Ctx *ctx, Token *o, NameSpace *ns, knh_class_t reqt)
 {
-	knh_bytes_t t = knh_Token_tobytes(o);
+	knh_bytes_t t = knh_Token_tobytes(ctx, o);
 	knh_index_t loc = knh_bytes_index(t, ':');
 	if(loc <= -1) {
 		KNH_ASSERT(IS_String(DP(o)->data));
@@ -832,7 +831,7 @@ int knh_Token_typing(Ctx *ctx, Token *tk, Asm *abr, NameSpace *ns, knh_type_t re
 	case TT_STR:
 		KNH_ASSERT(IS_String(DP(tk)->data));
 		knh_Token_toCONST(tk);
-//		if(knh_tClass[reqc].bcid == CLASS_String) {
+//		if(ctx->tClass[reqc].bcid == CLASS_String) {
 //			DP(o)->type = reqc;
 //		}
 		return 1;
@@ -913,7 +912,7 @@ static Object *TERMs_value(Ctx *ctx, Stmt *stmt, size_t n, knh_type_t type)
 		if(SP(tk)->tt == TT_CONST) return DP(tk)->data;
 		if(SP(tk)->tt == TT_ASIS) {
 			return IS_NNTYPE(type) ?
-					knh_tClass_defaultValueNN(ctx, CLASS_type(type)) : KNH_NULL;
+					konoha_getClassDefaultValue(ctx, CLASS_type(type)) : KNH_NULL;
 		}
 	}
 	return KNH_NULL;
@@ -938,7 +937,7 @@ static int TERMs_isCLASSID(Stmt *stmt, size_t n)
 /* ------------------------------------------------------------------------ */
 
 #define _TERMs_getcid(stmt, n)    CLASS_type(TERMs_gettype(stmt, n))
-#define _TERMs_getbcid(stmt, n)   knh_tClass[TERMs_getcid(stmt,n)].bcid
+#define _TERMs_getbcid(stmt, n)   ctx->tClass[TERMs_getcid(stmt,n)].bcid
 
 knh_type_t TERMs_gettype(Stmt *stmt, size_t n)
 {
@@ -976,7 +975,7 @@ char * knh_format_mtdparam(Ctx *ctx, char *buf, size_t bufsiz, Method *mtd, int
 		if(islower(buf[3])) buf[3] = toupper(buf[3]);
 		return buf;
 	}
-	if(knh_methodn_isNew(mn)) {
+	if(knh_methodn_isNew(ctx, mn)) {
 		knh_snprintf(buf, bufsiz, "%s %s(%s)", FIELDN(mn), CTXCLASSN(DP(mtd)->cid), bufa);
 	}
 	else {
@@ -988,7 +987,7 @@ char * knh_format_mtdparam(Ctx *ctx, char *buf, size_t bufsiz, Method *mtd, int
 /* ------------------------------------------------------------------------ */
 
 static
-char * knh_format_op(char *buf, size_t bufsiz, knh_methodn_t mn, int n)
+char * knh_format_op(Ctx *ctx, char *buf, size_t bufsiz, knh_methodn_t mn, int n)
 {
 	char *op;
 	switch(mn) {
@@ -1025,13 +1024,24 @@ void TERMs_perrorTYPE(Ctx *ctx, Stmt *stmt, size_t n, int pe, knh_type_t reqt)
 		{
 			Method *mtd = DP(DP(stmt)->tokens[0])->mtd;
 			DEBUG3_ASSERT(IS_Method(mtd));
-			knh_format_op(buf, sizeof(buf), DP(mtd)->mn, n - 1);
+			knh_format_op(ctx, buf, sizeof(buf), DP(mtd)->mn, n - 1);
 		}
 	case STT_LET:
 		at = knh_message_text(KMSG_ASSIGNMENT);
 		break;
+	case STT_FOREACH:
+		at = "foreach";
+		if(n == 2) {
+			if(knh_Stmt_isMAPNEXT(stmt)) at = "from ..";
+			else at = "in ..";
+		}
+		else if(n == 3) {
+			at = "where ..";
+		}
+		break;
 	default :
 		at = knh_stmt_tochar(SP(stmt)->stt);
+		DBG2_P("%s n=%d", at, (int)n);
 	}
 	knh_perror__s(ctx, SP(stmt)->fileid, SP(stmt)->line, pe, at);
 }
@@ -1073,11 +1083,11 @@ knh_bool_t knh_Asm_existsName(Ctx *ctx, Asm *abr, knh_fieldn_t fnq)
 	if(idx != -1) return 1;
 
 	L_FIELD:;
-	idx = knh_Class_queryField(DP(abr)->this_cid, fnq);
+	idx = knh_Class_queryField(ctx, DP(abr)->this_cid, fnq);
 	if(idx != -1) return 1;
 
 	L_GLOBAL:;
-	idx = knh_Class_queryField(knh_Object_cid(knh_Asm_getScript(ctx, abr)), fnq);
+	idx = knh_Class_queryField(ctx, knh_Object_cid(knh_Asm_getScript(ctx, abr)), fnq);
 	if(idx != -1) return 1;
 	return 0;
 }
@@ -1087,7 +1097,7 @@ knh_bool_t knh_Asm_existsName(Ctx *ctx, Asm *abr, knh_fieldn_t fnq)
 static
 Object *knh_StmtDECL_value(Ctx *ctx, knh_type_t type)
 {
-	return IS_NNTYPE(type) ? knh_tClass_defaultValueNN(ctx, CLASS_type(type)) : KNH_NULL;
+	return IS_NNTYPE(type) ? konoha_getClassDefaultValue(ctx, CLASS_type(type)) : KNH_NULL;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -1167,7 +1177,7 @@ Term * knh_StmtDECL_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 	}
 	else {
 		if(FIELDN_IS_U2(fnq) || FIELDN_IS_U1(fnq)) {
-			knh_Asm_perror(ctx, abr, KMSG_IGQNAME, knh_Token_tochar(StmtDECL_name(stmt)));
+			knh_Asm_perror(ctx, abr, KMSG_IGQNAME, sToken(StmtDECL_name(stmt)));
 		}
 		Object *value = knh_StmtDECL_value(ctx, type);
 		if(TERMs_isASIS(stmt, 2)) {
@@ -1200,7 +1210,7 @@ int TERMs_isCONSTNAME(Ctx *ctx, Stmt *stmt, size_t n, Asm *abr, NameSpace *ns)
 		return 1;
 	}
 	if(SP(tk)->tt == TT_TYPEN) {
-		knh_bytes_t cname = knh_Token_tobytes(tk);
+		knh_bytes_t cname = knh_Token_tobytes(ctx, tk);
 		size_t i;
 		for(i = 0; i < cname.len; i++) {
 			if(islower(cname.buf[i])) return 0;
@@ -1217,6 +1227,7 @@ static
 Term *knh_StmtLET_declCONST(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, int level)
 {
 	Token *tk = DP(stmt)->tokens[0];
+	knh_bytes_t cn = knh_Token_tobytes(ctx, tk);
 	int pe = KMSG_ESYNTAX;
 	if(level > 1) {
 		pe = KMSG_NOTHERE; goto L_PERROR;
@@ -1231,7 +1242,6 @@ Term *knh_StmtLET_declCONST(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, int l
 	}
 
 	Object *value = TERMs_const(stmt, 1);
-	knh_bytes_t cn = knh_Token_tobytes(tk);
 	knh_index_t dotidx = knh_bytes_index(cn, '.');
 
 	if(level == 0) {
@@ -1384,6 +1394,17 @@ Term *knh_StmtCALL1_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_ty
 	}
 }
 
+/* ------------------------------------------------------------------------ */
+
+static
+knh_Class_t *new_Class__type(Ctx *ctx, knh_type_t type)
+{
+	knh_Class_t *o = (knh_Class_t*)new_hObject(ctx, FLAG_Class, CLASS_Class, CLASS_Class);
+	o->type = type;
+	o->cid = CLASS_type(type);
+	return o;
+}
+
 /* ------------------------------------------------------------------------ */
 /* [CALL] */
 
@@ -1439,7 +1460,7 @@ Term *knh_StmtCALLBASE_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh
 	}
 	else {
 		char bufmn[CLASSNAME_BUFSIZ];
-		knh_format_methodn(bufmn, sizeof(bufmn), mn);
+		knh_format_methodn(ctx, bufmn, sizeof(bufmn), mn);
 
 		//DEBUG3("1. lookup function.. %s()", bufmn);
 		knh_class_t mtd_cid = knh_NameSpace_getFuncClass(ctx, ns, B(bufmn));
@@ -1472,7 +1493,7 @@ Term *knh_StmtCALLBASE_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh
 	}
 
 	L_ERROR:;
-	knh_Asm_perror(ctx, abr, pe, knh_Token_tochar(DP(stmt)->tokens[0]));
+	knh_Asm_perror(ctx, abr, pe, sToken(DP(stmt)->tokens[0]));
 	return NULL;
 }
 
@@ -1483,7 +1504,7 @@ void KNH_BOX(Ctx *ctx, knh_sfp_t *sfp, knh_type_t type)
 {
 	knh_class_t cid = CLASS_type(type);
 	KNH_ASSERT_cid(cid);
-	knh_class_t bcid = knh_tClass[cid].bcid;
+	knh_class_t bcid = ctx->tClass[cid].bcid;
 	if(bcid == CLASS_Int || bcid == CLASS_Float || bcid == CLASS_Boolean) {
 		if(IS_NNTYPE(type) || IS_NOTNULL(sfp[0].o)) {
 			KNH_SETv(ctx, sfp[0].o, knh_boxing(ctx, sfp, cid));
@@ -1522,7 +1543,7 @@ static Term *knh_StmtCALL_toCONST(Ctx *ctx, Stmt *stmt, Method *mtd)
 	}
 	DEBUG3("STMT TO CONST ..");
 	KNH_SCALL(ctx, lsfp, 0, mtd, DP(stmt)->size);
-	KNH_BOX(ctx, &lsfp[0], knh_Method_rtype(mtd));
+	KNH_BOX(ctx, &lsfp[0], knh_Method_rztype(mtd));
 	knh_Token_setCONST(ctx, DP(stmt)->tokens[0], lsfp[0].o);
 	return DP(stmt)->terms[0];
 }
@@ -1534,11 +1555,11 @@ Term *knh_StmtPARAMS_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_c
 {
 	KNH_ASSERT(IS_Method(mtd));
 	size_t i, size = DP(stmt)->size;
-	knh_type_t rtype = knh_pmztype_totype(ctx, knh_Method_rtype(mtd), mtd_cid);
+	knh_type_t rtype = knh_pmztype_totype(ctx, knh_Method_rztype(mtd), mtd_cid);
 	knh_Stmt_setType(ctx, stmt, rtype);
 
 	for(i = 0; i < knh_Method_psize(mtd); i++) {
-		knh_type_t reqt = knh_pmztype_totype(ctx, knh_Method_ptype(mtd, i), mtd_cid);
+		knh_type_t reqt = knh_pmztype_totype(ctx, knh_Method_pztype(mtd, i), mtd_cid);
 		size_t n = i + 2; //DEBUG3("reqt[%d]=%d,%s%s", n, reqt, TYPEQN(reqt));
 		if(n < size) {
 			if(!TERMs_typing(ctx, stmt, n, abr, ns, reqt, TCHECK_)) {
@@ -1558,7 +1579,7 @@ Term *knh_StmtPARAMS_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_c
 		}
 	}
 	if(knh_Method_isVarArgs(mtd)) {
-		knh_type_t reqt = knh_pmztype_totype(ctx, knh_Method_ptype(mtd, knh_Method_psize(mtd) - 1), mtd_cid);
+		knh_type_t reqt = knh_pmztype_totype(ctx, knh_Method_pztype(mtd, knh_Method_psize(mtd) - 1), mtd_cid);
 		for(i = knh_Method_psize(mtd); i + 2 < size; i++) {
 			if(!TERMs_typing(ctx, stmt, i + 2, abr, ns, reqt, TCHECK_)) {
 				return NULL;
@@ -1635,14 +1656,14 @@ Term *knh_StmtNEW_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_clas
 		Token *mtk = new_TokenCONST(ctx, FL(stmt), DP(DP(stmt)->tokens[1])->data);
 		mn = METHODN_new__init;
 		if(DP(stmt)->size == 2) {
-			knh_Stmt_add(ctx, stmt, mtk);
+			knh_Stmt_add(ctx, stmt, TM(mtk));
 		}
 		else if(DP(stmt)->size == 3) {
-			knh_Stmt_add(ctx, stmt, DP(stmt)->tokens[2]);
+			knh_Stmt_add(ctx, stmt, DP(stmt)->terms[2]);
 			KNH_SETv(ctx, DP(stmt)->terms[2], mtk);
 		}
 		else if(DP(stmt)->size == 4) {
-			knh_Stmt_add(ctx, stmt, DP(stmt)->tokens[3]);
+			knh_Stmt_add(ctx, stmt, DP(stmt)->terms[3]);
 			KNH_SETv(ctx, DP(stmt)->terms[3], DP(stmt)->terms[2]);
 			KNH_SETv(ctx, DP(stmt)->terms[2], mtk);
 		}
@@ -1664,7 +1685,7 @@ Term *knh_StmtNEW_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_clas
 
 	if(mtd_cid == CLASS_Array) {
 		knh_class_t reqc = CLASS_type(reqt); KNH_ASSERT_cid(reqc);
-		if(knh_tClass[reqc].bcid == CLASS_Array) {
+		if(ctx->tClass[reqc].bcid == CLASS_Array) {
 			mtd_cid = reqc;
 		}
 		else if(reqc == CLASS_Any && mn == METHODN_new__init && DP(stmt)->size > 2) {
@@ -1694,7 +1715,7 @@ Term *knh_StmtNEW_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_clas
 	Method *mtd = knh_Class_getMethod(ctx, mtd_cid, mn);
 	if(IS_NULL(mtd)) {
 		char bufnw[CLASSNAME_BUFSIZ];
-		knh_snprintf(bufnw, sizeof(bufnw), "%s %s(...)", knh_Token_tochar(DP(stmt)->tokens[0]), CLASSN(mtd_cid));
+		knh_snprintf(bufnw, sizeof(bufnw), "%s %s(...)", sToken(DP(stmt)->tokens[0]), CLASSN(mtd_cid));
 		knh_Asm_perror(ctx, abr, KMSG_UNEW, bufnw);
 		return NULL;
 	}
@@ -1740,8 +1761,8 @@ knh_class_t knh_StmtOPADD_basecid(Ctx *ctx, Stmt *stmt)
 	if(cid1 == CLASS_Any || cid2 == CLASS_Any) return CLASS_Any;
 	if(cid1 == cid2) return cid1;
 
-	knh_class_t bcid1 = knh_tClass[cid1].bcid;
-	knh_class_t bcid2 = knh_tClass[cid2].bcid;
+	knh_class_t bcid1 = ctx->tClass[cid1].bcid;
+	knh_class_t bcid2 = ctx->tClass[cid2].bcid;
 
 	if(bcid1 == bcid2) return bcid1;
 
@@ -1765,8 +1786,8 @@ knh_class_t knh_StmtOPEQ_basecid(Ctx *ctx, Stmt *stmt)
 	if(cid1 == cid2) return cid1;
 	if(cid1 == CLASS_Any || cid2 == CLASS_Any) return CLASS_Any;
 
-	knh_class_t bcid1 = knh_tClass[cid1].bcid;
-	knh_class_t bcid2 = knh_tClass[cid2].bcid;
+	knh_class_t bcid1 = ctx->tClass[cid1].bcid;
+	knh_class_t bcid2 = ctx->tClass[cid2].bcid;
 
 	if(bcid1 == cid2) return bcid1;
 	if(bcid2 == cid1) return bcid2;
@@ -1973,7 +1994,7 @@ Term *knh_StmtMAPCAST_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_
 
 	if(TERMs_isNULL(stmt, 1)) {
 		if(knh_Token_isNotNullType(tk)) {
-			knh_Token_setCONST(ctx, DP(stmt)->tokens[1], knh_tClass_defaultValueNN(ctx, mprcid));
+			knh_Token_setCONST(ctx, DP(stmt)->tokens[1], konoha_getClassDefaultValue(ctx, mprcid));
 		}
 		return DP(stmt)->terms[1];
 	}
@@ -1987,7 +2008,7 @@ Term *knh_StmtMAPCAST_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_
 		return TM(stmt);
 	}
 
-	if(mprcid == exprc || knh_class_instanceof(mprcid, exprc)) {
+	if(mprcid == exprc || knh_class_instanceof(ctx, mprcid, exprc)) {
 		if(knh_Token_isNotNullType(tk)) {
 			DEBUG3("Nonnull cast");
 			if(!IS_NNTYPE(exprt)) {
@@ -2033,7 +2054,7 @@ Term *knh_StmtMT_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_type_
 	if(!TERMs_typing(ctx, stmt, 1, abr, ns, CLASS_Any, TCHECK_)) {
 		return NULL;
 	}
-	if(knh_bytes_isOptionalMT(knh_Token_tobytes(tk))) {
+	if(knh_bytes_isOptionalMT(knh_Token_tobytes(ctx, tk))) {
 		KNH_ASSERT(DP(stmt)->size == 2);
 		KNH_ASSERT(IS_String(DP(tk)->data));
 		knh_Stmt_add(ctx, stmt, TM(new_TokenCONST(ctx, FL(tk), DP(tk)->data)));
@@ -2226,7 +2247,9 @@ int TERMs_typecheck(Ctx *ctx, Stmt *stmt, size_t n, Asm *abr, knh_type_t reqt, i
 		return 0;
 	}
 
-	if(vart == reqt || reqc == CLASS_Any || knh_class_instanceof(varc, reqc)) return 1;
+	if(vart == reqt || reqc == CLASS_Any || knh_class_instanceof(ctx, varc, reqc)) return 1;
+
+	if(mode == TITERCONV_ && ctx->tClass[varc].bcid == CLASS_Iterator) return 1;
 
 	if(varc == CLASS_Any ||
 		(varc == CLASS_Float && reqc == CLASS_Int) ||
@@ -2234,12 +2257,13 @@ int TERMs_typecheck(Ctx *ctx, Stmt *stmt, size_t n, Asm *abr, knh_type_t reqt, i
 		mode = TCONV_;
 	}
 
-	if(mode == TCONV_) {
+	if(mode == TCONV_ || mode == TITERCONV_) {
 		Term *mcast = new_TermINCAST(ctx, reqc, stmt, n);
 		if(mcast != NULL) {
 			KNH_SETv(ctx, DP(stmt)->stmts[n], mcast);
 			return 1;
 		}
+		if(mode == TITERCONV_) return 0;
 	}
 	TERMs_perrorTYPE(ctx, stmt, n, KMSG_ETYPEFMT, reqt);
 	return 0;
@@ -2397,10 +2421,10 @@ knh_fieldn_t knh_tName_local(Ctx *ctx, const char *fmt, int level)
 	char bufn[40];
 	knh_snprintf(bufn, sizeof(bufn), fmt, level);
 	knh_bytes_t tname = B(bufn);
-	knh_index_t idx = knh_DictIdx_index(ctx, DP(knh_rootSystem)->tfieldnDictIdx, tname);
+	knh_index_t idx = knh_DictIdx_index(ctx, DP(ctx->sys)->tfieldnDictIdx, tname);
 	if(idx == -1) {
 		String *s = new_String(ctx, tname, NULL);
-		idx = knh_DictIdx_add__fast(ctx, DP(knh_rootSystem)->tfieldnDictIdx, s);
+		idx = knh_DictIdx_add__fast(ctx, DP(ctx->sys)->tfieldnDictIdx, s);
 	}
 	return (knh_fieldn_t)idx;
 }
@@ -2414,7 +2438,7 @@ Term *knh_StmtFOREACH_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 	knh_fieldn_t fn = knh_Token_getfnq(ctx, StmtFOREACH_name(stmt));
 	if(TERMs_isASIS(stmt, FOREACH_type)) {  /* foreach(n in itr) */
 		knh_index_t idx = knh_Asm_indexOfVariable(abr, fn);
-		if(idx == -1) { DEBUG3("type inferencing..");
+		if(idx == -1) { //DEBUG3("type inferencing..");
 			if(!TERMs_typing(ctx, stmt, FOREACH_iter, abr, ns, TYPE_Any, TWARN_)) {
 				return NULL;
 			}
@@ -2429,36 +2453,44 @@ Term *knh_StmtFOREACH_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 				}
 			}
 			knh_class_t itrcid = TERMs_getcid(stmt, FOREACH_iter);
-			type = NNTYPE_cid(knh_tClass[itrcid].p1);
+			type = NNTYPE_cid(ctx->tClass[itrcid].p1);
 			type = knh_Asm_typeinfer(ctx, abr, type, FIELDN(fn));
-			knh_Asm_declareVariable(ctx, abr, 0, type, fn, knh_tClass_defaultValueNN(ctx, CLASS_type(type)));
+			knh_Asm_declareVariable(ctx, abr, 0, type, fn, konoha_getClassDefaultValue(ctx, CLASS_type(type)));
 			if(!TERMs_typing(ctx, stmt, FOREACH_name, abr, ns, type, TCHECK_)) {
 				return NULL;
 			}
 			goto L_BLOCK;
 		}
 	}
-	else { DEBUG3("defined type .."); /* foreach(String line in itr) */
+	else { //DEBUG3("defined type .."); /* foreach(String line in itr) */
 		type = knh_Token_gettype(ctx, StmtFOREACH_type(stmt), ns, CLASS_Any);
 		type = knh_pmztype_totype(ctx, type, DP(abr)->this_cid);
 		knh_index_t idx = knh_Asm_indexOfVariable(abr, fn);
-		if(idx == -1) { DEBUG3("defining new type ..");
+		if(idx == -1) { //DEBUG3("defining new type ..");
 			knh_Asm_declareVariable(ctx, abr, /*flag*/0, type, fn, KNH_NULL);
 		}
 	}
-	DBG2_P("type=%s%s", TYPEQN(type));
+	//DBG2_P("type=%s%s", TYPEQN(type));
 	if(!TERMs_typing(ctx, stmt, FOREACH_name, abr, ns, type, TWARN_)) {
 		return NULL;
 	}
 	knh_class_t itrcid = knh_class_Iterator(ctx, TERMs_getcid(stmt, FOREACH_name));
-	if(!TERMs_typing(ctx, stmt, FOREACH_iter, abr, ns, NNTYPE_cid(itrcid), TWARN_)) {
-		return NULL;
+	if(!TERMs_typing(ctx, stmt, FOREACH_iter, abr, ns, NNTYPE_cid(itrcid), TITERCONV_)) {
+		//DBG2_P("No mapper, using opItr");
+		Stmt *stmt_call = new_Stmt(ctx, 0, STT_CALL);
+		Token *tk = new_TokenMN(ctx, FL(stmt), METHODN_opItr);
+		knh_Stmt_add(ctx, stmt_call, UP(tk));
+		knh_Stmt_add(ctx, stmt_call, StmtFOREACH_iter(stmt));
+		KNH_SETv(ctx, DP(stmt)->stmts[FOREACH_iter], stmt_call);
+		if(!TERMs_typing(ctx, stmt, FOREACH_iter, abr, ns, NNTYPE_Iterator, TCHECK_)) {
+			return NULL;
+		}
 	}
 
 	L_BLOCK:;
 	int sfpidx = knh_Asm_declareVariable(ctx, abr, 0, TYPE_Any,
 			knh_tName_local(ctx, "__S%d__", DP(abr)->level), KNH_NULL);
-	DBG2_P("sfpidx=%d", sfpidx);
+	//DBG2_P("sfpidx=%d", sfpidx);
 	if(sfpidx == -1) return NULL;
 	Token *tkitr = new_TokenNULL(ctx, FL(stmt), itrcid);
 	knh_Token_toSTACK(ctx, tkitr, sfpidx, itrcid);
@@ -2494,7 +2526,7 @@ Term *knh_StmtTRY_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 		while(SP(instmt)->stt == STT_CATCH) {
 			knh_fieldn_t fn = knh_Token_getfnq(ctx, DP(instmt)->tokens[1]);
 			if(fn == FIELDN_NONAME) {
-				knh_Asm_perror(ctx, abr, KMSG_ESYNTAX, knh_Token_tochar(DP(instmt)->tokens[1]));
+				knh_Asm_perror(ctx, abr, KMSG_ESYNTAX, sToken(DP(instmt)->tokens[1]));
 				return NULL;
 			}
 			knh_Asm_declareVariable(ctx, abr, 0, TYPE_Exception, fn, KNH_NULL);
@@ -2550,7 +2582,7 @@ Term *knh_StmtRETURN_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 			DP(stmt)->size = 0;
 		}
 	}
-	else if(knh_Method_isNew(DP(abr)->mtd)) {
+	else if(knh_Method_isNew(ctx, DP(abr)->mtd)) {
 		if(DP(stmt)->size > 0) {
 			knh_Asm_perror(ctx, abr, KMSG_IGRETURN, NULL);
 			DP(stmt)->size = 1;
@@ -2627,7 +2659,7 @@ knh_class_t knh_StmtMETHOD_class(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns,
 		}
 	}
 	{
-		knh_bytes_t name = knh_Token_tobytes(tk);
+		knh_bytes_t name = knh_Token_tobytes(ctx, tk);
 		if(SP(tk)->tt == TT_CMETHODN) {
 			knh_index_t idx = knh_bytes_rindex(name, '.');
 			KNH_ASSERT(idx != 1);
@@ -2665,7 +2697,7 @@ knh_methodn_t knh_StmtMETHOD_name(Ctx *ctx, Stmt *stmt, Asm *abr, int level)
 		}
 	}
 	else {
-		knh_bytes_t name = knh_Token_tobytes(tk);
+		knh_bytes_t name = knh_Token_tobytes(ctx, tk);
 		knh_index_t idx = knh_bytes_rindex(name, '.');
 		if(idx != -1) {
 			name = knh_bytes_last(name, idx);
@@ -2721,26 +2753,26 @@ int knh_Stmt_initParams(Ctx *ctx, Stmt *pstmt, Asm *abr, NameSpace *ns, int leve
 /* ------------------------------------------------------------------------ */
 
 static
-MethodField *knh_tMethodField_find(Ctx *ctx, Asm *abr, knh_type_t rztype, int psize)
+MethodField *knh_tMethodField_find(Ctx *ctx, Asm *abr, knh_type_t rztype, size_t mfsize)
 {
 	int i, n = 0, size = knh_tMethodField_size;
 	L_CONTINUE:;
 	while(n < size) {
-		MethodField *mf = knh_tMethodField(n);
-		if(psize != DP(mf)->size) {
+		MethodField *mf = knh_tMethodField(ctx, n);
+		if(mfsize != mf->size) {
 			n++;
 			continue;
 		}
-		if(DP(mf)->params[0].type != rztype) {
+		if(mf->params[0].type != rztype) {
 			n++;
 			continue;
 		}
-		for(i = 1; i < psize; i++) {
+		for(i = 1; i < mfsize; i++) {
 			knh_type_t vtype = DP(abr)->vars[i].type;
 			if(IS_NOTNULL(DP(abr)->vars[i].value)) {
 				vtype = CLASS_type(vtype);
 			}
-			if(DP(mf)->params[i].type != vtype) {
+			if(mf->params[i].type != vtype) {
 				n++;
 				goto L_CONTINUE;
 			}
@@ -2771,13 +2803,13 @@ int knh_Stmt_checkLastReturn(Ctx *ctx, Stmt *stmt, Method *mtd)
 
 	/* Generate default return statement */
 	Stmt *rstmt = new_Stmt(ctx, 0, STT_RETURN);
-	if(knh_Method_isNew(mtd) || knh_Method_rtype(mtd) == TYPE_void) {
+	if(knh_Method_isNew(ctx, mtd) || knh_Method_rztype(mtd) == TYPE_void) {
 		//last_stmt = knh_StmtNULL_tail_append(ctx, last_stmt, rstmt);
 	}
 	else { /* return default value */
 		knh_perror(ctx, SP(last_stmt)->fileid, SP(last_stmt)->line, KMSG_NORETURNVALUE, NULL);
 		knh_StmtRETURN_addValue(ctx, rstmt,
-				knh_pmztype_totype(ctx, knh_Method_rtype(mtd), DP(mtd)->cid));
+				knh_pmztype_totype(ctx, knh_Method_rztype(mtd), DP(mtd)->cid));
 	}
 	last_stmt = knh_StmtNULL_tail_append(ctx, last_stmt, rstmt);
 	return 1;
@@ -2806,7 +2838,7 @@ Term * knh_StmtMETHOD_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 	Method *mtd = knh_Class_getMethod(ctx, mtd_cid, mn);
 
 	/* Check the type of Constructor */
-	if(knh_methodn_isNew(mn) &&
+	if(knh_methodn_isNew(ctx, mn) &&
 			(mtd_cid != CLASS_type(rztype) && mtd_cid != CLASS_This)) {
 		knh_Asm_perror(ctx, abr, KMSG_DIFFNEWTYPE, NULL);
 		knh_Token_perrata(ctx, StmtMETHOD_rtype(stmt), CLASSN(mtd_cid));
@@ -2814,21 +2846,21 @@ Term * knh_StmtMETHOD_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 	}
 
 	/* New method, and constructors are always new */
-	if(IS_NULL(mtd) || knh_methodn_isNew(mn)) {
-		int i, psize = DP(abr)->vars_size;
-		MethodField *mf = knh_tMethodField_find(ctx, abr, rztype, psize);
+	if(IS_NULL(mtd) || knh_methodn_isNew(ctx, mn)) {
+		size_t i, mfsize = DP(abr)->vars_size;
+		MethodField *mf = knh_tMethodField_find(ctx, abr, rztype, mfsize);
 		if(mf == NULL) {
-			mf = new_MethodField(ctx, flag, psize);
-			DP(mf)->params[0].type = rztype;
-			DP(mf)->params[0].fn   = FIELDN_return;
-			for(i = 1; i < psize; i++) {
+			mf = new_MethodField(ctx, mfsize);
+			mf->params[0].type = rztype;
+			mf->params[0].fn   = FIELDN_return;
+			for(i = 1; i < mfsize; i++) {
 				if(IS_NOTNULL(DP(abr)->vars[i].value)) {
-					DP(mf)->params[i].type = CLASS_type(DP(abr)->vars[i].type);
+					mf->params[i].type = CLASS_type(DP(abr)->vars[i].type);
 				}
 				else {
-					DP(mf)->params[i].type = DP(abr)->vars[i].type;
+					mf->params[i].type = DP(abr)->vars[i].type;
 				}
-				DP(mf)->params[i].fn = DP(abr)->vars[i].fn;
+				mf->params[i].fn = DP(abr)->vars[i].fn;
 			}
 			knh_tMethodField_add(ctx, mf);
 		}
@@ -2837,7 +2869,7 @@ Term * knh_StmtMETHOD_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 		knh_Class_addMethod(ctx, mtd_cid, mtd);
 	}
 	else {
-		int i, psize = DP(abr)->vars_size;
+		size_t i, mfsize = DP(abr)->vars_size;
 		MethodField *mf = DP(mtd)->mf;
 		//DEBUG3("(mtd)->cid=%s, mtd_cid=%s", CLASSN(DP(mtd)->cid), CLASSN(mtd_cid));
 		if(DP(mtd)->cid != mtd_cid) { /* Overriding */
@@ -2850,23 +2882,23 @@ Term * knh_StmtMETHOD_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 			knh_Class_addMethod(ctx, mtd_cid, mtd);
 		}
 
-		if(!knh_StmtMETA_isOverride(stmt) &&
+		if(!knh_StmtMETA_isOverride(ctx, stmt) &&
 				!knh_Method_isAbstract(mtd) && DP(mtd)->mn != METHODN_main) {
 			/* Konoha: To override, you need @Override */
 			knh_Asm_perrorMTD(ctx, abr, KMSG_AOVERRIDE, mtd_cid, mn);
 			return NULL;
 		}
 
-		if(psize != DP(mf)->size) {
+		if(mfsize != mf->size) {
 			pe = KMSG_DIFFPSIZE; goto L_PERROR;
 		}
 
-		if(DP(mf)->params[0].type != rztype) {
+		if(mf->params[0].type != rztype) {
 			pe = KMSG_DIFFRTYPE; goto L_PERROR;
 		}
 
-		for(i = 1; i < psize; i++) {
-			knh_type_t ptype = DP(mf)->params[i].type;
+		for(i = 1; i < mfsize; i++) {
+			knh_type_t ptype = mf->params[i].type;
 			knh_type_t vtype = DP(abr)->vars[i].type;
 			if(IS_NOTNULL(DP(abr)->vars[i].value)) {
 				vtype = CLASS_type(vtype);
@@ -2929,9 +2961,9 @@ void knh_Asm_initClassField(Ctx *ctx, Asm *abr, knh_class_t cid)
 static
 void knh_Asm_declareClassField(Ctx *ctx, Asm *abr, knh_class_t cid)
 {
-	knh_tClass_t *TC = (knh_tClass_t*)(&knh_tClass[cid]);
+	knh_tClass_t *TC = (knh_tClass_t*)(&ctx->tClass[cid]);
 	DEBUG3_ASSERT(IS_ClassStruct(TC->cstruct));
-	DEBUG3_ASSERT(DP(TC->cstruct)->fields == NULL);
+	DEBUG3_ASSERT((TC->cstruct)->fields == NULL);
 	{
 		int i, fsize = DP(abr)->vars_size;
 		knh_cfield_t *cf = (knh_cfield_t*)KNH_MALLOC(ctx, sizeof(knh_cfield_t) * fsize);
@@ -2942,9 +2974,9 @@ void knh_Asm_declareClassField(Ctx *ctx, Asm *abr, knh_class_t cid)
 			cf[i].fn = DP(abr)->vars[i].fn;
 			KNH_INITv(cf[i].value, DP(abr)->vars[i].value);
 		}
-		DP(TC->cstruct)->fields =cf;
-		DP(TC->cstruct)->fsize = fsize;
-		DP(TC->cstruct)->sid = BSIZE_TOSID(fsize);
+		(TC->cstruct)->fields =cf;
+		(TC->cstruct)->fsize = fsize;
+//		(TC->cstruct)->sid = BSIZE_TOSID(fsize);
 		TC->sid = BSIZE_TOSID(fsize);
 		TC->bsize = fsize + TC->offset;
 		TC->size = sizeof(Object*) * TC->bsize;
