@@ -57,17 +57,15 @@ static size_t cache_miss = 0;
 /* ======================================================================== */
 /* [tmapmap] */
 
-void KNH_TMAPPER(Ctx *ctx, knh_flag_t flag, knh_type_t stype, knh_type_t ttype, knh_fmapper fmap, Object *mapdata)
+void konoha_addMapper(Ctx *ctx, knh_flag_t flag, knh_type_t stype, knh_type_t ttype, knh_fmapper fmap, Object *mapdata)
 {
-	knh_class_t cid = TYPE_UNMASK(stype);
+	knh_class_t cid = CLASS_type(stype);
 	KNH_ASSERT_cid(cid);
 	knh_tClass_readyClassMap(ctx, cid);
-	knh_ClassMap_add(ctx, knh_tClass[cid].cmap,
-		new_Mapper(ctx, flag, TYPE_UNMASK(stype), TYPE_UNMASK(ttype), fmap, mapdata));
+	knh_ClassMap_add(ctx, ctx->tClass[cid].cmap,
+		new_Mapper(ctx, flag, CLASS_type(stype), CLASS_type(ttype), fmap, mapdata));
 }
 
-#define _KNH_TMAPMAP  KNH_TMAPPER
-
 /* ======================================================================== */
 /* [tcache] */
 
@@ -163,14 +161,14 @@ Mapper *knh_tMapper_find_(Ctx *ctx, knh_class_t scid, knh_class_t tcid, int isge
 	Mapper *mpr = knh_tmapper_getcache(ctx, scid, tcid);
 	if(IS_NOTNULL(mpr)) { return mpr; }
 
-	if(scid == tcid || scid == CLASS_Nue || knh_class_instanceof(scid, tcid)) {  /* default */
+	if(scid == tcid || scid == CLASS_Nue || knh_class_instanceof(ctx, scid, tcid)) {  /* default */
 		return knh_tmapper_setcache(ctx, scid, tcid, new_Mapper__asis(ctx, scid, tcid));
 	}
 
 	knh_class_t sbcid = scid;
 
 	while(1) {
-		ClassMap *cmap = knh_tClass[sbcid].cmap;
+		ClassMap *cmap = ctx->tClass[sbcid].cmap;
 		int i;
 		DBG2_P("scid=%d,%s", sbcid, CLASSN(sbcid));
 		DBG2_(knh_ClassMap__dump(ctx, cmap, KNH_STDOUT, (String*)KNH_NULL));
@@ -180,10 +178,10 @@ Mapper *knh_tMapper_find_(Ctx *ctx, knh_class_t scid, knh_class_t tcid, int isge
 		}
 		for(i = 0; i < DP(cmap)->size; i++) {
 			mpr = DP(cmap)->maplist[i];
-			if(knh_class_instanceof(DP(mpr)->tcid, tcid)) return knh_tmapper_setcache(ctx, scid, tcid, mpr);
+			if(knh_class_instanceof(ctx, DP(mpr)->tcid, tcid)) return knh_tmapper_setcache(ctx, scid, tcid, mpr);
 		}
 		for(i = 0; i < DP(cmap)->size; i++) {
-			ClassMap *cmap2 = knh_tClass[DP(DP(cmap)->maplist[i])->tcid].cmap;
+			ClassMap *cmap2 = ctx->tClass[DP(DP(cmap)->maplist[i])->tcid].cmap;
 			int j;
 			KNH_ASSERT_cid(DP(DP(cmap)->maplist[i])->tcid);
 			for(j = 0; j < DP(cmap2)->size; j++) {
@@ -194,28 +192,28 @@ Mapper *knh_tMapper_find_(Ctx *ctx, knh_class_t scid, knh_class_t tcid, int isge
 			}
 			for(j = 0; j < DP(cmap2)->size; j++) {
 				mpr = DP(cmap2)->maplist[j];
-				if(knh_class_instanceof(DP(mpr)->tcid, tcid)) {
+				if(knh_class_instanceof(ctx, DP(mpr)->tcid, tcid)) {
 					return knh_tmapper_setcache(ctx, scid, tcid, new_MapMap(ctx, DP(cmap)->maplist[i], mpr));
 				}
 			}
 		}
 		if(sbcid == CLASS_Object) break;
-		if(knh_tClass_isPMZ(sbcid) && knh_tClass[sbcid].bcid != sbcid) {
-			sbcid = knh_tClass[sbcid].bcid;
+		if(knh_tClass_isPMZ(sbcid) && ctx->tClass[sbcid].bcid != sbcid) {
+			sbcid = ctx->tClass[sbcid].bcid;
 		}
 		else {
-			sbcid = knh_tClass[sbcid].supcid;
+			sbcid = ctx->tClass[sbcid].supcid;
 		}
 	}
 
 	/* GENERATIVE PART */
-	if(knh_tClass[scid].p1 != CLASS_Nue && knh_tClass[scid].p1 == knh_tClass[tcid].p1) {
+	if(ctx->tClass[scid].p1 != CLASS_Nue && ctx->tClass[scid].p1 == ctx->tClass[tcid].p1) {
 		/* C.. ==> C[] */
-		if(knh_tClass[scid].bcid == CLASS_Iterator && knh_tClass[tcid].bcid == CLASS_Array) {
+		if(ctx->tClass[scid].bcid == CLASS_Iterator && ctx->tClass[tcid].bcid == CLASS_Array) {
 			return knh_tmapper_setcache(ctx, scid, tcid, knh_tMapper_newIteratorArray(ctx, scid, tcid));
 		}
 		/* C[] ==> C.. */
-		if(knh_tClass[scid].bcid == CLASS_Array && knh_tClass[tcid].bcid == CLASS_Iterator) {
+		if(ctx->tClass[scid].bcid == CLASS_Array && ctx->tClass[tcid].bcid == CLASS_Iterator) {
 			return knh_tmapper_setcache(ctx, scid, tcid, knh_tMapper_newArrayIterator(ctx, scid, tcid));
 		}
 	}
