@@ -36,11 +36,11 @@ extern "C" {
 /* ======================================================================== */
 /* [global] */
 
-#define knh_homeDir                DP(knh_rootSystem)->homeDir
-#define knh_sysnsDictMap           DP(knh_rootSystem)->sysnsDictMap
-#define knh_funcDictSet            DP(knh_rootSystem)->funcDictSet
-#define knh_drvapiDictSet          DP(knh_rootSystem)->drvapiDictSet
-#define knh_makespecDictSet        DP(knh_rootSystem)->makespecDictSet
+#define knh_homeDir                DP(ctx->sys)->homeDir
+#define knh_sysnsDictMap           DP(ctx->sys)->sysnsDictMap
+#define knh_funcDictSet            DP(ctx->sys)->funcDictSet
+#define knh_drvapiDictSet          DP(ctx->sys)->drvapiDictSet
+#define knh_makespecDictSet        DP(ctx->sys)->makespecDictSet
 
 /* ======================================================================== */
 /* [SLOCK] */
@@ -64,7 +64,7 @@ void KNH_TDRVAPI(Ctx *ctx, char *alias, knh_drvapi_t* p)
 		knh_snprintf(bufn, sizeof(bufn), "%s:%02d", alias, p->type);
 	}
 	KNH_SLOCK(LOCK_FUNC);
-	knh_DictSet_set(ctx, DP(knh_rootSystem)->drvapiDictSet, new_String0(ctx, B(bufn), NULL), (knh_uint_t)p);
+	knh_DictSet_set(ctx, DP(ctx->sys)->drvapiDictSet, new_String(ctx, B(bufn), NULL), (knh_uint_t)p);
 	KNH_SUNLOCK(LOCK_FUNC);
 }
 
@@ -76,7 +76,7 @@ knh_drvapi_t *knh_System_getDRVAPI(Ctx *ctx, int type, knh_bytes_t name)
 	knh_format_bytes(bufn2, sizeof(bufn2), name);
 	knh_snprintf(bufn, sizeof(bufn), "%s:%02d", bufn2, type);
 	KNH_SLOCK(LOCK_FUNC);
-	knh_drvapi_t *p = (knh_drvapi_t*)knh_DictSet_get__b(DP(knh_rootSystem)->drvapiDictSet, B(bufn));
+	knh_drvapi_t *p = (knh_drvapi_t*)knh_DictSet_get__b(DP(ctx->sys)->drvapiDictSet, B(bufn));
 	KNH_SUNLOCK(LOCK_FUNC);
 	return p;
 }
@@ -84,14 +84,6 @@ knh_drvapi_t *knh_System_getDRVAPI(Ctx *ctx, int type, knh_bytes_t name)
 /* ======================================================================== */
 /* [func] */
 
-void KNH_TSTATICLIB__T(Ctx *ctx, char *text, void *func)
-{
-	if(func != NULL) {
-		KNH_SLOCK(LOCK_FUNC);
-		knh_DictSet_set(ctx, knh_funcDictSet, new_String__T(ctx, text), (knh_uint_t)func);
-		KNH_SUNLOCK(LOCK_FUNC);
-	}
-}
 
 /* ======================================================================== */
 /* [namespace] */
@@ -147,7 +139,7 @@ METHOD knh__System_hasLibrary(Ctx *ctx, knh_sfp_t *sfp)
 static
 NameSpace *knh_System_getNameSpace(Ctx *ctx, knh_bytes_t pkgname, int isnew)
 {
-	NameSpace *ns = (NameSpace*)knh_DictMap_get__b( knh_sysnsDictMap, pkgname);
+	NameSpace *ns = (NameSpace*)knh_DictMap_get__b(ctx,  knh_sysnsDictMap, pkgname);
 	if(IS_NULL(ns) && isnew) {
 		KNH_SLOCK(LOCK_PKGMGR);
 		String *nsname = new_String(ctx, pkgname, NULL);
@@ -179,7 +171,7 @@ char *knh_format_packagedir(char *buf, size_t bufsiz, char *homedir)
 
 /* ------------------------------------------------------------------------ */
 
-char *knh_format_packagepath(char *buf, size_t bufsiz, char *pre, char *name, char *ext)
+char *knh_format_packagepath(Ctx *ctx, char *buf, size_t bufsiz, char *pre, char *name, char *ext)
 {
 	char *p = knh_getenv("KONOHA_PACKAGE");
 	if(p != NULL) {
@@ -198,8 +190,7 @@ char *knh_format_packagepath(char *buf, size_t bufsiz, char *pre, char *name, ch
 
 /* ------------------------------------------------------------------------ */
 
-static
-int knh_System_loadPackage(Ctx *ctx, knh_bytes_t pkgname)
+static int knh_System_loadPackage(Ctx *ctx, knh_bytes_t pkgname)
 {
 	int res = 0;
 	char bufp[CLASSNAME_BUFSIZ], buff[FILENAME_BUFSIZ];
@@ -221,7 +212,7 @@ int knh_System_loadPackage(Ctx *ctx, knh_bytes_t pkgname)
 	}
 #else/*KONOHA_MONOLITHIC*/
 	void *dlhdr = NULL;
-	knh_format_packagepath(buff, sizeof(buff), "lib", bufp, KONOHA_OS_DLLEXT);
+	knh_format_packagepath(ctx, buff, sizeof(buff), "lib", bufp, KONOHA_OS_DLLEXT);
 	DBG2_P("loading package .. '%s'", buff);
 	dlhdr =knh_dlopen(ctx, buff, KNH_RTLD_LAZY);
 	if (dlhdr != NULL) {
@@ -248,7 +239,7 @@ int knh_System_loadPackage(Ctx *ctx, knh_bytes_t pkgname)
 		DBG_P("cannot open '%s'", buff);
 	}
 #endif/*KONOHA_MONOLITHIC*/
-	knh_format_packagepath(buff, sizeof(buff), "", bufp, ".k");
+	knh_format_packagepath(ctx, buff, sizeof(buff), "", bufp, ".k");
 	if(knh_isfile(ctx, B(buff))) {
 		DBG2_P("opening.. '%s'", buff);
 		InputStream *in = new_FileInputStream(ctx, B(buff));
@@ -268,7 +259,7 @@ int knh_System_loadPackage(Ctx *ctx, knh_bytes_t pkgname)
 
 int knh_System_using(Ctx *ctx, knh_bytes_t name)
 {
-	NameSpace *ns = (NameSpace*)knh_DictMap_get__b( knh_sysnsDictMap, name);
+	NameSpace *ns = (NameSpace*)knh_DictMap_get__b(ctx,  knh_sysnsDictMap, name);
 	if(IS_NULL(ns)) {
 		return knh_System_loadPackage(ctx, name);
 	}
@@ -287,7 +278,7 @@ NameSpace *knh_Runtime_getNameSpace(Ctx *ctx, knh_bytes_t name)
 		return ctx->ns;
 	}
 	else {
-		NameSpace *ns = (NameSpace*)knh_DictMap_get__b( DP(ctx->runtime)->tnameSpaceDictMap, name);
+		NameSpace *ns = (NameSpace*)knh_DictMap_get__b(ctx,  DP(ctx->sys)->tnameSpaceDictMap, name);
 		if(IS_NULL(ns)) {
 			String *nsname = NULL;
 			if(knh_System_loadPackage(ctx, name)) {
@@ -299,7 +290,7 @@ NameSpace *knh_Runtime_getNameSpace(Ctx *ctx, knh_bytes_t name)
 				nsname = new_String(ctx, name, NULL);
 				ns = new_NameSpace(ctx, knh_rootNameSpace, nsname);
 			}
-			knh_DictMap_set(ctx, DP(ctx->runtime)->tnameSpaceDictMap, nsname, UP(ns));
+			knh_DictMap_set(ctx, DP(ctx->sys)->tnameSpaceDictMap, nsname, UP(ns));
 		}
 		return ns;
 	}
