@@ -29,6 +29,11 @@
 
 #include"commons.h"
 
+#ifdef KNH_USING_READLINE
+	#include<readline/readline.h>  /* @using readline -lreadline */
+	#include<readline/history.h>
+#endif
+
 /* ************************************************************************ */
 
 #ifdef __cplusplus
@@ -36,146 +41,39 @@ extern "C" {
 #endif
 
 /* ======================================================================== */
-/* [konoha] */
+/* [option] */
 
 static int optionAdmin = 0;
 static int optionCompiling = 0;
 
 /* ----------------------------------------------------------------------- */
 
-KNHAPI(int) konoha_setopt(int argc, char **argv)
+KNHAPI(int) konoha_parseopt(konoha_t konoha, int argc, char **argv)
 {
+	KONOHA_CHECK(konoha, 1);
 	int n;
 	for(n = 1; n < argc; n++) {
 		char *t = argv[n];
 		if(t[0] != '-') return n;
 		if(t[1] == 's' && t[2] == 0) {
 			optionAdmin = 1; continue;
+			knh_Context_setAdmin(konoha.ctx, 1);
 		}
 		if(t[1] == 'c' && t[2] == 0) {
 			optionCompiling = 1; continue;
+			knh_Context_setCompiling(konoha.ctx, 1);
 		}
 	}
 	return n;
 }
 
-/* ------------------------------------------------------------------------ */
-
-KNHAPI(void) konoha_context_initOption(Ctx *ctx)
-{
-	if(optionAdmin == 1) {
-		knh_Context_setAdmin(ctx, 1);
-	}
-	if(optionCompiling == 1) {
-		knh_Context_setCompiling(ctx, 1);
-	}
-}
-
-/* ------------------------------------------------------------------------ */
-
-static int knh_systemIsInitialized = 0;
-static knh_uint_t knh_runtimeId = 0;
-
-void konoha_class_init0(Ctx *ctx);
-
-/* ------------------------------------------------------------------------ */
-
-static
-void konoha_init0()
-{
-	knh_tObject_init0();
-	knh_tContext_init0();
-	knh_tStruct_init0();
-	knh_tClass_init0();
-	knh_tExpt_init0();
-	konoha_class_init0(NULL);
-	knh_srand(0);
-}
-
-/* ------------------------------------------------------------------------ */
-
-KNHAPI(konoha_t) konoha_open(void)
-{
-	if(knh_systemIsInitialized == 0) {
-		konoha_init0();
-		knh_systemIsInitialized = 1;
-	}
-	{
-		Runtime *runtime = new_Runtime0(knh_runtimeId++);
-		konoha_t k = {KONOHA_MAGIC, runtime};
-		knh_Object_RCinc((Object*)runtime);
-		return k;
-	}
-}
-
-/* ------------------------------------------------------------------------ */
-
-static
-void konoha_traverse(Ctx *ctx, knh_ftraverse f)
-{
-	knh_tContext_traverse(ctx, f);
-	knh_tObject_traverse(ctx, f);
-	knh_tStruct_traverse(ctx, f);
-}
-
-/* ------------------------------------------------------------------------ */
-
-KNHAPI(void) konoha_close(konoha_t konoha)
-{
-	if(konoha.magic == KONOHA_MAGIC) {
-		if(!IS_Runtime(konoha.runtime)) {
-			KNH_SAYS("Maybe already closed");
-			return;
-		}
-		knh_Object_RCsweep(NULL, UP(konoha.runtime));
-		if(knh_Runtime_isAllClosed()) {
-			konoha_traverse(NULL, knh_Object_RCsweep);
-#ifdef KNH_DBGMODE
-			if(knh_stat_usedMemorySize() != 0) {
-				KNH_SAYS("MEMORY LEAK: %d bytes", (int)knh_stat_usedMemorySize());
-			}
-#endif
-		}
-	}
-	else{
-		KNH_SAYS("This is not a Konoha Scripting Engine");
-	}
-}
-
-/* ======================================================================== */
-/* [context] */
-
-KNHAPI(Ctx*) konoha_open_context(konoha_t konoha)
-{
-	if(konoha.magic == KONOHA_MAGIC && IS_Runtime(konoha.runtime)) {
-		return new_Context0(konoha.runtime, KNH_STACKSIZE);
-	}
-	else {
-		KNH_SAYS("This is not a Konoha Scripting Engine");
-	}
-	return NULL;
-}
-
-/* ------------------------------------------------------------------------ */
-
-KNHAPI(void) konoha_close_context(konoha_t konoha, Ctx *ctx)
-{
-	knh_Object_t *o = (knh_Object_t*)ctx;
-	knh_Context_clearstack(ctx);
-	if(konoha.magic == KONOHA_MAGIC && IS_Context(o)) {
-		knh_Object_RCsweep(ctx, o);
-	}
-	else {
-		KNH_SAYS("This is not a Konoha Scripting Engine");
-	}
-}
-
 /* ======================================================================== */
 /* [konoha] */
 
-KNHAPI(char*) konoha_eval(Ctx *ctx, char *script)
+KNHAPI(char*) konoha_eval(konoha_t konoha, char *script)
 {
-	if(ctx == NULL) return NULL;
+	KONOHA_CHECK(konoha, NULL);
+	Ctx *ctx = konoha.ctx;
 	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
 	knh_wbuf_t wbuf = knh_Context_wbuf(ctx);
 	knh_Bytes_write(ctx, wbuf.ba, B(script));
@@ -201,11 +99,13 @@ KNHAPI(char*) konoha_eval(Ctx *ctx, char *script)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) konoha_read(Ctx *ctx, char *fpath)
+KNHAPI(void) konoha_read(konoha_t konoha, char *fpath)
 {
+	KONOHA_CHECK_(konoha);
+	Ctx *ctx = konoha.ctx;
 	if(ctx == NULL) return;
 	if(!knh_isfile(ctx, B(fpath))) {
-		KNH_WARNING(ctx, "No such script file: %s\n", fpath);
+		KNH_WARNING(ctx, "No such file: %s\n", fpath);
 		return ;
 	}
 	InputStream *in = new_FileInputStream(ctx, B(fpath));
@@ -220,8 +120,10 @@ KNHAPI(void) konoha_read(Ctx *ctx, char *fpath)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(int) konoha_main(Ctx *ctx, int argc, char **argv)
+KNHAPI(int) konoha_main(konoha_t konoha, int argc, char **argv)
 {
+	KONOHA_CHECK(konoha, 0);
+	Ctx *ctx = konoha.ctx;
 	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
 	KNH_MOV(ctx, lsfp[0].o, new_ExceptionHandler(ctx));
 	KNH_TRY(ctx, L_CATCH, lsfp, 0);
@@ -234,7 +136,7 @@ KNHAPI(int) konoha_main(Ctx *ctx, int argc, char **argv)
 			goto L_END_TRY;
 		}
 		if(knh_Method_psize(mtd) == 1) {
-			knh_type_t ptype = knh_Method_ptype(mtd, 0);
+			knh_type_t ptype = knh_Method_pztype(mtd, 0);
 			if(ptype != ATYPE_String && ptype != NNATYPE_String) {
 				KNH_WARNING(ctx, "Type!!: cannot invoke main()");
 				goto L_END_TRY;
@@ -266,8 +168,246 @@ KNHAPI(int) konoha_main(Ctx *ctx, int argc, char **argv)
 	return 0;
 }
 
+/* ======================================================================== */
+/* [readline] */
+
+static
+char *knh_readline(char *prompt)
+{
+	char *ln = NULL;
+#if defined(KNH_USING_READLINE)
+	ln = readline(prompt);
+	if(ln != NULL && *ln) add_history(ln);
+	return ln;
+#else
+	char bufl[1024];
+	int pos = 0, ch;
+	fputs(prompt, stdout);
+	while((ch = fgetc(stdin)) != EOF) {
+		if(ch == '\r') continue;
+		if(ch == '\n') break;
+		bufl[pos] = ch;
+		pos++;
+		if(pos < sizeof(bufl)) {
+			continue;
+		}
+		else {
+			bufl[pos-1] = 0;
+		}
+	}
+	KNH_ASSERT(pos < sizeof(bufl));
+	bufl[pos] = 0;
+	ln = (char*)malloc(KNH_SIZE(pos+1));
+	knh_memcpy(ln, bufl, pos);
+	ln[pos] = 0;
+	return ln;
+#endif
+}
+
 /* ------------------------------------------------------------------------ */
 
+static
+void knh_add_history(char *line)
+{
+#if defined(KNH_USING_READLINE)
+	if(line != NULL && *line) add_history(line);
+#endif
+}
+
+/* ======================================================================== */
+/* [shell] */
+
+static
+int konoha_shell_checkline(knh_bytes_t line)
+{
+	char *ln = (char*)line.buf;
+	size_t i = 0, len = line.len;
+	int ch, quote = 0, nest =0;
+	L_NORMAL:
+	for(; i < len; i++) {
+		ch = ln[i];
+		if(ch == '{' || ch == '[' || ch == '(') nest++;
+		if(ch == '}' || ch == ']' || ch == ')') nest--;
+		if(ch == '\'' || ch == '"') {
+			quote = ch; i++;
+			goto L_QUOTE;
+		}
+	}
+	return nest;
+
+	L_QUOTE:
+	KNH_ASSERT(i > 0);
+	for(; i < len; i++) {
+		ch = ln[i];
+		if(ln[i-1] != '\\' && ch == quote) {
+			i++;
+			goto L_NORMAL;
+		}
+	}
+	return 1;
+}
+
+/* ------------------------------------------------------------------------ */
+
+KNHAPI(void) konoha_shell(konoha_t konoha)
+{
+	KONOHA_CHECK_(konoha);
+	Ctx *ctx = konoha.ctx;
+	knh_System__dump(ctx, NULL, KNH_STDOUT, (String*)KNH_NULL);
+	int linenum, linecnt = 0;
+	knh_wbuf_t cb = knh_Context_wbuf(ctx);
+	char *ln = NULL;
+	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
+	KNH_LPUSH(ctx, KNH_NULL);  // step ebp++;
+
+	START_LINE:;
+	if(ln != NULL) free(ln);
+#ifdef KNH_DBGMODE
+	ln = knh_readline("\n>>> ");
+#else
+	ln = knh_readline(">>> ");
+#endif
+	linecnt++;
+	linenum = linecnt;
+
+	while(1) {
+		int nest = 0;
+		if(ln == NULL) {
+			KNH_LOCALBACK(ctx, lsfp);
+			knh_Context_clearstack(ctx);
+			return;
+		}
+		knh_Bytes_write(ctx, cb.ba, B(ln));
+		nest = konoha_shell_checkline(knh_wbuf_tobytes(cb));
+		if(nest == 0) break;
+		if(nest < 0) {
+			knh_println(ctx, KNH_STDOUT, STEXT("(Canceled)"));
+			knh_Bytes_subclear(cb.ba, cb.pos);
+			goto START_LINE;
+		}else {
+			knh_Bytes_putc(ctx, cb.ba, '\n');
+			if(ln != NULL) free(ln);
+			ln = knh_readline("... ");
+			linecnt++;
+		}
+	}
+	if(cb.pos < knh_Bytes_size(cb.ba)) {
+		knh_bytes_t t = knh_wbuf_tobytes(cb);
+		if(ISB(t, "exit") || ISB(t, "quit") || ISB(t, "bye")) {
+			if(ln != NULL) free(ln);
+			knh_Bytes_subclear(cb.ba, cb.pos);
+			KNH_LOCALBACK(ctx, lsfp);
+			knh_Context_clearstack(ctx);
+			return;
+		}
+		if(knh_bytes_startsWith(t, STEXT("man "))) {
+			char bufc[CLASSNAME_BUFSIZ], *p = bufc;
+			knh_snprintf(bufc, sizeof(bufc), "%%man(%s)", (char*)knh_bytes_last(t, 4).buf);
+			while(*p != 0) {
+				if(*p == ';') *p = ' ';
+				p++;
+			}
+			knh_Bytes_subclear(cb.ba, cb.pos);
+			knh_Bytes_write(ctx, cb.ba, B(bufc));
+		}
+		else if(knh_bytes_startsWith(t, STEXT("dump "))) {
+			char bufc[CLASSNAME_BUFSIZ], *p = bufc;
+			knh_snprintf(bufc, sizeof(bufc), "%%dump(%s)", (char*)knh_bytes_last(t, 5).buf);
+			while(*p != 0) {
+				if(*p == ';') *p = ' ';
+				p++;
+			}
+			knh_Bytes_subclear(cb.ba, cb.pos);
+			knh_Bytes_write(ctx, cb.ba, B(bufc));
+		}
+		knh_Bytes_putc(ctx, cb.ba, ';');
+		knh_Bytes_putc(ctx, cb.ba, '\n');
+		{
+			InputStream *in = new_BytesInputStream(ctx, cb.ba, cb.pos, knh_Bytes_size(cb.ba));
+			KNH_MOV(ctx, lsfp[0].o, in);
+			DP(in)->fileid = knh_tName_getFileId(ctx, STEXT("(shell)"));
+			DP(in)->line = linenum;
+			knh_InputStream_setEncoding(ctx, in, knh_systemEncoding);
+			konohac_eval(ctx, TS_main, in);
+			knh_Bytes_subclear(cb.ba, cb.pos);
+		}
+	}
+	goto START_LINE;
+}
+
+/* ======================================================================== */
+/* @method void Script.eval(String script, String file, Int linenum) @Static */
+
+METHOD knh__Script_eval(Ctx *ctx, knh_sfp_t *sfp)
+{
+	if(IS_NOTNULL(sfp[1].s)) {
+		knh_sfp_t *lsfp = KNH_LOCAL(ctx);
+		knh_wbuf_t wbuf = knh_Context_wbuf(ctx);
+		knh_Bytes_write(ctx, wbuf.ba, knh_String_tobytes(sfp[1].s));
+		knh_Bytes_putc(ctx, wbuf.ba, ';');
+		knh_Bytes_putc(ctx, wbuf.ba, '\n');
+		InputStream *in = new_BytesInputStream(ctx, wbuf.ba, wbuf.pos, knh_Bytes_size(wbuf.ba));
+		KNH_LPUSH(ctx, in);
+		if(IS_NULL(sfp[2].o)) {
+			DP(in)->fileid = knh_tName_getFileId(ctx, STEXT("(eval)"));
+		}
+		else {
+			DP(in)->fileid = knh_tName_getFileId(ctx, knh_String_tobytes(sfp[2].s));
+		}
+		if(IS_NULL(sfp[3].o)) {
+			DP(in)->line = 0;
+		}
+		else {
+			DP(in)->fileid = (knh_fileid_t)p_int(sfp[3]);
+		}
+		konohac_eval(ctx, TS_main, in);
+		knh_Bytes_subclear(wbuf.ba, wbuf.pos);
+		KNH_LOCALBACK(ctx, lsfp);
+	}
+	KNH_RETURN_void(ctx, sfp);
+}
+
+/* ------------------------------------------------------------------------ */
+/* @method Boolean! Script.isStatement(String! script) @Static */
+
+METHOD knh__Script_isStatement(Ctx *ctx, knh_sfp_t *sfp)
+{
+	KNH_RETURN_Boolean(ctx, sfp, !(konoha_shell_checkline(knh_String_tobytes(sfp[1].s))));
+}
+
+/* ------------------------------------------------------------------------ */
+/* @method String Script.readLine(String prompt) @Static */
+
+METHOD knh__Script_readLine(Ctx *ctx, knh_sfp_t *sfp)
+{
+	char *line;
+	if(IS_NULL(sfp[1].o)) {
+		line = knh_readline("");
+	}
+	else {
+		line = knh_readline(knh_String_tochar(sfp[1].s));
+	}
+	if(line == NULL) {
+		KNH_RETURN(ctx, sfp, KNH_NULL);
+	}
+	else {
+		KNH_RETURN(ctx, sfp, new_String(ctx, B(line), NULL));
+	}
+}
+
+/* ------------------------------------------------------------------------ */
+/* @method void Script.addHistory(String! s) @Static */
+
+METHOD knh__Script_addHistory(Ctx *ctx, knh_sfp_t *sfp)
+{
+	knh_add_history(knh_String_tochar(sfp[1].s));
+	KNH_RETURN_void(ctx, sfp);
+}
+
+/* ------------------------------------------------------------------------ */
+/* ------------------------------------------------------------------------ */
+
+
 #ifdef __cplusplus
 }
 #endif
